/*
 * Copyright 2001-2013 Artima, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Normalization
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue
import scala.collection.mutable.LinkedList

class ArrayShouldContainInOrderOnlySpec extends Spec with Matchers {

  private def upperCase(value: Any): Any = 
    value match {
      case l: Array[_] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case _ => value
    }

  val upperCaseStringEquality =
    new Equality[String] {
      def areEqual(a: String, b: Any): Boolean = upperCase(a) == upperCase(b)
    }

  //ADDITIONAL//

  object `a Array` {

    val fumArray: Array[String] = Array("fum", "fum", "foe", "fie", "fie", "fie", "fee", "fee")
    val toArray: Array[String] = Array("happy", "happy", "happy", "birthday", "to", "you")

    object `when used with contain inOrderOnly (..)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should contain inOrderOnly ("fum", "foe", "fie", "fee")
        val e1 = intercept[TestFailedException] {
          fumArray should contain inOrderOnly ("fee", "fie", "foe", "fum")
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainInOrderOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumArray), "\"fee\", \"fie\", \"foe\", \"fum\""))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should contain inOrderOnly ("FUM", "FOE", "FIE", "FEE")
        intercept[TestFailedException] {
          fumArray should contain inOrderOnly ("fee", "fie", "foe", "fum")
        }
      }
      def `should use an explicitly provided Equality` {
        (fumArray should contain inOrderOnly ("FUM", "FOE", "FIE", "FEE")) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (fumArray should contain inOrderOnly ("fee", "fie", "foe", "fum")) (decided by upperCaseStringEquality)
        }
        intercept[TestFailedException] {
          fumArray should contain inOrderOnly (" FUM ", " FOE ", " FIE ", " FEE ")
        }
        (fumArray should contain inOrderOnly (" FUM ", " FOE ", " FIE ", " FEE ")) (after being lowerCased and trimmed)
      }
    }

    object `when used with (contain inOrderOnly (..))` {
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should (contain inOrderOnly ("fum", "foe", "fie", "fee"))
        val e1 = intercept[TestFailedException] {
          fumArray should (contain inOrderOnly ("fee", "fie", "foe", "fum"))
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainInOrderOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainInOrderOnlyElements", decorateToStringValue(fumArray),  "\"fee\", \"fie\", \"foe\", \"fum\""))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should (contain inOrderOnly ("FUM", "FOE", "FIE", "FEE"))
        intercept[TestFailedException] {
          fumArray should (contain inOrderOnly ("fee", "fie", "foe", "fum"))
        }
      }
      def `should use an explicitly provided Equality` {
        (fumArray should (contain inOrderOnly ("FUM", "FOE", "FIE", "FEE"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (fumArray should (contain inOrderOnly ("fee", "fie", "foe", "fum"))) (decided by upperCaseStringEquality)
        }
        intercept[TestFailedException] {
          fumArray should (contain inOrderOnly (" FUM ", " FOE ", " FIE ", " FEE "))
        }
        (fumArray should (contain inOrderOnly (" FUM ", " FOE ", " FIE ", " FEE "))) (after being lowerCased and trimmed)
      }
    }

    object `when used with not contain inOrderOnly (..)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        toArray should not contain inOrderOnly ("you", "to", "birthday", "happy")
        val e1 = intercept[TestFailedException] {
          toArray should not contain inOrderOnly ("happy", "birthday", "to", "you")
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainInOrderOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedInOrderOnlyElements", decorateToStringValue(toArray), "\"happy\", \"birthday\", \"to\", \"you\""))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        toArray should not contain inOrderOnly ("YOU", "TO", "BIRTHDAY", "HAPPY")
        intercept[TestFailedException] {
          toArray should not contain inOrderOnly ("HAPPY", "BIRTHDAY", "TO", "YOU")
        }
      }
      def `should use an explicitly provided Equality` {
        (toArray should not contain inOrderOnly ("YOU", "TO", "BIRTHDAY", "HAPPY")) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (toArray should not contain inOrderOnly ("HAPPY", "BIRTHDAY", "TO", "YOU")) (decided by upperCaseStringEquality)
        }
        toArray should not contain inOrderOnly (" HAPPY ", " BIRTHDAY ", " TO ", " YOU ")
        intercept[TestFailedException] {
          (toArray should not contain inOrderOnly (" HAPPY ", " BIRTHDAY ", " TO ", " YOU ")) (after being lowerCased and trimmed)
        }
      }
    }

    object `when used with (not contain inOrderOnly (..))` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        toArray should (not contain inOrderOnly ("you", "to", "birthday", "happy"))
        val e1 = intercept[TestFailedException] {
          toArray should (not contain inOrderOnly ("happy", "birthday", "to", "you"))
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainInOrderOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedInOrderOnlyElements", decorateToStringValue(toArray), "\"happy\", \"birthday\", \"to\", \"you\""))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        toArray should (not contain inOrderOnly ("YOU", "TO", "BIRTHDAY", "HAPPY"))
        intercept[TestFailedException] {
          toArray should (not contain inOrderOnly ("HAPPY", "BIRTHDAY", "TO", "YOU"))
        }
      }
      def `should use an explicitly provided Equality` {
        (toArray should (not contain inOrderOnly ("YOU", "TO", "BIRTHDAY", "HAPPY"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (toArray should (not contain inOrderOnly ("HAPPY", "BIRTHDAY", "TO", "YOU"))) (decided by upperCaseStringEquality)
        }
        toArray should (not contain inOrderOnly (" HAPPY ", " BIRTHDAY ", " TO ", " YOU "))
        intercept[TestFailedException] {
          (toArray should (not contain inOrderOnly (" HAPPY ", " BIRTHDAY ", " TO ", " YOU "))) (after being lowerCased and trimmed)
        }
      }
    }
  }

  object `a col of Arrays` {

    val list1s: Vector[Array[Int]] = Vector( Array(1, 2, 2, 3), Array(1, 1, 2, 3, 3, 3), Array(1, 2, 3))
    val lists: Vector[Array[Int]] = Vector( Array(1, 2, 2, 3, 3, 3), Array(1, 1, 1, 2, 3), Array(2, 2, 3, 4))
    val listsArray: Vector[Array[Int]] = Vector( Array(1, 1, 1, 2, 2, 2, 3, 3, 3), Array(1, 2, 2, 3), Array())
    val hiArrays: Vector[Array[String]] = Vector( Array("hi", "hi", "he"), Array("hi", "he", "he", "he"), Array("hi", "he"))
    val toArrays: Vector[Array[String]] = Vector( Array("to", "you"), Array("to", "you"), Array("to", "you"))

    object `when used with contain inOrderOnly (..)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should contain inOrderOnly (1, 2, 3)
        atLeast (2, lists) should contain inOrderOnly (1, 2, 3)
        atMost (2, lists) should contain inOrderOnly (1, 2, 3)
        no (lists) should contain inOrderOnly (3, 4, 5)

        val e1 = intercept[TestFailedException] {
          all (lists) should contain inOrderOnly (1, 2, 3)
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainInOrderOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(lists(2)) + " did not contain only " + "(1, 2, 3)" + " in order (ArrayShouldContainInOrderOnlySpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))

        val e3 = intercept[TestFailedException] {
          all (listsArray) should contain inOrderOnly (1, 2, 3)
        }
        e3.failedCodeFileName.get should be ("ArrayShouldContainInOrderOnlySpec.scala")
        e3.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e3.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(Array()) + " did not contain only " + "(1, 2, 3)" + " in order (ArrayShouldContainInOrderOnlySpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(listsArray)))
      }

      def `should use the implicit Equality in scope` {
        all (hiArrays) should contain inOrderOnly ("hi", "he")
        intercept[TestFailedException] {
          all (hiArrays) should contain inOrderOnly ("hi", "ho")
        }
        implicit val ise = upperCaseStringEquality
        all (hiArrays) should contain inOrderOnly ("HI", "HE")
        intercept[TestFailedException] {
          all (hiArrays) should contain inOrderOnly ("HI", "HO")
        }
      }
      def `should use an explicitly provided Equality` {
        (all (hiArrays) should contain inOrderOnly ("HI", "HE")) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (hiArrays) should contain inOrderOnly ("HI", "HO")) (decided by upperCaseStringEquality)
        }
        implicit val ise = upperCaseStringEquality
        (all (hiArrays) should contain inOrderOnly ("hi", "he")) (decided by defaultEquality[String])
        intercept[TestFailedException] {
          (all (hiArrays) should contain inOrderOnly ("hi", "ho")) (decided by defaultEquality[String])
        }
      }
    }

    object `when used with (contain inOrderOnly (..))` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain inOrderOnly (1, 2, 3))
        atLeast (2, lists) should (contain inOrderOnly (1, 2, 3))
        atMost (2, lists) should (contain inOrderOnly (1, 2, 3))
        no (lists) should (contain inOrderOnly (3, 4, 5))
        no (listsArray) should (contain inOrderOnly (3, 4, 5))

        val e1 = intercept[TestFailedException] {
          all (lists) should (contain inOrderOnly (1, 2, 3))
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainInOrderOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(lists(2)) + " did not contain only " + "(1, 2, 3)" + " in order (ArrayShouldContainInOrderOnlySpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))

        val e4 = intercept[TestFailedException] {
          all (listsArray) should (contain inOrderOnly (1, 2, 3))
        }
        e4.failedCodeFileName.get should be ("ArrayShouldContainInOrderOnlySpec.scala")
        e4.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e4.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(Array()) + " did not contain only " + "(1, 2, 3)" + " in order (ArrayShouldContainInOrderOnlySpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(listsArray)))
      }

      def `should use the implicit Equality in scope` {
        all (hiArrays) should (contain inOrderOnly ("hi", "he"))
        intercept[TestFailedException] {
          all (hiArrays) should (contain inOrderOnly ("he", "hi"))
        }
        implicit val ise = upperCaseStringEquality
        all (hiArrays) should (contain inOrderOnly ("HI", "HE"))
        intercept[TestFailedException] {
          all (hiArrays) should (contain inOrderOnly ("HI", "HO"))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (hiArrays) should (contain inOrderOnly ("HI", "HE"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (hiArrays) should (contain inOrderOnly ("HI", "HO"))) (decided by upperCaseStringEquality)
        }
        implicit val ise = upperCaseStringEquality
        (all (hiArrays) should (contain inOrderOnly ("hi", "he"))) (decided by defaultEquality[String])
        intercept[TestFailedException] {
          (all (hiArrays) should (contain inOrderOnly ("he", "hi"))) (decided by defaultEquality[String])
        }
      }
    }

    object `when used with not contain inOrderOnly (..)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (toArrays) should not contain inOrderOnly ("you", "to")
        val e1 = intercept[TestFailedException] {
          all (toArrays) should not contain inOrderOnly ("to", "you")
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainInOrderOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(toArrays(0)) + " contained only " + "(\"to\", \"you\")" +  " in order (ArrayShouldContainInOrderOnlySpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(toArrays)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        all (toArrays) should not contain inOrderOnly ("YOU", "TO")
        intercept[TestFailedException] {
          all (toArrays) should not contain inOrderOnly ("TO", "YOU")
        }
      }
      def `should use an explicitly provided Equality` {
        (all (toArrays) should not contain inOrderOnly ("YOU", "TO")) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (toArrays) should not contain inOrderOnly ("TO", "YOU")) (decided by upperCaseStringEquality)
        }
        all (toArrays) should not contain inOrderOnly (" TO ", " YOU ")
        intercept[TestFailedException] {
          (all (toArrays) should not contain inOrderOnly (" TO ", " YOU ")) (after being lowerCased and trimmed)
        }
      }
    }

    object `when used with (not contain inOrderOnly (..))` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (toArrays) should (not contain inOrderOnly ("you", "to"))
        val e1 = intercept[TestFailedException] {
          all (toArrays) should (not contain inOrderOnly ("to", "you"))
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainInOrderOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(toArrays(0)) + " contained only " + "(\"to\", \"you\")" + " in order (ArrayShouldContainInOrderOnlySpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(toArrays)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        all (toArrays) should (not contain inOrderOnly ("YOU", "TO"))
        intercept[TestFailedException] {
          all (toArrays) should (not contain inOrderOnly ("TO", "YOU"))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (toArrays) should (not contain inOrderOnly ("YOU", "TO"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (toArrays) should (not contain inOrderOnly ("TO", "YOU"))) (decided by upperCaseStringEquality)
        }
        all (toArrays) should (not contain inOrderOnly (" TO ", " YOU "))
        intercept[TestFailedException] {
          (all (toArrays) should (not contain inOrderOnly (" TO ", " YOU "))) (after being lowerCased and trimmed)
        }
      }
    }
  }
}
