/*
 * Copyright 2001-2013 Artima, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Normalization
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class ArrayShouldContainOnlySpec extends Spec with Matchers {
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: Array[_] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case _ => value
    }
  
  val upperCaseStringEquality =
    new Equality[String] {
      def areEqual(a: String, b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  //ADDITIONAL//

  object `a Array` {

    val fumArray: Array[String] = Array("fum", "foe", "fie", "fee")
    val toArray: Array[String] = Array("you", "to", "birthday", "happy")

    object `when used with contain only (..)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumArray should contain only ("fee", "fie", "foe", "fum")
        val e1 = intercept[TestFailedException] {
          fumArray should contain only ("happy", "birthday", "to", "you")
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainOnlyElements", decorateToStringValue(fumArray), "\"happy\", \"birthday\", \"to\", \"you\""))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should contain only ("FEE", "FIE", "FOE", "FUM")
        intercept[TestFailedException] {
          fumArray should contain only ("fee", "fie", "foe")
        }
      }
      def `should use an explicitly provided Equality` {
        (fumArray should contain only ("FEE", "FIE", "FOE", "FUM")) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (fumArray should contain only ("fee", "fie", "foe")) (decided by upperCaseStringEquality)
        }
        intercept[TestFailedException] {
          fumArray should contain only (" FEE ", " FIE ", " FOE ", " FUM ")
        }
        (fumArray should contain only (" FEE ", " FIE ", " FOE ", " FUM ")) (after being lowerCased and trimmed)
      }
    }

    object `when used with (contain only (..))` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {

        fumArray should (contain only ("fee", "fie", "foe", "fum"))
        val e1 = intercept[TestFailedException] {
          fumArray should (contain only ("happy", "birthday", "to", "you"))
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainOnlyElements", decorateToStringValue(fumArray), "\"happy\", \"birthday\", \"to\", \"you\""))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumArray should (contain only ("FEE", "FIE", "FOE", "FUM"))
        intercept[TestFailedException] {
          fumArray should (contain only ("fee", "fie", "foe"))
        }
      }
      def `should use an explicitly provided Equality` {
        (fumArray should (contain only ("FEE", "FIE", "FOE", "FUM"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (fumArray should (contain only ("fee", "fie", "foe"))) (decided by upperCaseStringEquality)
        }
        intercept[TestFailedException] {
          fumArray should (contain only (" FEE ", " FIE ", " FOE ", " FUM "))
        }
        (fumArray should (contain only (" FEE ", " FIE ", " FOE ", " FUM "))) (after being lowerCased and trimmed)
      }
    }

    object `when used with not contain only (..)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        toArray should not contain only ("fee", "fie", "foe", "fum")
        val e1 = intercept[TestFailedException] {
          toArray should not contain only ("happy", "birthday", "to", "you")
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedOnlyElements", decorateToStringValue(toArray), "\"happy\", \"birthday\", \"to\", \"you\""))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        toArray should not contain only ("happy", "birthday", "to")
        intercept[TestFailedException] {
          toArray should not contain only ("HAPPY", "BIRTHDAY", "TO", "YOU")
        }
      }
      def `should use an explicitly provided Equality` {
        (toArray should not contain only ("happy", "birthday", "to")) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (toArray should not contain only ("HAPPY", "BIRTHDAY", "TO", "YOU")) (decided by upperCaseStringEquality)
        }
        toArray should not contain only (" HAPPY ", " BIRTHDAY ", " TO ", " YOU ")
        intercept[TestFailedException] {
          (toArray should not contain only (" HAPPY ", " BIRTHDAY ", " TO ", " YOU ")) (after being lowerCased and trimmed)
        }
      }
    }

    object `when used with (not contain only (..))` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        toArray should (not contain only ("HAPPY", "BIRTHDAY", "TO", "YOU"))
        val e1 = intercept[TestFailedException] {
          toArray should (not contain only ("happy", "birthday", "to", "you"))
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedOnlyElements", decorateToStringValue(toArray), "\"happy\", \"birthday\", \"to\", \"you\""))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        toArray should (not contain only ("NICE", "TO", "MEET", "YOU"))
        intercept[TestFailedException] {
          toArray should (not contain only ("HAPPY", "BIRTHDAY", "TO", "YOU"))
        }
      }
      def `should use an explicitly provided Equality` {
        (toArray should (not contain only ("NICE", "TO", "MEET", "YOU"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (toArray should (not contain only ("HAPPY", "BIRTHDAY", "TO", "YOU"))) (decided by upperCaseStringEquality)
        }
        toArray should (not contain only (" HAPPY ", " BIRTHDAY ", " TO ", " YOU "))
        intercept[TestFailedException] {
          (toArray should (not contain only (" HAPPY ", " BIRTHDAY ", " TO ", " YOU "))) (after being lowerCased and trimmed)
        }
      }
    }
  }

  object `a col of Arrays` {

    val list1s: Vector[Array[Int]] = Vector(Array(3, 2, 1), Array(3, 2, 1), Array(3, 2, 1))
    val lists: Vector[Array[Int]] = Vector(Array(3, 2, 1), Array(3, 2, 1), Array(4, 3, 2))
    val listsArray: Vector[Array[Int]] = Vector(Array(3, 2, 1), Array(3, 2, 1), Array())
    val hiArrays: Vector[Array[String]] = Vector(Array("hi", "he"), Array("hi", "he"), Array("hi", "he"))
    val toArrays: Vector[Array[String]] = Vector(Array("to", "you"), Array("to", "you"), Array("to", "you"))

    object `when used with contain only (..)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should contain only (1, 2, 3)
        atLeast (2, lists) should contain only (1, 2, 3)
        atMost (2, lists) should contain only (1, 2, 3)
        no (lists) should contain only (3, 4, 5)

        val e1 = intercept[TestFailedException] {
          all (lists) should contain only (1, 2, 3)
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(Array(4, 3, 2)) + " did not contain only " + "(1, 2, 3)" + " (ArrayShouldContainOnlySpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))

        val e3 = intercept[TestFailedException] {
          all (lists) should contain only (1, 2, 3)
        }
        e3.failedCodeFileName.get should be ("ArrayShouldContainOnlySpec.scala")
        e3.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e3.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(Array(4, 3, 2)) + " did not contain only " + "(1, 2, 3)" + " (ArrayShouldContainOnlySpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))
      }

      def `should use the implicit Equality in scope` {
        all (hiArrays) should contain only ("he", "hi")
        intercept[TestFailedException] {
          all (hiArrays) should contain only ("ho", "hi")
        }
        implicit val ise = upperCaseStringEquality
        all (hiArrays) should contain only ("HE", "HI")
        intercept[TestFailedException] {
          all (hiArrays) should contain only ("HO", "HI")
        }
      }
      def `should use an explicitly provided Equality` {
        (all (hiArrays) should contain only ("HE", "HI")) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (hiArrays) should contain only ("HO", "HI")) (decided by upperCaseStringEquality)
        }
        implicit val ise = upperCaseStringEquality
        (all (hiArrays) should contain only ("he", "hi")) (decided by defaultEquality[String])
        intercept[TestFailedException] {
          (all (hiArrays) should contain only ("ho", "hi")) (decided by defaultEquality[String])
        }
      }
    }

    object `when used with (contain only (..))` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain only (1, 2, 3))
        atLeast (2, lists) should (contain only (1, 2, 3))
        atMost (2, lists) should (contain only (1, 2, 3))
        no (lists) should (contain only (3, 4, 5))

        val e1 = intercept[TestFailedException] {
          all (lists) should (contain only (1, 2, 3))
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(Array(4, 3, 2)) + " did not contain only " + "(1, 2, 3)" + " (ArrayShouldContainOnlySpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))

        val e4 = intercept[TestFailedException] {
          all (lists) should (contain only (1, 2, 3))
        }
        e4.failedCodeFileName.get should be ("ArrayShouldContainOnlySpec.scala")
        e4.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e4.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(Array(4, 3, 2)) + " did not contain only " + "(1, 2, 3)" + " (ArrayShouldContainOnlySpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))
      }

      def `should use the implicit Equality in scope` {
        all (hiArrays) should (contain only ("he", "hi"))
        intercept[TestFailedException] {
          all (hiArrays) should (contain only ("ho", "hi"))
        }
        implicit val ise = upperCaseStringEquality
        all (hiArrays) should (contain only ("HE", "HI"))
        intercept[TestFailedException] {
          all (hiArrays) should (contain only ("HO", "HI"))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (hiArrays) should (contain only ("HE", "HI"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (hiArrays) should (contain only ("HO", "HI"))) (decided by upperCaseStringEquality)
        }
        implicit val ise = upperCaseStringEquality
        (all (hiArrays) should (contain only ("he", "hi"))) (decided by defaultEquality[String])
        intercept[TestFailedException] {
          (all (hiArrays) should (contain only ("ho", "hi"))) (decided by defaultEquality[String])
        }
      }
    }

    object `when used with not contain only (..)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (toArrays) should not contain only ("fee", "fie", "foe", "fum")
        val e1 = intercept[TestFailedException] {
          all (toArrays) should not contain only ("you", "to")
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(Array("to", "you")) + " contained only " + "(\"you\", \"to\")" +  " (ArrayShouldContainOnlySpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(toArrays)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        all (toArrays) should not contain only ("NICE", "MEET", "YOU")
        intercept[TestFailedException] {
          all (toArrays) should not contain only ("YOU", "TO")
        }
      }
      def `should use an explicitly provided Equality` {
        (all (toArrays) should not contain only ("NICE", "MEET", "YOU")) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (toArrays) should not contain only ("YOU", "TO")) (decided by upperCaseStringEquality)
        }
        all (toArrays) should not contain only (" YOU ", " TO ")
        intercept[TestFailedException] {
          (all (toArrays) should not contain only (" YOU ", " TO ")) (after being lowerCased and trimmed)
        }
      }
    }

    object `when used with (not contain only (..))` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (toArrays) should (not contain only ("fee", "fie", "foe", "fum"))
        val e1 = intercept[TestFailedException] {
          all (toArrays) should (not contain only ("you", "to"))
        }
        e1.failedCodeFileName.get should be ("ArrayShouldContainOnlySpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(Array("to", "you")) + " contained only " + "(\"you\", \"to\")" + " (ArrayShouldContainOnlySpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(toArrays)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        all (toArrays) should (not contain only ("NICE", "MEET", "YOU"))
        intercept[TestFailedException] {
          all (toArrays) should (not contain only ("YOU", "TO"))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (toArrays) should (not contain only ("NICE", "MEET", "YOU"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (toArrays) should (not contain only ("YOU", "TO"))) (decided by upperCaseStringEquality)
        }
        all (toArrays) should (not contain only (" YOU ", " TO "))
        intercept[TestFailedException] {
          (all (toArrays) should (not contain only (" YOU ", " TO "))) (after being lowerCased and trimmed)
        }
      }
    }
  }
}
