/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Normalization
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class JavaMapShouldContainAtLeastOneOfLogicalOrSpec extends FreeSpec with Matchers {
  
  val invertedStringEquality =
    new Equality[(String, String)] {
      def areEqual(a: (String, String), b: Any): Boolean = a != b
    }
  
  val invertedjavaMapOfStringEquality = 
    new Equality[java.util.Map[String, String]] {
      def areEqual(a: java.util.Map[String, String], b: Any): Boolean = a != b
    }
  
  val lowerCased: Normalization[(String, String)] = new Normalization[(String, String)] {
    def normalized(s: (String, String)): (String, String) = (s._1.toLowerCase, s._2.toLowerCase)
    def normalizedIfInstanceOfA(b: Any) =
      b match {
        case (s1: String, s2: String) => normalized((s1, s2))
        case _ => b
      }
    }
val trimmed: Normalization[(String, String)] = new Normalization[(String, String)] {
    def normalized(s: (String, String)): (String, String) = (s._1.trim, s._2.trim)
    def normalizedIfInstanceOfA(b: Any) =
      b match {
        case (s1: String, s2: String) => normalized((s1, s2))
        case _ => b
      }
    }
def javaMap[K, V](elements: (K, V)*): java.util.LinkedHashMap[K, V] = {
    val m = new java.util.LinkedHashMap[K, V]
    elements.foreach(e => m.put(e._1, e._2))
    m
  }
  
  val fileName: String = "JavaMapShouldContainAtLeastOneOfLogicalOrSpec.scala"
  
  "a javaMap" - {
    
    val fumjavaMap: java.util.Map[String, String] = javaMap("fum" -> "fum")
    val tojavaMap: java.util.Map[String, String] = javaMap("to" -> "to")
    
      "when used with (contain oneOf (...) or contain oneOf (...)) syntax" - {
      
        "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumjavaMap should (contain atLeastOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") or contain atLeastOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        fumjavaMap should (contain atLeastOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fam" -> "fam") or contain atLeastOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        fumjavaMap should (contain atLeastOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") or contain atLeastOneOf("fie" -> "fie", "fee" -> "fee", "fam" -> "fam", "foe" -> "foe"))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain atLeastOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fam" -> "fam") or contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumjavaMap), "(fee,fee), (fie,fie), (foe,foe), (fam,fam)") + ", and " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumjavaMap), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
      }

      "should use the implicit Equality in scope" in {
        implicit val ise = invertedStringEquality
        fumjavaMap should (contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you") or contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day"))
        fumjavaMap should (contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you") or contain atLeastOneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        fumjavaMap should (contain atLeastOneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe") or contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum", "fum" -> "fum") or (contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum")))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumjavaMap), "(fum,fum), (fum,fum), (fum,fum), (fum,fum)") + ", and " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumjavaMap), "(fum,fum), (fum,fum), (fum,fum)"), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumjavaMap should (contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you") or contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        (fumjavaMap should (contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you") or contain atLeastOneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        (fumjavaMap should (contain atLeastOneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe") or contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum") or contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumjavaMap), "(fum,fum), (fum,fum)") + ", and " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumjavaMap), "(fum,fum), (fum,fum), (fum,fum)"), fileName, thisLineNumber - 2)
        (fumjavaMap should (contain atLeastOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM ") or contain atLeastOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    "when used with (equal (...) and contain oneOf (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumjavaMap should (equal (fumjavaMap) or contain atLeastOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        fumjavaMap should (equal (tojavaMap) or contain atLeastOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        fumjavaMap should (equal (fumjavaMap) or contain atLeastOneOf("fie" -> "fie", "fee" -> "fee", "fam" -> "fam", "foe" -> "foe"))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (equal (tojavaMap) or contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", and " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumjavaMap), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = invertedStringEquality
        fumjavaMap should (equal (fumjavaMap) or contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day"))
        fumjavaMap should (equal (tojavaMap) or contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day"))
        fumjavaMap should (equal (fumjavaMap) or contain atLeastOneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (equal (tojavaMap) or (contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", and " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumjavaMap), "(fum,fum), (fum,fum), (fum,fum)"), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumjavaMap should (equal (tojavaMap) or contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day"))) (decided by invertedjavaMapOfStringEquality, decided by invertedStringEquality)
        (fumjavaMap should (equal (fumjavaMap) or contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day"))) (decided by invertedjavaMapOfStringEquality, decided by invertedStringEquality)
        (fumjavaMap should (equal (tojavaMap) or contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum"))) (decided by invertedjavaMapOfStringEquality, decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (equal (fumjavaMap) or contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum"))) (decided by invertedjavaMapOfStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", and " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumjavaMap), "(fum,fum), (fum,fum), (fum,fum)"), fileName, thisLineNumber - 2)
        (fumjavaMap should (equal (tojavaMap) or contain atLeastOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))) (decided by invertedjavaMapOfStringEquality, after being lowerCased and trimmed)
      }
    }
    
    "when used with (legacyEqual (...) and contain oneOf (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumjavaMap should (legacyEqual (fumjavaMap) or contain atLeastOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        fumjavaMap should (legacyEqual (tojavaMap) or contain atLeastOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        fumjavaMap should (legacyEqual (fumjavaMap) or contain atLeastOneOf("fie" -> "fie", "fee" -> "fee", "fam" -> "fam", "foe" -> "foe"))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (legacyEqual (tojavaMap) or contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", and " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumjavaMap), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = invertedStringEquality
        fumjavaMap should (legacyEqual (fumjavaMap) or contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day"))
        fumjavaMap should (legacyEqual (tojavaMap) or contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day"))
        fumjavaMap should (legacyEqual (fumjavaMap) or contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum"))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (legacyEqual (tojavaMap) or (contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", and " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumjavaMap), "(fum,fum), (fum,fum), (fum,fum)"), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumjavaMap should (legacyEqual (fumjavaMap) or contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day"))) (decided by invertedStringEquality)
        (fumjavaMap should (legacyEqual (tojavaMap) or contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day"))) (decided by invertedStringEquality)
        (fumjavaMap should (legacyEqual (fumjavaMap) or contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum"))) (decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (legacyEqual (tojavaMap) or contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum"))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", and " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumjavaMap), "(fum,fum), (fum,fum), (fum,fum)"), fileName, thisLineNumber - 2)
        (fumjavaMap should (legacyEqual (fumjavaMap) or contain atLeastOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))) (after being lowerCased and trimmed)
      }
    }

    "when used with (contain oneOf (...) and legacyEqual (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumjavaMap should (contain atLeastOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe") or legacyEqual (fumjavaMap))
        fumjavaMap should (contain atLeastOneOf("fie" -> "fie", "fee" -> "fee", "fam" -> "fam", "foe" -> "foe") or legacyEqual (fumjavaMap))
        fumjavaMap should (contain atLeastOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe") or legacyEqual (tojavaMap))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain atLeastOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fam" -> "fam") or legacyEqual (tojavaMap))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumjavaMap), "(fee,fee), (fie,fie), (foe,foe), (fam,fam)") + ", and " + Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = invertedStringEquality
        fumjavaMap should (contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day") or legacyEqual (fumjavaMap))
        fumjavaMap should (contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum") or legacyEqual (fumjavaMap))
        fumjavaMap should (contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day") or legacyEqual (tojavaMap))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum") or legacyEqual (tojavaMap))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumjavaMap), "(fum,fum), (fum,fum), (fum,fum)") + ", and " + Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumjavaMap should (contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day") or legacyEqual (fumjavaMap))) (decided by invertedStringEquality)
        (fumjavaMap should (contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum") or legacyEqual (fumjavaMap))) (decided by invertedStringEquality)
        (fumjavaMap should (contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day") or legacyEqual (tojavaMap))) (decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum") or legacyEqual (tojavaMap))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumjavaMap), "(fum,fum), (fum,fum), (fum,fum)") + ", and " + Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
        (fumjavaMap should (contain atLeastOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM ") or legacyEqual (fumjavaMap))) (after being lowerCased and trimmed)
      }
    }
    
    "when used with (not contain oneOf (...) and not contain oneOf (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumjavaMap should (not contain atLeastOneOf ("fee", "fie", "foe", "fuu") or not contain atLeastOneOf("fie", "fee", "fuu", "foe"))
        fumjavaMap should (not contain atLeastOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") or not contain atLeastOneOf("fie", "fee", "fuu", "foe"))
        fumjavaMap should (not contain atLeastOneOf ("fee", "fie", "foe", "fuu") or not contain atLeastOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not contain atLeastOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") or not contain atLeastOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e1, Resources("containedAtLeastOneOf", decorateToStringValue(fumjavaMap), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)") + ", and " + Resources("containedAtLeastOneOf", decorateToStringValue(fumjavaMap), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = invertedStringEquality
        fumjavaMap should (not contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum", "fum" -> "fum") or not contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum"))
        fumjavaMap should (not contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you") or not contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum"))
        fumjavaMap should (not contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum", "fum" -> "fum") or not contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you") or not contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day"))
        }
        checkMessageStackDepth(e1, Resources("containedAtLeastOneOf", decorateToStringValue(fumjavaMap), "(happy,happy), (birthday,birthday), (to,to), (you,you)") + ", and " + Resources("containedAtLeastOneOf", decorateToStringValue(fumjavaMap), "(have,have), (a,a), (nice,nice), (day,day)"), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumjavaMap should (not contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum", "fum" -> "fum") or not contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        (fumjavaMap should (not contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you") or not contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        (fumjavaMap should (not contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum", "fum" -> "fum") or not contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (not contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you") or not contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("containedAtLeastOneOf", decorateToStringValue(fumjavaMap), "(happy,happy), (birthday,birthday), (to,to), (you,you)") + ", and " + Resources("containedAtLeastOneOf", decorateToStringValue(fumjavaMap), "(have,have), (a,a), (nice,nice), (day,day)"), fileName, thisLineNumber - 2)
        (fumjavaMap should (contain atLeastOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM ") or contain atLeastOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    "when used with (not equal (...) and not contain oneOf (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumjavaMap should (not equal (tojavaMap) or not contain atLeastOneOf("fie", "fee", "fuu", "foe"))
        fumjavaMap should (not equal (fumjavaMap) or not contain atLeastOneOf("fie", "fee", "fuu", "foe"))
        fumjavaMap should (not equal (tojavaMap) or not contain atLeastOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not equal (fumjavaMap) or not contain atLeastOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", and " + Resources("containedAtLeastOneOf", decorateToStringValue(fumjavaMap), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = invertedStringEquality
        fumjavaMap should (not equal (tojavaMap) or not contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum"))
        fumjavaMap should (not equal (fumjavaMap) or not contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum"))
        fumjavaMap should (not equal (tojavaMap) or not contain atLeastOneOf ("fie", "fee", "fuu", "foe"))
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (not equal (fumjavaMap) or (not contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you")))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", and " + Resources("containedAtLeastOneOf", decorateToStringValue(fumjavaMap), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumjavaMap should (not equal (fumjavaMap) or not contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum"))) (decided by invertedjavaMapOfStringEquality, decided by invertedStringEquality)
        (fumjavaMap should (not equal (tojavaMap) or not contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum"))) (decided by invertedjavaMapOfStringEquality, decided by invertedStringEquality)
        (fumjavaMap should (not equal (fumjavaMap) or not contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))) (decided by invertedjavaMapOfStringEquality, decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (not equal (tojavaMap) or not contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))) (decided by invertedjavaMapOfStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", and " + Resources("containedAtLeastOneOf", decorateToStringValue(fumjavaMap), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
        (fumjavaMap should (not contain atLeastOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUU " -> " FUU ") or not contain atLeastOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUU " -> " FUU "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    "when used with (not be (...) and not contain oneOf (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumjavaMap should (not be (tojavaMap) or not contain atLeastOneOf("fie", "fee", "fuu", "foe"))
        fumjavaMap should (not be (fumjavaMap) or not contain atLeastOneOf("fie", "fee", "fuu", "foe"))
        fumjavaMap should (not be (tojavaMap) or not contain atLeastOneOf("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not be (fumjavaMap) or not contain atLeastOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", and " + Resources("containedAtLeastOneOf", decorateToStringValue(fumjavaMap), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = invertedStringEquality
        fumjavaMap should (not be (tojavaMap) or not contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum"))
        fumjavaMap should (not be (fumjavaMap) or not contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum"))
        fumjavaMap should (not be (tojavaMap) or not contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not be (fumjavaMap) or (not contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you")))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", and " + Resources("containedAtLeastOneOf", decorateToStringValue(fumjavaMap), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumjavaMap should (not be (tojavaMap) or not contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum"))) (decided by invertedStringEquality)
        (fumjavaMap should (not be (fumjavaMap) or not contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum"))) (decided by invertedStringEquality)
        (fumjavaMap should (not be (tojavaMap) or not contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))) (decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (not be (fumjavaMap) or not contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", and " + Resources("containedAtLeastOneOf", decorateToStringValue(fumjavaMap), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
        (fumjavaMap should (not contain atLeastOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUU " -> " FUU ") or not contain atLeastOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUU " -> " FUU "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
  }
  
  "collection of javaMaps" - {
    
    val list1s: Vector[java.util.Map[Int, Int]] = Vector(javaMap(1 -> 1), javaMap(1 -> 1), javaMap(1 -> 1))
    val lists: Vector[java.util.Map[Int, Int]] = Vector(javaMap(1 -> 1), javaMap(1 -> 1), javaMap(2 -> 2))
    val nils: Vector[java.util.Map[Int, Int]] = Vector(javaMap(), javaMap(), javaMap())
    val listsMap: Vector[java.util.Map[Int, Int]] = Vector(javaMap(1 -> 1), javaMap(1 -> 1), javaMap())
    val hijavaMaps: Vector[java.util.Map[String, String]] = Vector(javaMap("hi" -> "hi"), javaMap("hi" -> "hi"), javaMap("hi" -> "hi"))
    val tojavaMaps: Vector[java.util.Map[String, String]] = Vector(javaMap("to" -> "to"), javaMap("to" -> "to"), javaMap("to" -> "to"))
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + left
    
    "when used with (contain oneOf (..) and contain oneOf (..)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        all (list1s) should (contain atLeastOneOf (3 -> 3, 2 -> 2, 1 -> 1) or contain atLeastOneOf (1 -> 1, 3 -> 3, 4 -> 4))
        all (list1s) should (contain atLeastOneOf (3, 2, 5) or contain atLeastOneOf (1 -> 1, 3 -> 3, 4 -> 4))
        all (list1s) should (contain atLeastOneOf (3 -> 3, 2 -> 2, 1 -> 1) or contain atLeastOneOf (2 -> 2, 3 -> 3, 4 -> 4))
        
        atLeast (2, lists) should (contain atLeastOneOf (3 -> 3, 1 -> 1, 5 -> 5) or contain atLeastOneOf (1 -> 1, 3 -> 3, 4 -> 4))
        atLeast (2, lists) should (contain atLeastOneOf (3, 6, 5) or contain atLeastOneOf (1 -> 1, 3 -> 3, 4 -> 4))
        atLeast (2, lists) should (contain atLeastOneOf (3 -> 3, 1 -> 1, 5 -> 5) or contain atLeastOneOf (8 -> 8, 3 -> 3, 4 -> 4))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain atLeastOneOf (6, 7, 8) or contain atLeastOneOf (1 -> 1, 3 -> 3, 4 -> 4))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(javaMap(2 -> 2)) + " did not contain at least one of (6, 7, 8), and " + decorateToStringValue(javaMap(2 -> 2)) + " did not contain at least one of ((1,1), (3,3), (4,4))", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = invertedStringEquality
        
        all (hijavaMaps) should (contain atLeastOneOf ("ho" -> "ho") or contain atLeastOneOf ("he" -> "he"))
        all (hijavaMaps) should (contain atLeastOneOf ("hi" -> "hi") or contain atLeastOneOf ("he" -> "he"))
        all (hijavaMaps) should (contain atLeastOneOf ("ho" -> "ho") or contain atLeastOneOf ("hi" -> "hi"))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaMaps) should (contain atLeastOneOf ("hi" -> "hi") or contain atLeastOneOf ("hi" -> "hi"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap("hi" -> "hi")) + " did not contain at least one of ((hi,hi)), and " + decorateToStringValue(javaMap("hi" -> "hi")) + " did not contain at least one of ((hi,hi))", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (all (hijavaMaps) should (contain atLeastOneOf ("ho" -> "ho") or contain atLeastOneOf ("ho" -> "ho"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        (all (hijavaMaps) should (contain atLeastOneOf ("hi" -> "hi") or contain atLeastOneOf ("ho" -> "ho"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        (all (hijavaMaps) should (contain atLeastOneOf ("ho" -> "ho") or contain atLeastOneOf ("hi" -> "hi"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hijavaMaps) should (contain atLeastOneOf ("hi" -> "hi") or contain atLeastOneOf ("hi" -> "hi"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap("hi" -> "hi")) + " did not contain at least one of ((hi,hi)), and " + decorateToStringValue(javaMap("hi" -> "hi")) + " did not contain at least one of ((hi,hi))", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
    }
    
    "when used with (be (...) and contain oneOf (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        all (list1s) should (be (javaMap(1 -> 1)) or contain atLeastOneOf (1 -> 1, 3 -> 3, 4 -> 4))
        all (list1s) should (be (javaMap(2 -> 2)) or contain atLeastOneOf (1 -> 1, 3 -> 3, 4 -> 4))
        all (list1s) should (be (javaMap(1 -> 1)) or contain atLeastOneOf (2 -> 2, 3 -> 3, 4 -> 4))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (be (javaMap(2 -> 2)) or contain atLeastOneOf (2 -> 2, 3 -> 3, 8 -> 8))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(1 -> 1)) + " was not equal to " + decorateToStringValue(javaMap(2 -> 2)) + ", and " + decorateToStringValue(javaMap(1 -> 1)) + " did not contain at least one of ((2,2), (3,3), (8,8))", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = invertedStringEquality
        
        all (hijavaMaps) should (be (javaMap("hi" -> "hi")) or contain atLeastOneOf ("he" -> "he"))
        all (hijavaMaps) should (be (javaMap("ho" -> "ho")) or contain atLeastOneOf ("he" -> "he"))
        all (hijavaMaps) should (be (javaMap("hi" -> "hi")) or contain atLeastOneOf ("hi" -> "hi"))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaMaps) should (be (javaMap("ho" -> "ho")) or contain atLeastOneOf ("hi" -> "hi"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap("hi" -> "hi")) + " was not equal to " + decorateToStringValue(javaMap("ho" -> "ho")) + ", and " + decorateToStringValue(javaMap("hi" -> "hi")) + " did not contain at least one of ((hi,hi))", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (all (hijavaMaps) should (be (javaMap("hi" -> "hi")) or contain atLeastOneOf ("ho" -> "ho"))) (decided by invertedStringEquality)
        (all (hijavaMaps) should (be (javaMap("ho" -> "ho")) or contain atLeastOneOf ("ho" -> "ho"))) (decided by invertedStringEquality)
        (all (hijavaMaps) should (be (javaMap("hi" -> "hi")) or contain atLeastOneOf ("hi" -> "hi"))) (decided by invertedStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hijavaMaps) should (be (javaMap("ho" -> "ho")) or contain atLeastOneOf ("hi" -> "hi"))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap("hi" -> "hi")) + " was not equal to " + decorateToStringValue(javaMap("ho" -> "ho")) + ", and " + decorateToStringValue(javaMap("hi" -> "hi")) + " did not contain at least one of ((hi,hi))", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
    }
    
    "when used with (not contain oneOf (..) and not contain oneOf (..))" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        all (list1s) should (not contain atLeastOneOf (3 -> 3, 2 -> 2, 8 -> 8) or not contain atLeastOneOf (8 -> 8, 3 -> 3, 4 -> 4))
        all (list1s) should (not contain atLeastOneOf (1 -> 1, 2 -> 2, 8 -> 8) or not contain atLeastOneOf (8 -> 8, 3 -> 3, 4 -> 4))
        all (list1s) should (not contain atLeastOneOf (3 -> 3, 2 -> 2, 8 -> 8) or not contain atLeastOneOf (8 -> 8, 3 -> 3, 1 -> 1))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain atLeastOneOf (2 -> 2, 6 -> 6, 8 -> 8) or not contain atLeastOneOf (2 -> 2, 3 -> 3, 4 -> 4))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(javaMap(2 -> 2)) + " contained at least one of ((2,2), (6,6), (8,8)), and " + decorateToStringValue(javaMap(2 -> 2)) + " contained at least one of ((2,2), (3,3), (4,4))", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = invertedStringEquality
        
        all (hijavaMaps) should (not contain atLeastOneOf ("hi" -> "hi") or not contain atLeastOneOf ("hi" -> "hi"))
        all (hijavaMaps) should (not contain atLeastOneOf ("ho" -> "ho") or not contain atLeastOneOf ("hi" -> "hi"))
        all (hijavaMaps) should (not contain atLeastOneOf ("hi" -> "hi") or not contain atLeastOneOf ("h0"))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaMaps) should (not contain atLeastOneOf ("ho" -> "ho") or not contain atLeastOneOf ("ho" -> "ho"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap("hi" -> "hi")) + " contained at least one of ((ho,ho)), and " + decorateToStringValue(javaMap("hi" -> "hi")) + " contained at least one of ((ho,ho))", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (all (hijavaMaps) should (not contain atLeastOneOf ("hi" -> "hi") or not contain atLeastOneOf ("hi" -> "hi"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        (all (hijavaMaps) should (not contain atLeastOneOf ("ho" -> "ho") or not contain atLeastOneOf ("hi" -> "hi"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        (all (hijavaMaps) should (not contain atLeastOneOf ("hi" -> "hi") or not contain atLeastOneOf ("ho" -> "ho"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hijavaMaps) should (not contain atLeastOneOf ("ho" -> "ho") or not contain atLeastOneOf ("ho" -> "ho"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap("hi" -> "hi")) + " contained at least one of ((ho,ho)), and " + decorateToStringValue(javaMap("hi" -> "hi")) + " contained at least one of ((ho,ho))", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
    }
    
    "when used with (not be (...) and not contain oneOf (...))" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        all (list1s) should (not be (javaMap(2 -> 2)) or not contain atLeastOneOf (8 -> 8, 3 -> 3, 4 -> 4))
        all (list1s) should (not be (javaMap(1 -> 1)) or not contain atLeastOneOf (8 -> 8, 3 -> 3, 4 -> 4))
        all (list1s) should (not be (javaMap(2 -> 2)) or not contain atLeastOneOf (8 -> 8, 3 -> 3, 1 -> 1))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (not be (javaMap(1 -> 1)) or not contain atLeastOneOf (2 -> 2, 3 -> 3, 1 -> 1))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap(1 -> 1)) + " was equal to " + decorateToStringValue(javaMap(1 -> 1)) + ", and " + decorateToStringValue(javaMap(1 -> 1)) + " contained at least one of ((2,2), (3,3), (1,1))", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = invertedStringEquality
        
        all (hijavaMaps) should (not be (javaMap("ho" -> "ho")) or not contain atLeastOneOf ("hi" -> "hi"))
        all (hijavaMaps) should (not be (javaMap("hi" -> "hi")) or not contain atLeastOneOf ("hi" -> "hi"))
        all (hijavaMaps) should (not be (javaMap("ho" -> "ho")) or not contain atLeastOneOf ("ho" -> "ho"))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaMaps) should (not be (javaMap("hi" -> "hi")) or not contain atLeastOneOf ("ho" -> "ho"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap("hi" -> "hi")) + " was equal to " + decorateToStringValue(javaMap("hi" -> "hi")) + ", and " + decorateToStringValue(javaMap("hi" -> "hi")) + " contained at least one of ((ho,ho))", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (all (hijavaMaps) should (not be (javaMap("ho" -> "ho")) or not contain atLeastOneOf ("hi" -> "hi"))) (decided by invertedStringEquality)
        (all (hijavaMaps) should (not be (javaMap("hi" -> "hi")) or not contain atLeastOneOf ("hi" -> "hi"))) (decided by invertedStringEquality)
        (all (hijavaMaps) should (not be (javaMap("ho" -> "ho")) or not contain atLeastOneOf ("ho" -> "ho"))) (decided by invertedStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hijavaMaps) should (not be (javaMap("hi" -> "hi")) or not contain atLeastOneOf ("ho" -> "ho"))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap("hi" -> "hi")) + " was equal to " + decorateToStringValue(javaMap("hi" -> "hi")) + ", and " + decorateToStringValue(javaMap("hi" -> "hi")) + " contained at least one of ((ho,ho))", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
    }
  }
}
