/*
 * Copyright 2001-2013 Artima, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Normalization
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class JavaMapShouldContainNoneOfSpec extends Spec with Matchers {

  val upperCaseEquality =
    new Equality[(String, String)] {
      def areEqual(a: (String, String), b: Any): Boolean = b match {
    case (b1: String, b2: String) => a._1.toUpperCase == b1 && a._2.toUpperCase == b2
    case _ => (a._1.toUpperCase, a._2.toUpperCase) == b
  }
    }
  
  val lowerCased: Normalization[(String, String)] = new Normalization[(String, String)] {
    def normalized(s: (String, String)): (String, String) = (s._1.toLowerCase, s._2.toLowerCase)
    def normalizedIfInstanceOfA(b: Any) =
      b match {
        case (s1: String, s2: String) => normalized((s1, s2))
        case _ => b
      }
    }
val trimmed: Normalization[(String, String)] = new Normalization[(String, String)] {
    def normalized(s: (String, String)): (String, String) = (s._1.trim, s._2.trim)
    def normalizedIfInstanceOfA(b: Any) =
      b match {
        case (s1: String, s2: String) => normalized((s1, s2))
        case _ => b
      }
    }
def javaMap[K, V](elements: (K, V)*): java.util.LinkedHashMap[K, V] = {
    val m = new java.util.LinkedHashMap[K, V]
    elements.foreach(e => m.put(e._1, e._2))
    m
  }

  object `a javaMap` {

    val fumjavaMap: java.util.Map[String, String] = javaMap("fum" -> "fum")
    val tojavaMap: java.util.Map[String, String] = javaMap("to" -> "to")

    object `when used with contain noneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fam" -> "fam")
        val e1 = intercept[TestFailedException] {
          fumjavaMap should contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainNoneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedOneOfElements", decorateToStringValue(fumjavaMap), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"))
        // Contains duplicate elements in the right list
        val e2 = intercept[IllegalArgumentException] {
          fumjavaMap should contain noneOf ("fee", "fam", "foe", "fam")
        }
        e2.getMessage should be (Resources("noneOfDuplicate", "\"fam\""))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseEquality
        fumjavaMap should contain noneOf ("FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE", "FU" -> "FU")
        intercept[TestFailedException] {
          fumjavaMap should contain noneOf ("FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE", "FU" -> "FU")
        }
      }
      def `should use an explicitly provided Equality` {
        (fumjavaMap should contain noneOf ("FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE", "FU" -> "FU")) (decided by upperCaseEquality)
        intercept[TestFailedException] {
          (fumjavaMap should contain noneOf ("FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE", "FU" -> "FU")) (decided by upperCaseEquality)
        }
        intercept[TestFailedException] {
          (fumjavaMap should contain noneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM ")) (after being lowerCased and trimmed)
        }
        (fumjavaMap should contain noneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FAM " -> " FAM ")) (after being lowerCased and trimmed)
      }
    }

    object `when used with (contain noneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {

        fumjavaMap should (contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fam" -> "fam"))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainNoneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedOneOfElements", decorateToStringValue(fumjavaMap), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseEquality
        fumjavaMap should (contain noneOf ("FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE", "FU" -> "FU"))
        intercept[TestFailedException] {
          fumjavaMap should (contain noneOf ("FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE", "FU" -> "FU"))
        }
      }
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (contain noneOf ("FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE", "FU" -> "FU"))) (decided by upperCaseEquality)
        intercept[TestFailedException] {
          (fumjavaMap should (contain noneOf ("FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE", "FU" -> "FU"))) (decided by upperCaseEquality)
        }
        fumjavaMap should (contain noneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))
        intercept[TestFailedException] {
          (fumjavaMap should (contain noneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))) (after being lowerCased and trimmed)
        }
      }
    }

    object `when used with not contain noneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should not contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")
        val e1 = intercept[TestFailedException] {
          fumjavaMap should not contain noneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you")
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainNoneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainOneOfElements", decorateToStringValue(fumjavaMap), "(happy,happy), (birthday,birthday), (to,to), (you,you)"))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseEquality
        fumjavaMap should not contain noneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM")
        intercept[TestFailedException] {
          fumjavaMap should not contain noneOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU")
        }
      }
      def `should use an explicitly provided Equality` {
        (fumjavaMap should not contain noneOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM")) (decided by upperCaseEquality)
        intercept[TestFailedException] {
          (fumjavaMap should not contain noneOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU")) (decided by upperCaseEquality)
        }
        (fumjavaMap should not contain noneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM ")) (after being lowerCased and trimmed)
        intercept[TestFailedException] {
          fumjavaMap should not contain noneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM ")
        }
      }
    }

    object `when used with (not contain noneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        tojavaMap should (not contain noneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        val e1 = intercept[TestFailedException] {
          tojavaMap should (not contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainNoneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainOneOfElements", decorateToStringValue(tojavaMap), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseEquality
        tojavaMap should (not contain noneOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU"))
        intercept[TestFailedException] {
          tojavaMap should (not contain noneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        }
      }
      def `should use an explicitly provided Equality` {
        (tojavaMap should (not contain noneOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU"))) (decided by upperCaseEquality)
        intercept[TestFailedException] {
          (tojavaMap should (not contain noneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))) (decided by upperCaseEquality)
        }
        (tojavaMap should (not contain noneOf (" HAPPY " -> " HAPPY ", " BIRTHDAY " -> " BIRTHDAY ", " TO " -> " TO ", " YOU " -> " YOU "))) (after being lowerCased and trimmed)
        intercept[TestFailedException] {
          tojavaMap should (not contain noneOf (" HAPPY " -> " HAPPY ", " BIRTHDAY " -> " BIRTHDAY ", " TO " -> " TO ", " YOU " -> " YOU "))
        }
      }
    }
  }

  object `a collection of javaMaps` {

    val list1s: Vector[java.util.Map[Int, Int]] = Vector(javaMap(1 -> 1), javaMap(1 -> 1), javaMap(1 -> 1))
    val lists: Vector[java.util.Map[Int, Int]] = Vector(javaMap(1 -> 1), javaMap(1 -> 1), javaMap(2 -> 2))
    val nils: Vector[java.util.Map[Int, Int]] = Vector(javaMap(), javaMap(), javaMap())
    val listsMap: Vector[java.util.Map[Int, Int]] = Vector(javaMap(1 -> 1), javaMap(1 -> 1), javaMap())
    val hijavaMaps: Vector[java.util.Map[String, String]] = Vector(javaMap("hi" -> "hi"), javaMap("hi" -> "hi"), javaMap("hi" -> "hi"))
    val tojavaMaps: Vector[java.util.Map[String, String]] = Vector(javaMap("to" -> "to"), javaMap("to" -> "to"), javaMap("to" -> "to"))

    object `when used with contain noneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should contain noneOf (2 -> 2, 3 -> 3, 4 -> 4)
        atLeast (2, lists) should contain noneOf (8 -> 8, 3 -> 3, 4 -> 4)
        atMost (2, lists) should contain noneOf (2 -> 2, 3 -> 3, 4 -> 4)
        no (lists) should contain noneOf (1 -> 1, 2 -> 2, 5 -> 5)

        val e1 = intercept[TestFailedException] {
          all (lists) should contain noneOf (2 -> 2, 3 -> 3, 4 -> 4)
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainNoneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(javaMap(2 -> 2)) + " contained one of ((2,2), (3,3), (4,4)) (JavaMapShouldContainNoneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))
      }

      def `should use the implicit Equality in scope` {
        all (hijavaMaps) should contain noneOf ("ho" -> "ho")
        intercept[TestFailedException] {
          all (hijavaMaps) should contain noneOf ("hi" -> "hi")
        }
        implicit val ise = upperCaseEquality
        all (hijavaMaps) should contain noneOf ("hi" -> "hi")
        intercept[TestFailedException] {
          all (hijavaMaps) should contain noneOf ("HI" -> "HI")
        }
      }
      def `should use an explicitly provided Equality` {
        (all (hijavaMaps) should contain noneOf ("hi" -> "hi")) (decided by upperCaseEquality)
        intercept[TestFailedException] {
          (all (hijavaMaps) should contain noneOf ("HI" -> "HI")) (decided by upperCaseEquality)
        }
        implicit val ise = upperCaseEquality
        (all (hijavaMaps) should contain noneOf ("ho" -> "ho")) (decided by defaultEquality[(String, String)])
        intercept[TestFailedException] {
          (all (hijavaMaps) should contain noneOf ("hi" -> "hi")) (decided by defaultEquality[(String, String)])
        }
      }
    }

    object `when used with (contain noneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain noneOf (2 -> 2, 3 -> 3, 4 -> 4))
        atLeast (2, lists) should (contain noneOf (2 -> 2, 3 -> 3, 4 -> 4))
        atMost (2, lists) should (contain noneOf (2 -> 2, 3 -> 3, 4 -> 4))
        no (lists) should (contain noneOf (1 -> 1, 2 -> 2, 5 -> 5))

        val e1 = intercept[TestFailedException] {
          all (lists) should (contain noneOf (2 -> 2, 3 -> 3, 4 -> 4))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainNoneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(javaMap(2 -> 2)) + " contained one of ((2,2), (3,3), (4,4)) (JavaMapShouldContainNoneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))
      }

      def `should use the implicit Equality in scope` {
        all (hijavaMaps) should (contain noneOf ("ho" -> "ho"))
        intercept[TestFailedException] {
          all (hijavaMaps) should (contain noneOf ("hi" -> "hi"))
        }
        implicit val ise = upperCaseEquality
        all (hijavaMaps) should (contain noneOf ("hi" -> "hi"))
        intercept[TestFailedException] {
          all (hijavaMaps) should (contain noneOf ("HI" -> "HI"))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (hijavaMaps) should (contain noneOf ("hi" -> "hi"))) (decided by upperCaseEquality)
        intercept[TestFailedException] {
          (all (hijavaMaps) should (contain noneOf ("HI" -> "HI"))) (decided by upperCaseEquality)
        }
        implicit val ise = upperCaseEquality
        (all (hijavaMaps) should (contain noneOf ("ho" -> "ho"))) (decided by defaultEquality[(String, String)])
        intercept[TestFailedException] {
          (all (hijavaMaps) should (contain noneOf ("hi" -> "hi"))) (decided by defaultEquality[(String, String)])
        }
      }
    }

    object `when used with not contain noneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (tojavaMaps) should not contain noneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you")
        val e1 = intercept[TestFailedException] {
          all (tojavaMaps) should not contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainNoneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(javaMap("to" -> "to")) + " did not contain one of ((fee,fee), (fie,fie), (foe,foe), (fum,fum)) (JavaMapShouldContainNoneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(tojavaMaps)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseEquality
        all (tojavaMaps) should not contain noneOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU")
        intercept[TestFailedException] {
          all (tojavaMaps) should not contain noneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you")
        }
      }
      def `should use an explicitly provided Equality` {
        (all (tojavaMaps) should not contain noneOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU")) (decided by upperCaseEquality)
        intercept[TestFailedException] {
          (all (tojavaMaps) should not contain noneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you")) (decided by upperCaseEquality)
        }
        (all (tojavaMaps) should not contain noneOf (" HAPPY " -> " HAPPY ", " BIRTHDAY " -> " BIRTHDAY ", " TO " -> " TO ", " YOU " -> " YOU ")) (after being lowerCased and trimmed)
        intercept[TestFailedException] {
          all (tojavaMaps) should not contain noneOf (" HAPPY " -> " HAPPY ", " BIRTHDAY " -> " BIRTHDAY ", " TO " -> " TO ", " YOU " -> " YOU ")
        }
      }
    }

    object `when used with (not contain noneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (tojavaMaps) should (not contain noneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        val e1 = intercept[TestFailedException] {
          all (tojavaMaps) should (not contain noneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainNoneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(javaMap("to" -> "to")) + " did not contain one of ((fee,fee), (fie,fie), (foe,foe), (fum,fum)) (JavaMapShouldContainNoneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(tojavaMaps)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseEquality
        all (tojavaMaps) should (not contain noneOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU"))
        intercept[TestFailedException] {
          all (tojavaMaps) should (not contain noneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (tojavaMaps) should (not contain noneOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU"))) (decided by upperCaseEquality)
        intercept[TestFailedException] {
          (all (tojavaMaps) should (not contain noneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))) (decided by upperCaseEquality)
        }
        (all (tojavaMaps) should (not contain noneOf (" HAPPY " -> " HAPPY ", " BIRTHDAY " -> " BIRTHDAY ", " TO " -> " TO ", " YOU " -> " YOU "))) (after being lowerCased and trimmed)
        intercept[TestFailedException] {
          all (tojavaMaps) should (not contain noneOf (" HAPPY " -> " HAPPY ", " BIRTHDAY " -> " BIRTHDAY ", " TO " -> " TO ", " YOU " -> " YOU "))
        }
      }
    }
  }
}
