/*
 * Copyright 2001-2013 Artima, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Normalization
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class JavaMapShouldContainOneOfSpec extends Spec with Matchers {

  val upperCaseEquality =
    new Equality[(String, String)] {
      def areEqual(a: (String, String), b: Any): Boolean = b match {
    case (b1: String, b2: String) => a._1.toUpperCase == b1 && a._2.toUpperCase == b2
    case _ => (a._1.toUpperCase, a._2.toUpperCase) == b
  }
    }
  
  val lowerCased: Normalization[(String, String)] = new Normalization[(String, String)] {
    def normalized(s: (String, String)): (String, String) = (s._1.toLowerCase, s._2.toLowerCase)
    def normalizedIfInstanceOfA(b: Any) =
      b match {
        case (s1: String, s2: String) => normalized((s1, s2))
        case _ => b
      }
    }
val trimmed: Normalization[(String, String)] = new Normalization[(String, String)] {
    def normalized(s: (String, String)): (String, String) = (s._1.trim, s._2.trim)
    def normalizedIfInstanceOfA(b: Any) =
      b match {
        case (s1: String, s2: String) => normalized((s1, s2))
        case _ => b
      }
    }
def javaMap[K, V](elements: (K, V)*): java.util.LinkedHashMap[K, V] = {
    val m = new java.util.LinkedHashMap[K, V]
    elements.foreach(e => m.put(e._1, e._2))
    m
  }

  object `a javaMap` {

    val fumjavaMap: java.util.Map[String, String] = javaMap("fum" -> "fum")
    val tojavaMap: java.util.Map[String, String] = javaMap("to" -> "to")
    val fumfujavaMap: java.util.Map[String, String] = javaMap("fum" -> "fum", "fu" -> "fu")

    object `when used with contain oneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should contain oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")
        val e1 = intercept[TestFailedException] {
          fumjavaMap should contain oneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you")
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainOneOfElements", decorateToStringValue(fumjavaMap), "(happy,happy), (birthday,birthday), (to,to), (you,you)"))
        // Here it contains two of, not one of
        val e2 = intercept[TestFailedException] {
          fumfujavaMap should contain oneOf ("fee" -> "fee", "fum" -> "fum", "foe" -> "foe", "fu" -> "fu")
        }
        e2.failedCodeFileName.get should be ("JavaMapShouldContainOneOfSpec.scala")
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message.get should be (Resources("didNotContainOneOfElements", decorateToStringValue(fumfujavaMap), "(fee,fee), (fum,fum), (foe,foe), (fu,fu)"))
        // Contains duplicate elements in the right list
        val e3 = intercept[IllegalArgumentException] {
          fumjavaMap should contain oneOf ("fee", "fum", "foe", "fum")
        }
        e3.getMessage should be (Resources("oneOfDuplicate", "\"fum\""))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseEquality
        fumjavaMap should contain oneOf ("FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE", "FU" -> "FU")
        intercept[TestFailedException] {
          fumjavaMap should contain oneOf ("fee" -> "fee", "fum" -> "fum", "foe" -> "foe", "fu" -> "fu")
        }
      }
      def `should use an explicitly provided Equality` {
        (fumjavaMap should contain oneOf ("FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE", "FU" -> "FU")) (decided by upperCaseEquality)
        intercept[TestFailedException] {
          (fumjavaMap should contain oneOf ("fee" -> "fee", "fum" -> "fum", "foe" -> "foe", "fu" -> "fu")) (decided by upperCaseEquality)
        }
        intercept[TestFailedException] {
          fumjavaMap should contain oneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM ")
        }
        (fumjavaMap should contain oneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM ")) (after being lowerCased and trimmed)
      }
    }

    object `when used with (contain oneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {

        fumjavaMap should (contain oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain oneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainOneOfElements", decorateToStringValue(fumjavaMap), "(happy,happy), (birthday,birthday), (to,to), (you,you)"))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseEquality
        fumjavaMap should (contain oneOf ("FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE", "FU" -> "FU"))
        intercept[TestFailedException] {
          fumjavaMap should (contain oneOf ("fee" -> "fee", "fum" -> "fum", "foe" -> "foe", "fu" -> "fu"))
        }
      }
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (contain oneOf ("FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE", "FU" -> "FU"))) (decided by upperCaseEquality)
        intercept[TestFailedException] {
          (fumjavaMap should (contain oneOf ("fee" -> "fee", "fum" -> "fum", "foe" -> "foe", "fu" -> "fu"))) (decided by upperCaseEquality)
        }
        intercept[TestFailedException] {
          fumjavaMap should (contain oneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))
        }
        (fumjavaMap should (contain oneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))) (after being lowerCased and trimmed)
      }
    }

/*
 I purposely don't want to support this syntax:

        fumjavaMap should contain (oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        fumjavaMap should (contain (oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")))

 Reason is that I don't want people putting parentheses between contain and oneOf, etc. This will not compile.
*/
    object `when used with not contain oneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        tojavaMap should not contain oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")
        val e1 = intercept[TestFailedException] {
          tojavaMap should not contain oneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you")
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedOneOfElements", decorateToStringValue(tojavaMap), "(happy,happy), (birthday,birthday), (to,to), (you,you)"))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseEquality
        tojavaMap should not contain oneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you")
        intercept[TestFailedException] {
          tojavaMap should not contain oneOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU")
        }
      }
      def `should use an explicitly provided Equality` {
        (tojavaMap should not contain oneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you")) (decided by upperCaseEquality)
        intercept[TestFailedException] {
          (tojavaMap should not contain oneOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU")) (decided by upperCaseEquality)
        }
        tojavaMap should not contain oneOf (" HAPPY " -> " HAPPY ", " BIRTHDAY " -> " BIRTHDAY ", " TO " -> " TO ", " YOU " -> " YOU ")
        intercept[TestFailedException] {
          (tojavaMap should not contain oneOf (" HAPPY " -> " HAPPY ", " BIRTHDAY " -> " BIRTHDAY ", " TO " -> " TO ", " YOU " -> " YOU ")) (after being lowerCased and trimmed)
        }
      }
    }

/*
Interesting, of these three, the top one does happen to compile and run:

        tojavaMap should not contain (oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        // tojavaMap should not (contain (oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")))
        // tojavaMap should (not (contain (oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))))

The bottom two don't, but still I don't want to support that in general.
*/
    object `when used with (not contain oneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        tojavaMap should (not contain oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        val e1 = intercept[TestFailedException] {
          tojavaMap should (not contain oneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedOneOfElements", decorateToStringValue(tojavaMap), "(happy,happy), (birthday,birthday), (to,to), (you,you)"))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseEquality
        tojavaMap should (not contain oneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        intercept[TestFailedException] {
          tojavaMap should (not contain oneOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU"))
        }
      }
      def `should use an explicitly provided Equality` {
        (tojavaMap should (not contain oneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))) (decided by upperCaseEquality)
        intercept[TestFailedException] {
          (tojavaMap should (not contain oneOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU"))) (decided by upperCaseEquality)
        }
        tojavaMap should (not contain oneOf (" HAPPY " -> " HAPPY ", " BIRTHDAY " -> " BIRTHDAY ", " TO " -> " TO ", " YOU " -> " YOU "))
        intercept[TestFailedException] {
          (tojavaMap should (not contain oneOf (" HAPPY " -> " HAPPY ", " BIRTHDAY " -> " BIRTHDAY ", " TO " -> " TO ", " YOU " -> " YOU "))) (after being lowerCased and trimmed)
        }
      }
    }
  }

  object `a collection of javaMaps` {

    val list1s: Vector[java.util.Map[Int, Int]] = Vector(javaMap(1 -> 1), javaMap(1 -> 1), javaMap(1 -> 1))
    val lists: Vector[java.util.Map[Int, Int]] = Vector(javaMap(1 -> 1), javaMap(1 -> 1), javaMap(2 -> 2))
    val nils: Vector[java.util.Map[Int, Int]] = Vector(javaMap(), javaMap(), javaMap())
    val listsMap: Vector[java.util.Map[Int, Int]] = Vector(javaMap(1 -> 1), javaMap(1 -> 1), javaMap())
    val hijavaMaps: Vector[java.util.Map[String, String]] = Vector(javaMap("hi" -> "hi"), javaMap("hi" -> "hi"), javaMap("hi" -> "hi"))
    val tojavaMaps: Vector[java.util.Map[String, String]] = Vector(javaMap("to" -> "to"), javaMap("to" -> "to"), javaMap("to" -> "to"))

    object `when used with contain oneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should contain oneOf (1 -> 1, 3 -> 3, 4 -> 4)
        atLeast (2, lists) should contain oneOf (1 -> 1, 3 -> 3, 4 -> 4)
        atMost (2, lists) should contain oneOf (2 -> 2, 3 -> 3, 4 -> 4)
        no (lists) should contain oneOf (3 -> 3, 4 -> 4, 5 -> 5)
        no (nils) should contain oneOf (1 -> 1, 3 -> 3, 4 -> 4)
        no (listsMap) should contain oneOf (3 -> 3, 4 -> 4, 5 -> 5)

        val e1 = intercept[TestFailedException] {
          all (lists) should contain oneOf (1 -> 1, 3 -> 3, 4 -> 4)
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(javaMap(2 -> 2)) + " did not contain one of ((1,1), (3,3), (4,4)) (JavaMapShouldContainOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))

        val e2 = intercept[TestFailedException] {
          all (nils) should contain oneOf ("ho" -> "ho", "hey" -> "hey", "howdy" -> "howdy")
        }
        e2.failedCodeFileName.get should be ("JavaMapShouldContainOneOfSpec.scala")
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(javaMap()) + " did not contain one of ((ho,ho), (hey,hey), (howdy,howdy)) (JavaMapShouldContainOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(nils)))

        val e4 = intercept[TestFailedException] {
          all (listsMap) should contain oneOf (1 -> 1, 3 -> 3, 4 -> 4)
        }
        e4.failedCodeFileName.get should be ("JavaMapShouldContainOneOfSpec.scala")
        e4.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e4.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(javaMap()) + " did not contain one of ((1,1), (3,3), (4,4)) (JavaMapShouldContainOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(listsMap)))
      }

      def `should use the implicit Equality in scope` {
        all (hijavaMaps) should contain oneOf ("hi" -> "hi")
        intercept[TestFailedException] {
          all (hijavaMaps) should contain oneOf ("ho" -> "ho")
        }
        implicit val ise = upperCaseEquality
        all (hijavaMaps) should contain oneOf ("HI" -> "HI")
        intercept[TestFailedException] {
          all (hijavaMaps) should contain oneOf ("hi" -> "hi")
        }
      }
      def `should use an explicitly provided Equality` {
        (all (hijavaMaps) should contain oneOf ("HI" -> "HI")) (decided by upperCaseEquality)
        intercept[TestFailedException] {
          (all (hijavaMaps) should contain oneOf ("hi" -> "hi")) (decided by upperCaseEquality)
        }
        implicit val ise = upperCaseEquality
        (all (hijavaMaps) should contain oneOf ("hi" -> "hi")) (decided by defaultEquality[(String, String)])
        intercept[TestFailedException] {
          (all (hijavaMaps) should contain oneOf ("ho" -> "ho")) (decided by defaultEquality[(String, String)])
        }
      }
    }

    object `when used with (contain oneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain oneOf (1 -> 1, 3 -> 3, 4 -> 4))
        atLeast (2, lists) should (contain oneOf (1 -> 1, 3 -> 3, 4 -> 4))
        atMost (2, lists) should (contain oneOf (2 -> 2, 3 -> 3, 4 -> 4))
        no (lists) should (contain oneOf (3 -> 3, 4 -> 4, 5 -> 5))
        no (nils) should (contain oneOf (1 -> 1, 3 -> 3, 4 -> 4))
        no (listsMap) should (contain oneOf (3 -> 3, 4 -> 4, 5 -> 5))

        val e1 = intercept[TestFailedException] {
          all (lists) should (contain oneOf (1 -> 1, 3 -> 3, 4 -> 4))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(javaMap(2 -> 2)) + " did not contain one of ((1,1), (3,3), (4,4)) (JavaMapShouldContainOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))

        val e2 = intercept[TestFailedException] {
          all (nils) should (contain oneOf ("ho" -> "ho", "hey" -> "hey", "howdy" -> "howdy"))
        }
        e2.failedCodeFileName.get should be ("JavaMapShouldContainOneOfSpec.scala")
        e2.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e2.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(javaMap()) + " did not contain one of ((ho,ho), (hey,hey), (howdy,howdy)) (JavaMapShouldContainOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(nils)))

        val e4 = intercept[TestFailedException] {
          all (listsMap) should (contain oneOf (1 -> 1, 3 -> 3, 4 -> 4))
        }
        e4.failedCodeFileName.get should be ("JavaMapShouldContainOneOfSpec.scala")
        e4.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e4.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(javaMap()) + " did not contain one of ((1,1), (3,3), (4,4)) (JavaMapShouldContainOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(listsMap)))
      }

      def `should use the implicit Equality in scope` {
        all (hijavaMaps) should (contain oneOf ("hi" -> "hi"))
        intercept[TestFailedException] {
          all (hijavaMaps) should (contain oneOf ("ho" -> "ho"))
        }
        implicit val ise = upperCaseEquality
        all (hijavaMaps) should (contain oneOf ("HI" -> "HI"))
        intercept[TestFailedException] {
          all (hijavaMaps) should (contain oneOf ("hi" -> "hi"))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (hijavaMaps) should (contain oneOf ("HI" -> "HI"))) (decided by upperCaseEquality)
        intercept[TestFailedException] {
          (all (hijavaMaps) should (contain oneOf ("hi" -> "hi"))) (decided by upperCaseEquality)
        }
        implicit val ise = upperCaseEquality
        (all (hijavaMaps) should (contain oneOf ("hi" -> "hi"))) (decided by defaultEquality[(String, String)])
        intercept[TestFailedException] {
          (all (hijavaMaps) should (contain oneOf ("ho" -> "ho"))) (decided by defaultEquality[(String, String)])
        }
      }
    }

/*
 I purposely don't want to support this syntax:

scala> all (list1s) should contain (oneOf (1 -> 1, 3 -> 3, 4 -> 4))
<console>:15: error: org.scalatest.words.NewContainWord does not take parameters
              all (list1s) should contain (oneOf (1 -> 1, 3 -> 3, 4 -> 4))
                                             ^

scala> all (list1s) should (contain (oneOf (1 -> 1, 3 -> 3, 4 -> 4)))
<console>:15: error: org.scalatest.words.NewContainWord does not take parameters
              all (list1s) should (contain (oneOf (1 -> 1, 3 -> 3, 4 -> 4)))
                                              ^

 Reason is that I don't want people putting parentheses between contain and oneOf, etc. This will not compile.
*/
    object `when used with not contain oneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (tojavaMaps) should not contain oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")
        val e1 = intercept[TestFailedException] {
          all (tojavaMaps) should not contain oneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you")
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(javaMap("to" -> "to")) + " contained one of ((happy,happy), (birthday,birthday), (to,to), (you,you)) (JavaMapShouldContainOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(tojavaMaps)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseEquality
        all (tojavaMaps) should not contain oneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you")
        intercept[TestFailedException] {
          all (tojavaMaps) should not contain oneOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU")
        }
      }
      def `should use an explicitly provided Equality` {
        (all (tojavaMaps) should not contain oneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you")) (decided by upperCaseEquality)
        intercept[TestFailedException] {
          (all (tojavaMaps) should not contain oneOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU")) (decided by upperCaseEquality)
        }
        all (tojavaMaps) should not contain oneOf (" HAPPY " -> " HAPPY ", " BIRTHDAY " -> " BIRTHDAY ", " TO " -> " TO ", " YOU " -> " YOU ")
        intercept[TestFailedException] {
          (all (tojavaMaps) should not contain oneOf (" HAPPY " -> " HAPPY ", " BIRTHDAY " -> " BIRTHDAY ", " TO " -> " TO ", " YOU " -> " YOU ")) (after being lowerCased and trimmed)
        }
      }
    }

/*
Interesting, of these three, the last one does happen to compile and run:

scala> all (tojavaMaps) should (not (contain (oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))))
<console>:15: error: org.scalatest.words.NewContainWord does not take parameters
              all (tojavaMaps) should (not (contain (oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))))
                                                    ^

scala> all (tojavaMaps) should not (contain (oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")))
<console>:15: error: org.scalatest.words.NewContainWord does not take parameters
              all (tojavaMaps) should not (contain (oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")))
                                                   ^

scala> all (tojavaMaps) should not contain (oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))

The top two don't, but still I don't want to support that in general.
*/
    object `when used with (not contain oneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (tojavaMaps) should (not contain oneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        val e1 = intercept[TestFailedException] {
          all (tojavaMaps) should (not contain oneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        }
        e1.failedCodeFileName.get should be ("JavaMapShouldContainOneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(javaMap("to" -> "to")) + " contained one of ((happy,happy), (birthday,birthday), (to,to), (you,you)) (JavaMapShouldContainOneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(tojavaMaps)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseEquality
        all (tojavaMaps) should (not contain oneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        intercept[TestFailedException] {
          all (tojavaMaps) should (not contain oneOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU"))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (tojavaMaps) should (not contain oneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))) (decided by upperCaseEquality)
        intercept[TestFailedException] {
          (all (tojavaMaps) should (not contain oneOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU"))) (decided by upperCaseEquality)
        }
        all (tojavaMaps) should (not contain oneOf (" HAPPY " -> " HAPPY ", " BIRTHDAY " -> " BIRTHDAY ", " TO " -> " TO ", " YOU " -> " YOU "))
        intercept[TestFailedException] {
          (all (tojavaMaps) should (not contain oneOf (" HAPPY " -> " HAPPY ", " BIRTHDAY " -> " BIRTHDAY ", " TO " -> " TO ", " YOU " -> " YOU "))) (after being lowerCased and trimmed)
        }
      }
    }
  }
}
