/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Normalization
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class JavaMapShouldContainOnlyLogicalAndSpec extends Spec with Matchers {
  
  val invertedjavaMapOfStringEquality = 
    new Equality[java.util.Map[String, String]] {
      def areEqual(a: java.util.Map[String, String], b: Any): Boolean = a != b
    }
  
  val upperCaseStringEquality =
    new Equality[(String, String)] {
      def areEqual(a: (String, String), b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: Map[_, _] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case _ => value
    }
  
  val upperCasejavaMapOfStringEquality = 
    new Equality[java.util.Map[String, String]] {
      def areEqual(a: java.util.Map[String, String], b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  val lowerCased: Normalization[(String, String)] = new Normalization[(String, String)] {
    def normalized(s: (String, String)): (String, String) = (s._1.toLowerCase, s._2.toLowerCase)
    def normalizedIfInstanceOfA(b: Any) =
      b match {
        case (s1: String, s2: String) => normalized((s1, s2))
        case _ => b
      }
    }
val trimmed: Normalization[(String, String)] = new Normalization[(String, String)] {
    def normalized(s: (String, String)): (String, String) = (s._1.trim, s._2.trim)
    def normalizedIfInstanceOfA(b: Any) =
      b match {
        case (s1: String, s2: String) => normalized((s1, s2))
        case _ => b
      }
    }
def javaMap[K, V](elements: (K, V)*): java.util.LinkedHashMap[K, V] = {
    val m = new java.util.LinkedHashMap[K, V]
    elements.foreach(e => m.put(e._1, e._2))
    m
  }
  
  val fileName: String = "JavaMapShouldContainOnlyLogicalAndSpec.scala"
  
  object `a javaMap` {
    
    val fumjavaMap: java.util.Map[String, String] = javaMap("fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee")
    val tojavaMap: java.util.Map[String, String] = javaMap("you" -> "you", "to" -> "to", "birthday" -> "birthday", "happy" -> "happy")
    
    object `when used with (contain only (..) and contain only (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (contain only ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") and contain only ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain only ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you") and contain only ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOnlyElements", decorateToStringValue(fumjavaMap), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (contain only ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") and contain only ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        }
        checkMessageStackDepth(e2, Resources("containedOnlyElements", decorateToStringValue(fumjavaMap), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)") + ", but " + Resources("didNotContainOnlyElements", decorateToStringValue(fumjavaMap), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
      }

      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (contain only ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") and contain only ("FEE" -> "FEE", "FIE" -> "FIE", "FUM" -> "FUM", "FOE" -> "FOE"))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain only ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FAM" -> "FAM") and contain only ("FEE" -> "FEE", "FIE" -> "FIE", "FUM" -> "FUM", "FOE" -> "FOE"))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOnlyElements", decorateToStringValue(fumjavaMap), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FAM,FAM)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (contain only ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") and (contain only ("FEE" -> "FEE", "FIE" -> "FIE", "FAM" -> "FAM", "FOE" -> "FOE")))
        }
        checkMessageStackDepth(e2, Resources("containedOnlyElements", decorateToStringValue(fumjavaMap), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)") + ", but " + Resources("didNotContainOnlyElements", decorateToStringValue(fumjavaMap), "(FEE,FEE), (FIE,FIE), (FAM,FAM), (FOE,FOE)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (contain only ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") and contain only ("FEE" -> "FEE", "FIE" -> "FIE", "FUM" -> "FUM", "FOE" -> "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (contain only ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FAM" -> "FAM") and contain only ("FEE" -> "FEE", "FIE" -> "FIE", "FUM" -> "FUM", "FOE" -> "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainOnlyElements", decorateToStringValue(fumjavaMap), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FAM,FAM)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaMap should (contain only ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM") and contain only ("FEE" -> "FEE", "FIE" -> "FIE", "FAM" -> "FAM", "FOE" -> "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedOnlyElements", decorateToStringValue(fumjavaMap), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FUM,FUM)") + ", but " + Resources("didNotContainOnlyElements", decorateToStringValue(fumjavaMap), "(FEE,FEE), (FIE,FIE), (FAM,FAM), (FOE,FOE)"), fileName, thisLineNumber - 2)
        (fumjavaMap should (contain only (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM ") and contain only (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (equal (..) and contain only (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (equal (fumjavaMap) and contain only ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (equal (tojavaMap) and contain only ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (equal (fumjavaMap) and contain only ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", but " + Resources("didNotContainOnlyElements", decorateToStringValue(fumjavaMap), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (equal (fumjavaMap) and contain only ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (equal (tojavaMap) and contain only ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (equal (fumjavaMap) and (contain only ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FAM" -> "FAM")))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", but " + Resources("didNotContainOnlyElements", decorateToStringValue(fumjavaMap), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FAM,FAM)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (equal (tojavaMap) and contain only ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (equal (tojavaMap) and contain only ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FAM" -> "FAM"))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", but " + Resources("didNotContainOnlyElements", decorateToStringValue(fumjavaMap), "(FEE,FEE), (FIE,FIE), (FOE,FOE), (FAM,FAM)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaMap should (equal (fumjavaMap) and contain only ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)), fileName, thisLineNumber - 2)
        (fumjavaMap should (equal (tojavaMap) and contain only (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))) (decided by invertedjavaMapOfStringEquality, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (legacyEqual (..) and contain only (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (legacyEqual (fumjavaMap) and contain only ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (legacyEqual (tojavaMap) and contain only ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (legacyEqual (fumjavaMap) and contain only ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", but " + Resources("didNotContainOnlyElements", decorateToStringValue(fumjavaMap), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (legacyEqual (fumjavaMap) and contain only ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (legacyEqual (tojavaMap) and contain only ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (legacyEqual (fumjavaMap) and (contain only ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you")))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", but " + Resources("didNotContainOnlyElements", decorateToStringValue(fumjavaMap), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (legacyEqual (fumjavaMap) and contain only ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (legacyEqual (fumjavaMap) and contain only ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", but " + Resources("didNotContainOnlyElements", decorateToStringValue(fumjavaMap), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaMap should (legacyEqual (tojavaMap) and contain only ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
        (fumjavaMap should (legacyEqual (fumjavaMap) and contain only (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))) (after being lowerCased and trimmed)
      }
    }

    object `when used with (contain only (..) and legacyEqual (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (contain only ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe") and legacyEqual (fumjavaMap))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain only ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") and legacyEqual (tojavaMap))
        }
        checkMessageStackDepth(e1, Resources("containedOnlyElements", decorateToStringValue(fumjavaMap), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)") + ", but " + Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (contain only ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you") and legacyEqual (fumjavaMap))
        }
        checkMessageStackDepth(e2, Resources("didNotContainOnlyElements", decorateToStringValue(fumjavaMap), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (contain only ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE") and legacyEqual (fumjavaMap))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (contain only ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE") and legacyEqual (tojavaMap))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOnlyElements", decorateToStringValue(fumjavaMap), "(FIE,FIE), (FEE,FEE), (FAM,FAM), (FOE,FOE)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (contain only ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU") and (legacyEqual (fumjavaMap)))
        }
        checkMessageStackDepth(e2, Resources("didNotContainOnlyElements", decorateToStringValue(fumjavaMap), "(HAPPY,HAPPY), (BIRTHDAY,BIRTHDAY), (TO,TO), (YOU,YOU)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (contain only ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE") and legacyEqual (fumjavaMap))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (contain only ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE") and legacyEqual (fumjavaMap))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainOnlyElements", decorateToStringValue(fumjavaMap), "(FIE,FIE), (FEE,FEE), (FAM,FAM), (FOE,FOE)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaMap should (contain only ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE") and legacyEqual (tojavaMap))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedOnlyElements", decorateToStringValue(fumjavaMap), "(FIE,FIE), (FEE,FEE), (FUM,FUM), (FOE,FOE)") + ", but " + Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
        (fumjavaMap should (contain only (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM ") and legacyEqual (fumjavaMap))) (after being lowerCased and trimmed)
      }
    }
    
    object `when used with (not contain only xx and not contain only xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (not contain only ("fee", "fie", "foe", "fuu") and not contain only ("fie", "fee", "fuu", "foe"))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not contain only ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") and not contain only ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        }
        checkMessageStackDepth(e1, Resources("containedOnlyElements", decorateToStringValue(fumjavaMap), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (not contain only ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you") and not contain only ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e2, Resources("didNotContainOnlyElements", decorateToStringValue(fumjavaMap), "(happy,happy), (birthday,birthday), (to,to), (you,you)") + ", but " + Resources("containedOnlyElements", decorateToStringValue(fumjavaMap), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (not contain only ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE") and not contain only ("FIE" -> "FIE", "FEE" -> "FEE", "FOE" -> "FOE", "FAM" -> "FAM"))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not contain only ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE") and not contain only ("FIE" -> "FIE", "FEE" -> "FEE", "FOE" -> "FOE", "FAM" -> "FAM"))
        }
        checkMessageStackDepth(e1, Resources("containedOnlyElements", decorateToStringValue(fumjavaMap), "(FIE,FIE), (FEE,FEE), (FUM,FUM), (FOE,FOE)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (not contain only ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE") and (not contain only ("FIE" -> "FIE", "FEE" -> "FEE", "FOE" -> "FOE", "FUM" -> "FUM")))
        }
        checkMessageStackDepth(e2, Resources("didNotContainOnlyElements", decorateToStringValue(fumjavaMap), "(FIE,FIE), (FEE,FEE), (FAM,FAM), (FOE,FOE)") + ", but " + Resources("containedOnlyElements", decorateToStringValue(fumjavaMap), "(FIE,FIE), (FEE,FEE), (FOE,FOE), (FUM,FUM)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (not contain only ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE") and not contain only ("FIE" -> "FIE", "FEE" -> "FEE", "FOE" -> "FOE", "FAM" -> "FAM"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (not contain only ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE") and not contain only ("FIE" -> "FIE", "FEE" -> "FEE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainOnlyElements", decorateToStringValue(fumjavaMap), "(FIE,FIE), (FEE,FEE), (FAM,FAM), (FOE,FOE)") + ", but " + Resources("containedOnlyElements", decorateToStringValue(fumjavaMap), "(FIE,FIE), (FEE,FEE), (FOE,FOE), (FUM,FUM)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaMap should (not contain only ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE") and not contain only ("FIE" -> "FIE", "FEE" -> "FEE", "FOE" -> "FOE", "FAM" -> "FAM"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedOnlyElements", decorateToStringValue(fumjavaMap), "(FIE,FIE), (FEE,FEE), (FUM,FUM), (FOE,FOE)"), fileName, thisLineNumber - 2)
        (fumjavaMap should (contain only (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM ") and contain only (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (not equal (..) and not contain only (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (not equal (tojavaMap) and not contain only ("fie", "fee", "fuu", "foe"))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not equal (fumjavaMap) and not contain only ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (not equal (tojavaMap) and not contain only ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", but " + Resources("containedOnlyElements", decorateToStringValue(fumjavaMap), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (not equal (tojavaMap) and not contain only ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not equal (fumjavaMap) and not contain only ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (not equal (tojavaMap) and (not contain only ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE")))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", but " + Resources("containedOnlyElements", decorateToStringValue(fumjavaMap), "(FIE,FIE), (FEE,FEE), (FUM,FUM), (FOE,FOE)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (not equal (fumjavaMap) and not contain only ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (not equal (fumjavaMap) and not contain only ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE"))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)) + ", but " + Resources("containedOnlyElements", decorateToStringValue(fumjavaMap), "(FIE,FIE), (FEE,FEE), (FUM,FUM), (FOE,FOE)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaMap should (not equal (tojavaMap) and not contain only ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))) (decided by invertedjavaMapOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)), fileName, thisLineNumber - 2)
        (fumjavaMap should (not contain only (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUU " -> " FUU ") and not contain only (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUU " -> " FUU "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (not be (..) and not contain only (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumjavaMap should (not be (tojavaMap) and not contain only ("fie", "fee", "fuu", "foe"))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not be (fumjavaMap) and not contain only ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (not be (tojavaMap) and not contain only ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", but " + Resources("containedOnlyElements", decorateToStringValue(fumjavaMap), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumjavaMap should (not be (tojavaMap) and not contain only ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))
        val e1 = intercept[TestFailedException] {
          fumjavaMap should (not be (fumjavaMap) and not contain only ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumjavaMap should (not be (tojavaMap) and (not contain only ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE")))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", but " + Resources("containedOnlyElements", decorateToStringValue(fumjavaMap), "(FIE,FIE), (FEE,FEE), (FUM,FUM), (FOE,FOE)"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumjavaMap should (not be (tojavaMap) and not contain only ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumjavaMap should (not be (tojavaMap) and not contain only ("FIE" -> "FIE", "FEE" -> "FEE", "FUM" -> "FUM", "FOE" -> "FOE"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasNotEqualTo", decorateToStringValue(fumjavaMap), decorateToStringValue(tojavaMap)) + ", but " + Resources("containedOnlyElements", decorateToStringValue(fumjavaMap), "(FIE,FIE), (FEE,FEE), (FUM,FUM), (FOE,FOE)"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumjavaMap should (not be (fumjavaMap) and not contain only ("FIE" -> "FIE", "FEE" -> "FEE", "FAM" -> "FAM", "FOE" -> "FOE"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("wasEqualTo", decorateToStringValue(fumjavaMap), decorateToStringValue(fumjavaMap)), fileName, thisLineNumber - 2)
        (fumjavaMap should (not contain only (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUU " -> " FUU ") and not contain only (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUU " -> " FUU "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
  }
  
  object `col of javaMaps` {
    
    val list1s: Vector[java.util.Map[Int, Int]] = Vector(javaMap(3 -> 3, 2 -> 2, 1 -> 1), javaMap(3 -> 3, 2 -> 2, 1 -> 1), javaMap(3 -> 3, 2 -> 2, 1 -> 1))
    val lists: Vector[java.util.Map[Int, Int]] = Vector(javaMap(3 -> 3, 2 -> 2, 1 -> 1), javaMap(3 -> 3, 2 -> 2, 1 -> 1), javaMap(4 -> 4, 3 -> 3, 2 -> 2))
    val nils: Vector[java.util.Map[Int, Int]] = Vector(javaMap(), javaMap(), javaMap())
    val listsMap: Vector[java.util.Map[Int, Int]] = Vector(javaMap(3 -> 3, 2 -> 2, 1 -> 1), javaMap(3 -> 3, 2 -> 2, 1 -> 1), javaMap())
    val hijavaMaps: Vector[java.util.Map[String, String]] = Vector(javaMap("hi" -> "hi", "hello" -> "hello"), javaMap("hi" -> "hi", "hello" -> "hello"), javaMap("hi" -> "hi", "hello" -> "hello"))
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + decorateToStringValue(left)
    
    object `used with contain only xx and contain only xx` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain only (3 -> 3, 2 -> 2, 1 -> 1) and contain only (1 -> 1, 3 -> 3, 2 -> 2))
        atLeast (2, lists) should (contain only (3 -> 3, 1 -> 1, 2 -> 2) and contain only (2 -> 2, 3 -> 3, 1 -> 1))
        atMost (2, lists) should (contain only (3 -> 3, 1 -> 1, 2 -> 2) and contain only (2 -> 2, 3 -> 3, 1 -> 1))
        no (lists) should (contain only (3 -> 3, 6 -> 6, 9 -> 9) and contain only (3 -> 3, 4 -> 4, 5 -> 5))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain only (1 -> 1, 2 -> 2, 3 -> 3) and contain only (1 -> 1, 3 -> 3, 2 -> 2))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(javaMap(4 -> 4, 3 -> 3, 2 -> 2)) + " did not contain only " + "((1,1), (2,2), (3,3))", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (list1s) should (contain only (1 -> 1, 2 -> 2, 3 -> 3) and contain only (1 -> 1, 3 -> 3, 4 -> 4))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaMap(3 -> 3, 2 -> 2, 1 -> 1)) + " contained only " + "((1,1), (2,2), (3,3))" + ", but " + decorateToStringValue(javaMap(3 -> 3, 2 -> 2, 1 -> 1)) + " did not contain only " + "((1,1), (3,3), (4,4))", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hijavaMaps) should (contain only ("hi" -> "hi", "hello" -> "hello") and contain only ("ho" -> "ho", "hey" -> "hey", "howdy" -> "howdy"))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(javaMap("hi" -> "hi", "hello" -> "hello")) + " contained only " + "((hi,hi), (hello,hello))" + ", but " + decorateToStringValue(javaMap("hi" -> "hi", "hello" -> "hello")) + " did not contain only " + "((ho,ho), (hey,hey), (howdy,howdy))", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaMaps) should (contain only ("HELLO" -> "HELLO", "HI" -> "HI") and contain only ("HI" -> "HI", "HELLO" -> "HELLO"))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaMaps) should (contain only ("HO" -> "HO", "HELLO" -> "HELLO") and contain only ("HI" -> "HI", "HELLO" -> "HELLO"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap("hi" -> "hi", "hello" -> "hello")) + " did not contain only " + "((HO,HO), (HELLO,HELLO))", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hijavaMaps) should (contain only ("HELLO" -> "HELLO", "HI" -> "HI") and contain only ("HO" -> "HO", "HELLO" -> "HELLO"))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaMap("hi" -> "hi", "hello" -> "hello")) + " contained only " + "((HELLO,HELLO), (HI,HI))" + ", but " + decorateToStringValue(javaMap("hi" -> "hi", "hello" -> "hello")) + " did not contain only " + "((HO,HO), (HELLO,HELLO))", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaMaps) should (contain only ("HELLO" -> "HELLO", "HI" -> "HI") and contain only ("HI" -> "HI", "HELLO" -> "HELLO"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hijavaMaps) should (contain only ("HO" -> "HO", "HELLO" -> "HELLO") and contain only ("HI" -> "HI", "HELLO" -> "HELLO"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap("hi" -> "hi", "hello" -> "hello")) + " did not contain only " + "((HO,HO), (HELLO,HELLO))", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hijavaMaps) should (contain only ("HELLO" -> "HELLO", "HI" -> "HI") and contain only ("HO" -> "HO", "HELLO" -> "HELLO"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaMap("hi" -> "hi", "hello" -> "hello")) + " contained only " + "((HELLO,HELLO), (HI,HI))" + ", but " + decorateToStringValue(javaMap("hi" -> "hi", "hello" -> "hello")) + " did not contain only " + "((HO,HO), (HELLO,HELLO))", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (be (..) and contain only (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (be (javaMap(3 -> 3, 2 -> 2, 1 -> 1)) and contain only (1 -> 1, 3 -> 3, 2 -> 2))
        atLeast (2, lists) should (be (javaMap(3 -> 3, 2 -> 2, 1 -> 1)) and contain only (1 -> 1, 3 -> 3, 2 -> 2))
        atMost (2, lists) should (be (javaMap(3 -> 3, 2 -> 2, 1 -> 1)) and contain only (2 -> 2, 3 -> 3, 1 -> 1))
        no (lists) should (be (javaMap(3 -> 3, 6 -> 6, 9 -> 9)) and contain only (3 -> 3, 4 -> 4, 5 -> 5))
        no (nils) should (be (javaMap(1 -> 1, 6 -> 6, 8 -> 8)) and contain only (1 -> 1, 3 -> 3, 4 -> 4))
        no (listsMap) should (be (javaMap(2 -> 2, 6 -> 6, 8 -> 8)) and contain only (3 -> 3, 4 -> 4, 5 -> 5))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (be (javaMap(3 -> 3, 2 -> 2, 1 -> 1)) and contain only (1 -> 1, 3 -> 3, 2 -> 2))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(javaMap(4 -> 4, 3 -> 3, 2 -> 2)) + " was not equal to " + decorateToStringValue(javaMap(3 -> 3, 2 -> 2, 1 -> 1)), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (list1s) should (be (javaMap(3 -> 3, 2 -> 2, 1 -> 1)) and contain only (2 -> 2, 3 -> 3, 8 -> 8))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaMap(3 -> 3, 2 -> 2, 1 -> 1)) + " was equal to " + decorateToStringValue(javaMap(3 -> 3, 2 -> 2, 1 -> 1)) + ", but " + decorateToStringValue(javaMap(3 -> 3, 2 -> 2, 1 -> 1)) + " did not contain only " + "((2,2), (3,3), (8,8))", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (nils) should (be (javaMap("hey" -> "hey")) and contain only ("hello" -> "hello", "hi" -> "hi"))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(javaMap()) + " was not equal to " + decorateToStringValue(javaMap("hey" -> "hey")), thisLineNumber - 2, nils), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hijavaMaps) should (be (javaMap("hi" -> "hi", "hello" -> "hello")) and contain only ("ho" -> "ho", "hey" -> "hey", "howdy" -> "howdy"))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(javaMap("hi" -> "hi", "hello" -> "hello")) + " was equal to " + decorateToStringValue(javaMap("hi" -> "hi", "hello" -> "hello")) + ", but " + decorateToStringValue(javaMap("hi" -> "hi", "hello" -> "hello")) + " did not contain only " + "((ho,ho), (hey,hey), (howdy,howdy))", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e5 = intercept[TestFailedException] {
          all (listsMap) should (be (javaMap(3 -> 3, 2 -> 2, 1 -> 1)) and contain only (1 -> 1, 3 -> 3, 2 -> 2))
        }
        checkMessageStackDepth(e5, allErrMsg(2, decorateToStringValue(javaMap()) + " was not equal to " + decorateToStringValue(javaMap(3 -> 3, 2 -> 2, 1 -> 1)), thisLineNumber - 2, listsMap), fileName, thisLineNumber - 2)
        
        val e6 = intercept[TestFailedException] {
          all (list1s) should (be (javaMap(3 -> 3, 2 -> 2, 1 -> 1)) and contain only (2 -> 2, 3 -> 3, 8 -> 8))
        }
        checkMessageStackDepth(e6, allErrMsg(0, decorateToStringValue(javaMap(3 -> 3, 2 -> 2, 1 -> 1)) + " was equal to " + decorateToStringValue(javaMap(3 -> 3, 2 -> 2, 1 -> 1)) + ", but " + decorateToStringValue(javaMap(3 -> 3, 2 -> 2, 1 -> 1)) + " did not contain only " + "((2,2), (3,3), (8,8))", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaMaps) should (be (javaMap("hi" -> "hi", "hello" -> "hello")) and contain only ("HELLO" -> "HELLO", "HI" -> "HI"))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaMaps) should (be (javaMap("HI" -> "HI", "HELLO" -> "HELLO")) and contain only ("HELLO" -> "HELLO", "HI" -> "HI"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap("hi" -> "hi", "hello" -> "hello")) + " was not equal to " + decorateToStringValue(javaMap("HI" -> "HI", "HELLO" -> "HELLO")), thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hijavaMaps) should (be (javaMap("hi" -> "hi", "hello" -> "hello")) and contain only ("HO" -> "HO", "HELLO" -> "HELLO"))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaMap("hi" -> "hi", "hello" -> "hello")) + " was equal to " + decorateToStringValue(javaMap("hi" -> "hi", "hello" -> "hello")) + ", but " + decorateToStringValue(javaMap("hi" -> "hi", "hello" -> "hello")) + " did not contain only " + "((HO,HO), (HELLO,HELLO))", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaMaps) should (be (javaMap("hi" -> "hi", "hello" -> "hello")) and contain only ("HELLO" -> "HELLO", "HI" -> "HI"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hijavaMaps) should (be (javaMap("HI" -> "HI", "HELLO" -> "HELLO")) and contain only ("HELLO" -> "HELLO", "HI" -> "HI"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap("hi" -> "hi", "hello" -> "hello")) + " was not equal to " + decorateToStringValue(javaMap("HI" -> "HI", "HELLO" -> "HELLO")), thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hijavaMaps) should (be (javaMap("hi" -> "hi", "hello" -> "hello")) and contain only ("HO" -> "HO", "HELLO" -> "HELLO"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaMap("hi" -> "hi", "hello" -> "hello")) + " was equal to " + decorateToStringValue(javaMap("hi" -> "hi", "hello" -> "hello")) + ", but " + decorateToStringValue(javaMap("hi" -> "hi", "hello" -> "hello")) + " did not contain only " + "((HO,HO), (HELLO,HELLO))", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not contain only xx and not contain only xx)` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not contain only (3 -> 3, 2 -> 2, 8 -> 8) and not contain only (8 -> 8, 3 -> 3, 4 -> 4))
        atLeast (2, lists) should (not contain only (3 -> 3, 8 -> 8, 5 -> 5) and not contain only (8 -> 8, 3 -> 3, 4 -> 4))
        atMost (2, lists) should (not contain only (2, 4, 3) and contain only (4 -> 4, 3 -> 3, 2 -> 2))
        no (list1s) should (not contain only (1 -> 1, 2 -> 2, 3 -> 3) and not contain only (1 -> 1, 3 -> 3, 2 -> 2))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain only (2 -> 2, 3 -> 3, 4 -> 4) and not contain only (8 -> 8, 3 -> 3, 4 -> 4))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(javaMap(4 -> 4, 3 -> 3, 2 -> 2)) + " contained only " + "((2,2), (3,3), (4,4))", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not contain only (3 -> 3, 6 -> 6, 8 -> 8) and not contain only (2 -> 2, 3 -> 3, 4 -> 4))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(javaMap(4 -> 4, 3 -> 3, 2 -> 2)) + " did not contain only " + "((3,3), (6,6), (8,8))" + ", but " + decorateToStringValue(javaMap(4 -> 4, 3 -> 3, 2 -> 2)) + " contained only " + "((2,2), (3,3), (4,4))", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hijavaMaps) should (not contain only ("hello" -> "hello", "hi" -> "hi") and not contain only ("ho" -> "ho", "hey" -> "hey", "howdy" -> "howdy"))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(javaMap("hi" -> "hi", "hello" -> "hello")) + " contained only " + "((hello,hello), (hi,hi))", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hijavaMaps) should (not contain only ("ho" -> "ho", "hey" -> "hey", "howdy" -> "howdy") and not contain only ("hello" -> "hello", "hi" -> "hi"))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(javaMap("hi" -> "hi", "hello" -> "hello")) + " did not contain only " + "((ho,ho), (hey,hey), (howdy,howdy))" + ", but " + decorateToStringValue(javaMap("hi" -> "hi", "hello" -> "hello")) + " contained only " + "((hello,hello), (hi,hi))", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaMaps) should (not contain only ("HI" -> "HI") and not contain only ("HO" -> "HO"))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaMaps) should (not contain only ("HELLO" -> "HELLO", "HI" -> "HI") and not contain only ("HO" -> "HO"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap("hi" -> "hi", "hello" -> "hello")) + " contained only " + "((HELLO,HELLO), (HI,HI))", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hijavaMaps) should (not contain only ("HI" -> "HI") and not contain only ("HELLO" -> "HELLO", "HI" -> "HI"))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaMap("hi" -> "hi", "hello" -> "hello")) + " did not contain only " + "((HI,HI))" + ", but " + decorateToStringValue(javaMap("hi" -> "hi", "hello" -> "hello")) + " contained only " + "((HELLO,HELLO), (HI,HI))", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaMaps) should (not contain only ("HI" -> "HI") and not contain only ("HO" -> "HO"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hijavaMaps) should (not contain only ("HELLO" -> "HELLO", "HI" -> "HI") and not contain only ("HO" -> "HO"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap("hi" -> "hi", "hello" -> "hello")) + " contained only " + "((HELLO,HELLO), (HI,HI))", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hijavaMaps) should (not contain only ("HI" -> "HI") and not contain only ("HELLO" -> "HELLO", "HI" -> "HI"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaMap("hi" -> "hi", "hello" -> "hello")) + " did not contain only " + "((HI,HI))" + ", but " + decorateToStringValue(javaMap("hi" -> "hi", "hello" -> "hello")) + " contained only " + "((HELLO,HELLO), (HI,HI))", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not be (..) and not contain only (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not be (javaMap(2 -> 2)) and not contain only (8 -> 8, 3 -> 3, 4 -> 4))
        atLeast (2, lists) should (not be (javaMap(3 -> 3)) and not contain only (8 -> 8, 3 -> 3, 4 -> 4))
        atMost (2, lists) should (not be (javaMap(4 -> 4, 3 -> 3, 2 -> 2)) and not contain only (3 -> 3, 4 -> 4, 2 -> 2))
        no (list1s) should (not be (javaMap(3 -> 3, 2 -> 2, 1 -> 1)) and not contain only (1 -> 1, 2 -> 2, 3 -> 3))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not be (javaMap(4 -> 4, 3 -> 3, 2 -> 2)) and not contain only (8 -> 8, 3 -> 3, 4 -> 4))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(javaMap(4 -> 4, 3 -> 3, 2 -> 2)) + " was equal to " + decorateToStringValue(javaMap(4 -> 4, 3 -> 3, 2 -> 2)), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not be (javaMap(3 -> 3)) and not contain only (2 -> 2, 3 -> 3, 4 -> 4))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(javaMap(4 -> 4, 3 -> 3, 2 -> 2)) + " was not equal to " + decorateToStringValue(javaMap(3 -> 3)) + ", but " + decorateToStringValue(javaMap(4 -> 4, 3 -> 3, 2 -> 2)) + " contained only " + "((2,2), (3,3), (4,4))", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hijavaMaps) should (not be (javaMap("hi" -> "hi", "hello" -> "hello")) and not contain only ("ho" -> "ho", "hey" -> "hey", "howdy" -> "howdy"))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(javaMap("hi" -> "hi", "hello" -> "hello")) + " was equal to " + decorateToStringValue(javaMap("hi" -> "hi", "hello" -> "hello")), thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hijavaMaps) should (not be (javaMap("ho" -> "ho")) and not contain only ("hello" -> "hello", "hi" -> "hi"))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(javaMap("hi" -> "hi", "hello" -> "hello")) + " was not equal to " + decorateToStringValue(javaMap("ho" -> "ho")) + ", but " + decorateToStringValue(javaMap("hi" -> "hi", "hello" -> "hello")) + " contained only " + "((hello,hello), (hi,hi))", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hijavaMaps) should (not be (javaMap("ho" -> "ho")) and not contain only ("HO" -> "HO", "HELLO" -> "HELLO"))
        
        val e1 = intercept[TestFailedException] {
          all (hijavaMaps) should (not be (javaMap("hi" -> "hi", "hello" -> "hello")) and not contain only ("HELLO" -> "HELLO", "HI" -> "HI"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap("hi" -> "hi", "hello" -> "hello")) + " was equal to " + decorateToStringValue(javaMap("hi" -> "hi", "hello" -> "hello")), thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hijavaMaps) should (not be (javaMap("ho" -> "ho")) and not contain only ("HI" -> "HI", "HELLO" -> "HELLO"))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaMap("hi" -> "hi", "hello" -> "hello")) + " was not equal to " + decorateToStringValue(javaMap("ho" -> "ho")) + ", but " + decorateToStringValue(javaMap("hi" -> "hi", "hello" -> "hello")) + " contained only " + "((HI,HI), (HELLO,HELLO))", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hijavaMaps) should (not be (javaMap("ho" -> "ho")) and not contain only ("HO" -> "HO", "HELLO" -> "HELLO"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hijavaMaps) should (not be (javaMap("hi" -> "hi", "hello" -> "hello")) and not contain only ("HELLO" -> "HELLO", "HI" -> "HI"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(javaMap("hi" -> "hi", "hello" -> "hello")) + " was equal to " + decorateToStringValue(javaMap("hi" -> "hi", "hello" -> "hello")), thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hijavaMaps) should (not be (javaMap("ho" -> "ho")) and not contain only ("HI" -> "HI", "HELLO" -> "HELLO"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(javaMap("hi" -> "hi", "hello" -> "hello")) + " was not equal to " + decorateToStringValue(javaMap("ho" -> "ho")) + ", but " + decorateToStringValue(javaMap("hi" -> "hi", "hello" -> "hello")) + " contained only " + "((HI,HI), (HELLO,HELLO))", thisLineNumber - 2, hijavaMaps), fileName, thisLineNumber - 2)
      }
    }
  }
}
