/*
 * Copyright 2001-2013 Artima, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Normalization
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class MapShouldContainAllOfSpec extends Spec with Matchers {
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: Map[_, _] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case _ => value
    }
  
  val upperCaseStringEquality =
    new Equality[(String, String)] {
      def areEqual(a: (String, String), b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  val lowerCased: Normalization[(String, String)] = new Normalization[(String, String)] {
    def normalized(s: (String, String)): (String, String) = (s._1.toLowerCase, s._2.toLowerCase)
    def normalizedIfInstanceOfA(b: Any) =
      b match {
        case (s1: String, s2: String) => normalized((s1, s2))
        case _ => b
      }
    }
val trimmed: Normalization[(String, String)] = new Normalization[(String, String)] {
    def normalized(s: (String, String)): (String, String) = (s._1.trim, s._2.trim)
    def normalizedIfInstanceOfA(b: Any) =
      b match {
        case (s1: String, s2: String) => normalized((s1, s2))
        case _ => b
      }
    }

  object `a List` {

    val fumList: scala.collection.mutable.LinkedHashMap[String, String] = scala.collection.mutable.LinkedHashMap("fex" -> "fex", "fum" -> "fum", "foe" -> "foe", "fie" -> "fie", "fee" -> "fee")
    val toList: scala.collection.mutable.LinkedHashMap[String, String] = scala.collection.mutable.LinkedHashMap("too" -> "too", "you" -> "you", "to" -> "to", "birthday" -> "birthday", "happy" -> "happy")

    object `when used with contain allOf (..)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should contain allOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")
        val e1 = intercept[TestFailedException] {
          fumList should contain allOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you")
        }
        e1.failedCodeFileName.get should be ("MapShouldContainAllOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainAllOfElements", decorateToStringValue(fumList), "(happy,happy), (birthday,birthday), (to,to), (you,you)"))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should contain allOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM")
        intercept[TestFailedException] {
          fumList should contain allOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fam" -> "fam")
        }
      }
      def `should use an explicitly provided Equality` {
        (fumList should contain allOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM")) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (fumList should contain allOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fam" -> "fam")) (decided by upperCaseStringEquality)
        }
        intercept[TestFailedException] {
          fumList should contain allOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM ")
        }
        (fumList should contain allOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM ")) (after being lowerCased and trimmed)
      }
    }

    object `when used with (contain allOf (..))` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {

        fumList should (contain allOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        val e1 = intercept[TestFailedException] {
          fumList should (contain allOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        }
        e1.failedCodeFileName.get should be ("MapShouldContainAllOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainAllOfElements", decorateToStringValue(fumList), "(happy,happy), (birthday,birthday), (to,to), (you,you)"))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (contain allOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))
        intercept[TestFailedException] {
          fumList should (contain allOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fam" -> "fam"))
        }
      }
      def `should use an explicitly provided Equality` {
        (fumList should (contain allOf ("FEE" -> "FEE", "FIE" -> "FIE", "FOE" -> "FOE", "FUM" -> "FUM"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (fumList should (contain allOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fam" -> "fam"))) (decided by upperCaseStringEquality)
        }
        intercept[TestFailedException] {
          fumList should (contain allOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))
        }
        (fumList should (contain allOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))) (after being lowerCased and trimmed)
      }
    }

    object `when used with not contain allOf (..)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        toList should not contain allOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")
        val e1 = intercept[TestFailedException] {
          toList should not contain allOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you")
        }
        e1.failedCodeFileName.get should be ("MapShouldContainAllOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedAllOfElements", decorateToStringValue(toList), "(happy,happy), (birthday,birthday), (to,to), (you,you)"))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        toList should not contain allOf ("happy", "birthday", "to", "you", "dear")
        intercept[TestFailedException] {
          toList should not contain allOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU")
        }
      }
      def `should use an explicitly provided Equality` {
        (toList should not contain allOf ("happy", "birthday", "to", "you", "dear")) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (toList should not contain allOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU")) (decided by upperCaseStringEquality)
        }
        toList should not contain allOf (" HAPPY " -> " HAPPY ", " BIRTHDAY " -> " BIRTHDAY ", " TO " -> " TO ", " YOU " -> " YOU ")
        intercept[TestFailedException] {
          (toList should not contain allOf (" HAPPY " -> " HAPPY ", " BIRTHDAY " -> " BIRTHDAY ", " TO " -> " TO ", " YOU " -> " YOU ")) (after being lowerCased and trimmed)
        }
      }
    }

    object `when used with (not contain allOf (..))` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        toList should (not contain allOf ("HAPPY", "BIRTHDAY", "TO", "YOU", "DEAR"))
        val e1 = intercept[TestFailedException] {
          toList should (not contain allOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        }
        e1.failedCodeFileName.get should be ("MapShouldContainAllOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedAllOfElements", decorateToStringValue(toList), "(happy,happy), (birthday,birthday), (to,to), (you,you)"))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        toList should (not contain allOf ("NICE", "TO", "MEET", "YOU", "TOO"))
        intercept[TestFailedException] {
          toList should (not contain allOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU"))
        }
      }
      def `should use an explicitly provided Equality` {
        (toList should (not contain allOf ("NICE", "TO", "MEET", "YOU", "TOO"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (toList should (not contain allOf ("HAPPY" -> "HAPPY", "BIRTHDAY" -> "BIRTHDAY", "TO" -> "TO", "YOU" -> "YOU"))) (decided by upperCaseStringEquality)
        }
        toList should (not contain allOf (" HAPPY " -> " HAPPY ", " BIRTHDAY " -> " BIRTHDAY ", " TO " -> " TO ", " YOU " -> " YOU "))
        intercept[TestFailedException] {
          (toList should (not contain allOf (" HAPPY " -> " HAPPY ", " BIRTHDAY " -> " BIRTHDAY ", " TO " -> " TO ", " YOU " -> " YOU "))) (after being lowerCased and trimmed)
        }
      }
    }
  }

  object `a col of Lists` {

    val list1s: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(3 -> 3, 2 -> 2, 1 -> 1, 0 -> 0), scala.collection.mutable.LinkedHashMap(3 -> 3, 2 -> 2, 1 -> 1, 0 -> 0), scala.collection.mutable.LinkedHashMap(3 -> 3, 2 -> 2, 1 -> 1, 0 -> 0))
    val lists: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(3 -> 3, 2 -> 2, 1 -> 1, 0 -> 0), scala.collection.mutable.LinkedHashMap(3 -> 3, 2 -> 2, 1 -> 1, 0 -> 0), scala.collection.mutable.LinkedHashMap(8 -> 8, 4 -> 4, 3 -> 3, 2 -> 2))
    val listsMap: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(3 -> 3, 2 -> 2, 1 -> 1, 0 -> 0), scala.collection.mutable.LinkedHashMap(3 -> 3, 2 -> 2, 1 -> 1, 0 -> 0), scala.collection.mutable.LinkedHashMap())
    val hiLists: Vector[scala.collection.mutable.LinkedHashMap[String, String]] = Vector(scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "he" -> "he"), scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "he" -> "he"), scala.collection.mutable.LinkedHashMap("howdy" -> "howdy", "hi" -> "hi", "he" -> "he"))
    val toLists: Vector[scala.collection.mutable.LinkedHashMap[String, String]] = Vector(scala.collection.mutable.LinkedHashMap("happy" -> "happy", "to" -> "to", "you" -> "you"), scala.collection.mutable.LinkedHashMap("happy" -> "happy", "to" -> "to", "you" -> "you"), scala.collection.mutable.LinkedHashMap("happy" -> "happy", "to" -> "to", "you" -> "you"))

    object `when used with contain allOf (..)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should contain allOf (1 -> 1, 2 -> 2, 3 -> 3)
        atLeast (2, lists) should contain allOf (1 -> 1, 2 -> 2, 3 -> 3)
        atMost (2, lists) should contain allOf (1 -> 1, 2 -> 2, 3 -> 3)
        no (lists) should contain allOf (3 -> 3, 4 -> 4, 5 -> 5)

        val e1 = intercept[TestFailedException] {
          all (lists) should contain allOf (1 -> 1, 2 -> 2, 3 -> 3)
        }
        e1.failedCodeFileName.get should be ("MapShouldContainAllOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(8 -> 8, 4 -> 4, 3 -> 3, 2 -> 2)) + " did not contain all of " + "((1,1), (2,2), (3,3))" + " (MapShouldContainAllOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))
      }

      def `should use the implicit Equality in scope` {
        all (hiLists) should contain allOf ("he" -> "he", "hi" -> "hi")
        intercept[TestFailedException] {
          all (hiLists) should contain allOf ("ho" -> "ho", "hi" -> "hi")
        }
        implicit val ise = upperCaseStringEquality
        all (hiLists) should contain allOf ("HE" -> "HE", "HI" -> "HI")
        intercept[TestFailedException] {
          all (hiLists) should contain allOf ("HO" -> "HO", "HI" -> "HI")
        }
      }
      def `should use an explicitly provided Equality` {
        (all (hiLists) should contain allOf ("HE" -> "HE", "HI" -> "HI")) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (hiLists) should contain allOf ("HO" -> "HO", "HI" -> "HI")) (decided by upperCaseStringEquality)
        }
        implicit val ise = upperCaseStringEquality
        (all (hiLists) should contain allOf ("he" -> "he", "hi" -> "hi")) (decided by defaultEquality[(String, String)])
        intercept[TestFailedException] {
          (all (hiLists) should contain allOf ("ho" -> "ho", "hi" -> "hi")) (decided by defaultEquality[(String, String)])
        }
      }
    }

    object `when used with (contain allOf (..))` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain allOf (1 -> 1, 2 -> 2, 3 -> 3))
        atLeast (2, lists) should (contain allOf (1 -> 1, 2 -> 2, 3 -> 3))
        atMost (2, lists) should (contain allOf (1 -> 1, 2 -> 2, 3 -> 3))
        no (lists) should (contain allOf (3 -> 3, 4 -> 4, 5 -> 5))

        val e1 = intercept[TestFailedException] {
          all (lists) should (contain allOf (1 -> 1, 2 -> 2, 3 -> 3))
        }
        e1.failedCodeFileName.get should be ("MapShouldContainAllOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(8 -> 8, 4 -> 4, 3 -> 3, 2 -> 2)) + " did not contain all of " + "((1,1), (2,2), (3,3))" + " (MapShouldContainAllOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))
      }

      def `should use the implicit Equality in scope` {
        all (hiLists) should (contain allOf ("he" -> "he", "hi" -> "hi"))
        intercept[TestFailedException] {
          all (hiLists) should (contain allOf ("ho" -> "ho", "hi" -> "hi"))
        }
        implicit val ise = upperCaseStringEquality
        all (hiLists) should (contain allOf ("HE" -> "HE", "HI" -> "HI"))
        intercept[TestFailedException] {
          all (hiLists) should (contain allOf ("HO" -> "HO", "HI" -> "HI"))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (contain allOf ("HE" -> "HE", "HI" -> "HI"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (hiLists) should (contain allOf ("HO" -> "HO", "HI" -> "HI"))) (decided by upperCaseStringEquality)
        }
        implicit val ise = upperCaseStringEquality
        (all (hiLists) should (contain allOf ("he" -> "he", "hi" -> "hi"))) (decided by defaultEquality[(String, String)])
        intercept[TestFailedException] {
          (all (hiLists) should (contain allOf ("ho" -> "ho", "hi" -> "hi"))) (decided by defaultEquality[(String, String)])
        }
      }
    }

    object `when used with not contain allOf (..)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (toLists) should not contain allOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum")
        val e1 = intercept[TestFailedException] {
          all (toLists) should not contain allOf ("you" -> "you", "to" -> "to")
        }
        e1.failedCodeFileName.get should be ("MapShouldContainAllOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("happy" -> "happy", "to" -> "to", "you" -> "you")) + " contained all of " + "((you,you), (to,to))" +  " (MapShouldContainAllOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(toLists)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        all (toLists) should not contain allOf ("NICE", "MEET", "YOU")
        intercept[TestFailedException] {
          all (toLists) should not contain allOf ("YOU" -> "YOU", "TO" -> "TO")
        }
      }
      def `should use an explicitly provided Equality` {
        (all (toLists) should not contain allOf ("NICE", "MEET", "YOU")) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (toLists) should not contain allOf ("YOU" -> "YOU", "TO" -> "TO")) (decided by upperCaseStringEquality)
        }
        all (toLists) should not contain allOf (" YOU " -> " YOU ", " TO " -> " TO ")
        intercept[TestFailedException] {
          (all (toLists) should not contain allOf (" YOU " -> " YOU ", " TO " -> " TO ")) (after being lowerCased and trimmed)
        }
      }
    }

    object `when used with (not contain allOf (..))` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (toLists) should (not contain allOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        val e1 = intercept[TestFailedException] {
          all (toLists) should (not contain allOf ("you" -> "you", "to" -> "to"))
        }
        e1.failedCodeFileName.get should be ("MapShouldContainAllOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("happy" -> "happy", "to" -> "to", "you" -> "you")) + " contained all of " + "((you,you), (to,to))" + " (MapShouldContainAllOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(toLists)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        all (toLists) should (not contain allOf ("NICE", "MEET", "YOU"))
        intercept[TestFailedException] {
          all (toLists) should (not contain allOf ("YOU" -> "YOU", "TO" -> "TO"))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (toLists) should (not contain allOf ("NICE", "MEET", "YOU"))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (toLists) should (not contain allOf ("YOU" -> "YOU", "TO" -> "TO"))) (decided by upperCaseStringEquality)
        }
        all (toLists) should (not contain allOf (" YOU " -> " YOU ", " TO " -> " TO "))
        intercept[TestFailedException] {
          (all (toLists) should (not contain allOf (" YOU " -> " YOU ", " TO " -> " TO "))) (after being lowerCased and trimmed)
        }
      }
    }
  }
}
