/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Normalization
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class MapShouldContainAtLeastOneOfLogicalOrSpec extends FreeSpec with Matchers {
  
  val invertedStringEquality =
    new Equality[(String, String)] {
      def areEqual(a: (String, String), b: Any): Boolean = a != b
    }
  
  val invertedListOfStringEquality = 
    new Equality[scala.collection.mutable.LinkedHashMap[String, String]] {
      def areEqual(a: scala.collection.mutable.LinkedHashMap[String, String], b: Any): Boolean = a != b
    }
  
  val lowerCased: Normalization[(String, String)] = new Normalization[(String, String)] {
    def normalized(s: (String, String)): (String, String) = (s._1.toLowerCase, s._2.toLowerCase)
    def normalizedIfInstanceOfA(b: Any) =
      b match {
        case (s1: String, s2: String) => normalized((s1, s2))
        case _ => b
      }
    }
val trimmed: Normalization[(String, String)] = new Normalization[(String, String)] {
    def normalized(s: (String, String)): (String, String) = (s._1.trim, s._2.trim)
    def normalizedIfInstanceOfA(b: Any) =
      b match {
        case (s1: String, s2: String) => normalized((s1, s2))
        case _ => b
      }
    }
  
  val fileName: String = "MapShouldContainAtLeastOneOfLogicalOrSpec.scala"
  
  "a List" - {
    
    val fumList: scala.collection.mutable.LinkedHashMap[String, String] = scala.collection.mutable.LinkedHashMap("fum" -> "fum")
    val toList: scala.collection.mutable.LinkedHashMap[String, String] = scala.collection.mutable.LinkedHashMap("to" -> "to")
    
      "when used with (contain oneOf (...) or contain oneOf (...)) syntax" - {
      
        "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumList should (contain atLeastOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") or contain atLeastOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        fumList should (contain atLeastOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fam" -> "fam") or contain atLeastOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        fumList should (contain atLeastOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") or contain atLeastOneOf("fie" -> "fie", "fee" -> "fee", "fam" -> "fam", "foe" -> "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (contain atLeastOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fam" -> "fam") or contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fam,fam)") + ", and " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
      }

      "should use the implicit Equality in scope" in {
        implicit val ise = invertedStringEquality
        fumList should (contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you") or contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day"))
        fumList should (contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you") or contain atLeastOneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        fumList should (contain atLeastOneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe") or contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        val e1 = intercept[TestFailedException] {
          fumList should (contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum", "fum" -> "fum") or (contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum")))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "(fum,fum), (fum,fum), (fum,fum), (fum,fum)") + ", and " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "(fum,fum), (fum,fum), (fum,fum)"), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumList should (contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you") or contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        (fumList should (contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you") or contain atLeastOneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        (fumList should (contain atLeastOneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe") or contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum") or contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "(fum,fum), (fum,fum)") + ", and " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "(fum,fum), (fum,fum), (fum,fum)"), fileName, thisLineNumber - 2)
        (fumList should (contain atLeastOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM ") or contain atLeastOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    "when used with (equal (...) and contain oneOf (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumList should (equal (fumList) or contain atLeastOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        fumList should (equal (toList) or contain atLeastOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        fumList should (equal (fumList) or contain atLeastOneOf("fie" -> "fie", "fee" -> "fee", "fam" -> "fam", "foe" -> "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (equal (toList) or contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", and " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = invertedStringEquality
        fumList should (equal (fumList) or contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day"))
        fumList should (equal (toList) or contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day"))
        fumList should (equal (fumList) or contain atLeastOneOf ("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (equal (toList) or (contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", and " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "(fum,fum), (fum,fum), (fum,fum)"), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumList should (equal (toList) or contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day"))) (decided by invertedListOfStringEquality, decided by invertedStringEquality)
        (fumList should (equal (fumList) or contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day"))) (decided by invertedListOfStringEquality, decided by invertedStringEquality)
        (fumList should (equal (toList) or contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum"))) (decided by invertedListOfStringEquality, decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (equal (fumList) or contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum"))) (decided by invertedListOfStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "(fum,fum), (fum,fum), (fum,fum)"), fileName, thisLineNumber - 2)
        (fumList should (equal (toList) or contain atLeastOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))) (decided by invertedListOfStringEquality, after being lowerCased and trimmed)
      }
    }
    
    "when used with (legacyEqual (...) and contain oneOf (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumList should (legacyEqual (fumList) or contain atLeastOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        fumList should (legacyEqual (toList) or contain atLeastOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        fumList should (legacyEqual (fumList) or contain atLeastOneOf("fie" -> "fie", "fee" -> "fee", "fam" -> "fam", "foe" -> "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (legacyEqual (toList) or contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", and " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = invertedStringEquality
        fumList should (legacyEqual (fumList) or contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day"))
        fumList should (legacyEqual (toList) or contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day"))
        fumList should (legacyEqual (fumList) or contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum"))
        val e1 = intercept[TestFailedException] {
          fumList should (legacyEqual (toList) or (contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", and " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "(fum,fum), (fum,fum), (fum,fum)"), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumList should (legacyEqual (fumList) or contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day"))) (decided by invertedStringEquality)
        (fumList should (legacyEqual (toList) or contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day"))) (decided by invertedStringEquality)
        (fumList should (legacyEqual (fumList) or contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum"))) (decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (legacyEqual (toList) or contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum"))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", and " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "(fum,fum), (fum,fum), (fum,fum)"), fileName, thisLineNumber - 2)
        (fumList should (legacyEqual (fumList) or contain atLeastOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))) (after being lowerCased and trimmed)
      }
    }

    "when used with (contain oneOf (...) and legacyEqual (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumList should (contain atLeastOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe") or legacyEqual (fumList))
        fumList should (contain atLeastOneOf("fie" -> "fie", "fee" -> "fee", "fam" -> "fam", "foe" -> "foe") or legacyEqual (fumList))
        fumList should (contain atLeastOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe") or legacyEqual (toList))
        val e1 = intercept[TestFailedException] {
          fumList should (contain atLeastOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fam" -> "fam") or legacyEqual (toList))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fam,fam)") + ", and " + Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = invertedStringEquality
        fumList should (contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day") or legacyEqual (fumList))
        fumList should (contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum") or legacyEqual (fumList))
        fumList should (contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day") or legacyEqual (toList))
        val e1 = intercept[TestFailedException] {
          fumList should (contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum") or legacyEqual (toList))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "(fum,fum), (fum,fum), (fum,fum)") + ", and " + Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumList should (contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day") or legacyEqual (fumList))) (decided by invertedStringEquality)
        (fumList should (contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum") or legacyEqual (fumList))) (decided by invertedStringEquality)
        (fumList should (contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day") or legacyEqual (toList))) (decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum") or legacyEqual (toList))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "(fum,fum), (fum,fum), (fum,fum)") + ", and " + Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        (fumList should (contain atLeastOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM ") or legacyEqual (fumList))) (after being lowerCased and trimmed)
      }
    }
    
    "when used with (not contain oneOf (...) and not contain oneOf (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumList should (not contain atLeastOneOf ("fee", "fie", "foe", "fuu") or not contain atLeastOneOf("fie", "fee", "fuu", "foe"))
        fumList should (not contain atLeastOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") or not contain atLeastOneOf("fie", "fee", "fuu", "foe"))
        fumList should (not contain atLeastOneOf ("fee", "fie", "foe", "fuu") or not contain atLeastOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (not contain atLeastOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum") or not contain atLeastOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e1, Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)") + ", and " + Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = invertedStringEquality
        fumList should (not contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum", "fum" -> "fum") or not contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum"))
        fumList should (not contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you") or not contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum"))
        fumList should (not contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum", "fum" -> "fum") or not contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        val e1 = intercept[TestFailedException] {
          fumList should (not contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you") or not contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day"))
        }
        checkMessageStackDepth(e1, Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "(happy,happy), (birthday,birthday), (to,to), (you,you)") + ", and " + Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "(have,have), (a,a), (nice,nice), (day,day)"), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumList should (not contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum", "fum" -> "fum") or not contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        (fumList should (not contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you") or not contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        (fumList should (not contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum", "fum" -> "fum") or not contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you") or not contain atLeastOneOf ("have" -> "have", "a" -> "a", "nice" -> "nice", "day" -> "day"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "(happy,happy), (birthday,birthday), (to,to), (you,you)") + ", and " + Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "(have,have), (a,a), (nice,nice), (day,day)"), fileName, thisLineNumber - 2)
        (fumList should (contain atLeastOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM ") or contain atLeastOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUM " -> " FUM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    "when used with (not equal (...) and not contain oneOf (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumList should (not equal (toList) or not contain atLeastOneOf("fie", "fee", "fuu", "foe"))
        fumList should (not equal (fumList) or not contain atLeastOneOf("fie", "fee", "fuu", "foe"))
        fumList should (not equal (toList) or not contain atLeastOneOf("fie" -> "fie", "fee" -> "fee", "fum" -> "fum", "foe" -> "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (not equal (fumList) or not contain atLeastOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = invertedStringEquality
        fumList should (not equal (toList) or not contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum"))
        fumList should (not equal (fumList) or not contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum"))
        fumList should (not equal (toList) or not contain atLeastOneOf ("fie", "fee", "fuu", "foe"))
        val e2 = intercept[TestFailedException] {
          fumList should (not equal (fumList) or (not contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you")))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumList should (not equal (fumList) or not contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum"))) (decided by invertedListOfStringEquality, decided by invertedStringEquality)
        (fumList should (not equal (toList) or not contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum"))) (decided by invertedListOfStringEquality, decided by invertedStringEquality)
        (fumList should (not equal (fumList) or not contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))) (decided by invertedListOfStringEquality, decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not equal (toList) or not contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))) (decided by invertedListOfStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", and " + Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
        (fumList should (not contain atLeastOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUU " -> " FUU ") or not contain atLeastOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUU " -> " FUU "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    "when used with (not be (...) and not contain oneOf (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumList should (not be (toList) or not contain atLeastOneOf("fie", "fee", "fuu", "foe"))
        fumList should (not be (fumList) or not contain atLeastOneOf("fie", "fee", "fuu", "foe"))
        fumList should (not be (toList) or not contain atLeastOneOf("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        val e1 = intercept[TestFailedException] {
          fumList should (not be (fumList) or not contain atLeastOneOf ("fee" -> "fee", "fie" -> "fie", "foe" -> "foe", "fum" -> "fum"))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "(fee,fee), (fie,fie), (foe,foe), (fum,fum)"), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = invertedStringEquality
        fumList should (not be (toList) or not contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum"))
        fumList should (not be (fumList) or not contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum"))
        fumList should (not be (toList) or not contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))
        val e1 = intercept[TestFailedException] {
          fumList should (not be (fumList) or (not contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you")))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumList should (not be (toList) or not contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum"))) (decided by invertedStringEquality)
        (fumList should (not be (fumList) or not contain atLeastOneOf ("fum" -> "fum", "fum" -> "fum", "fum" -> "fum"))) (decided by invertedStringEquality)
        (fumList should (not be (toList) or not contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))) (decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not be (fumList) or not contain atLeastOneOf ("happy" -> "happy", "birthday" -> "birthday", "to" -> "to", "you" -> "you"))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "(happy,happy), (birthday,birthday), (to,to), (you,you)"), fileName, thisLineNumber - 2)
        (fumList should (not contain atLeastOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUU " -> " FUU ") or not contain atLeastOneOf (" FEE " -> " FEE ", " FIE " -> " FIE ", " FOE " -> " FOE ", " FUU " -> " FUU "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
  }
  
  "collection of Lists" - {
    
    val list1s: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(1 -> 1), scala.collection.mutable.LinkedHashMap(1 -> 1), scala.collection.mutable.LinkedHashMap(1 -> 1))
    val lists: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(1 -> 1), scala.collection.mutable.LinkedHashMap(1 -> 1), scala.collection.mutable.LinkedHashMap(2 -> 2))
    val nils: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(), scala.collection.mutable.LinkedHashMap(), scala.collection.mutable.LinkedHashMap())
    val listsMap: Vector[scala.collection.mutable.LinkedHashMap[Int, Int]] = Vector(scala.collection.mutable.LinkedHashMap(1 -> 1), scala.collection.mutable.LinkedHashMap(1 -> 1), scala.collection.mutable.LinkedHashMap())
    val hiLists: Vector[scala.collection.mutable.LinkedHashMap[String, String]] = Vector(scala.collection.mutable.LinkedHashMap("hi" -> "hi"), scala.collection.mutable.LinkedHashMap("hi" -> "hi"), scala.collection.mutable.LinkedHashMap("hi" -> "hi"))
    val toLists: Vector[scala.collection.mutable.LinkedHashMap[String, String]] = Vector(scala.collection.mutable.LinkedHashMap("to" -> "to"), scala.collection.mutable.LinkedHashMap("to" -> "to"), scala.collection.mutable.LinkedHashMap("to" -> "to"))
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + left
    
    "when used with (contain oneOf (..) and contain oneOf (..)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        all (list1s) should (contain atLeastOneOf (3 -> 3, 2 -> 2, 1 -> 1) or contain atLeastOneOf (1 -> 1, 3 -> 3, 4 -> 4))
        all (list1s) should (contain atLeastOneOf (3, 2, 5) or contain atLeastOneOf (1 -> 1, 3 -> 3, 4 -> 4))
        all (list1s) should (contain atLeastOneOf (3 -> 3, 2 -> 2, 1 -> 1) or contain atLeastOneOf (2 -> 2, 3 -> 3, 4 -> 4))
        
        atLeast (2, lists) should (contain atLeastOneOf (3 -> 3, 1 -> 1, 5 -> 5) or contain atLeastOneOf (1 -> 1, 3 -> 3, 4 -> 4))
        atLeast (2, lists) should (contain atLeastOneOf (3, 6, 5) or contain atLeastOneOf (1 -> 1, 3 -> 3, 4 -> 4))
        atLeast (2, lists) should (contain atLeastOneOf (3 -> 3, 1 -> 1, 5 -> 5) or contain atLeastOneOf (8 -> 8, 3 -> 3, 4 -> 4))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain atLeastOneOf (6 -> 6, 7 -> 7, 8 -> 8) or contain atLeastOneOf (1 -> 1, 3 -> 3, 4 -> 4))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2)) + " did not contain at least one of ((6,6), (7,7), (8,8)), and " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2)) + " did not contain at least one of ((1,1), (3,3), (4,4))", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = invertedStringEquality
        
        all (hiLists) should (contain atLeastOneOf ("ho" -> "ho") or contain atLeastOneOf ("he" -> "he"))
        all (hiLists) should (contain atLeastOneOf ("hi" -> "hi") or contain atLeastOneOf ("he" -> "he"))
        all (hiLists) should (contain atLeastOneOf ("ho" -> "ho") or contain atLeastOneOf ("hi" -> "hi"))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (contain atLeastOneOf ("hi" -> "hi") or contain atLeastOneOf ("hi" -> "hi"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " did not contain at least one of ((hi,hi)), and " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " did not contain at least one of ((hi,hi))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (all (hiLists) should (contain atLeastOneOf ("ho" -> "ho") or contain atLeastOneOf ("ho" -> "ho"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        (all (hiLists) should (contain atLeastOneOf ("hi" -> "hi") or contain atLeastOneOf ("ho" -> "ho"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        (all (hiLists) should (contain atLeastOneOf ("ho" -> "ho") or contain atLeastOneOf ("hi" -> "hi"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (contain atLeastOneOf ("hi" -> "hi") or contain atLeastOneOf ("hi" -> "hi"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " did not contain at least one of ((hi,hi)), and " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " did not contain at least one of ((hi,hi))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
    }
    
    "when used with (be (...) and contain oneOf (...)) syntax" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        all (list1s) should (be (scala.collection.mutable.LinkedHashMap(1 -> 1)) or contain atLeastOneOf (1 -> 1, 3 -> 3, 4 -> 4))
        all (list1s) should (be (scala.collection.mutable.LinkedHashMap(2 -> 2)) or contain atLeastOneOf (1 -> 1, 3 -> 3, 4 -> 4))
        all (list1s) should (be (scala.collection.mutable.LinkedHashMap(1 -> 1)) or contain atLeastOneOf (2 -> 2, 3 -> 3, 4 -> 4))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (be (scala.collection.mutable.LinkedHashMap(2 -> 2)) or contain atLeastOneOf (2 -> 2, 3 -> 3, 8 -> 8))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap(1 -> 1)) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2)) + ", and " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(1 -> 1)) + " did not contain at least one of ((2,2), (3,3), (8,8))", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = invertedStringEquality
        
        all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("hi" -> "hi")) or contain atLeastOneOf ("he" -> "he"))
        all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) or contain atLeastOneOf ("he" -> "he"))
        all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("hi" -> "hi")) or contain atLeastOneOf ("hi" -> "hi"))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) or contain atLeastOneOf ("hi" -> "hi"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("ho" -> "ho")) + ", and " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " did not contain at least one of ((hi,hi))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("hi" -> "hi")) or contain atLeastOneOf ("ho" -> "ho"))) (decided by invertedStringEquality)
        (all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) or contain atLeastOneOf ("ho" -> "ho"))) (decided by invertedStringEquality)
        (all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("hi" -> "hi")) or contain atLeastOneOf ("hi" -> "hi"))) (decided by invertedStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) or contain atLeastOneOf ("hi" -> "hi"))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " was not equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("ho" -> "ho")) + ", and " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " did not contain at least one of ((hi,hi))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
    }
    
    "when used with (not contain oneOf (..) and not contain oneOf (..))" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        all (list1s) should (not contain atLeastOneOf (3 -> 3, 2 -> 2, 8 -> 8) or not contain atLeastOneOf (8 -> 8, 3 -> 3, 4 -> 4))
        all (list1s) should (not contain atLeastOneOf (1 -> 1, 2 -> 2, 8 -> 8) or not contain atLeastOneOf (8 -> 8, 3 -> 3, 4 -> 4))
        all (list1s) should (not contain atLeastOneOf (3 -> 3, 2 -> 2, 8 -> 8) or not contain atLeastOneOf (8 -> 8, 3 -> 3, 1 -> 1))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain atLeastOneOf (2 -> 2, 6 -> 6, 8 -> 8) or not contain atLeastOneOf (2 -> 2, 3 -> 3, 4 -> 4))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2)) + " contained at least one of ((2,2), (6,6), (8,8)), and " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(2 -> 2)) + " contained at least one of ((2,2), (3,3), (4,4))", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = invertedStringEquality
        
        all (hiLists) should (not contain atLeastOneOf ("hi" -> "hi") or not contain atLeastOneOf ("hi" -> "hi"))
        all (hiLists) should (not contain atLeastOneOf ("ho" -> "ho") or not contain atLeastOneOf ("hi" -> "hi"))
        all (hiLists) should (not contain atLeastOneOf ("hi" -> "hi") or not contain atLeastOneOf ("h0"))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (not contain atLeastOneOf ("ho" -> "ho") or not contain atLeastOneOf ("ho" -> "ho"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " contained at least one of ((ho,ho)), and " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " contained at least one of ((ho,ho))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (all (hiLists) should (not contain atLeastOneOf ("hi" -> "hi") or not contain atLeastOneOf ("hi" -> "hi"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        (all (hiLists) should (not contain atLeastOneOf ("ho" -> "ho") or not contain atLeastOneOf ("hi" -> "hi"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        (all (hiLists) should (not contain atLeastOneOf ("hi" -> "hi") or not contain atLeastOneOf ("ho" -> "ho"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (not contain atLeastOneOf ("ho" -> "ho") or not contain atLeastOneOf ("ho" -> "ho"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " contained at least one of ((ho,ho)), and " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " contained at least one of ((ho,ho))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
    }
    
    "when used with (not be (...) and not contain oneOf (...))" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        all (list1s) should (not be (scala.collection.mutable.LinkedHashMap(2 -> 2)) or not contain atLeastOneOf (8 -> 8, 3 -> 3, 4 -> 4))
        all (list1s) should (not be (scala.collection.mutable.LinkedHashMap(1 -> 1)) or not contain atLeastOneOf (8 -> 8, 3 -> 3, 4 -> 4))
        all (list1s) should (not be (scala.collection.mutable.LinkedHashMap(2 -> 2)) or not contain atLeastOneOf (8 -> 8, 3 -> 3, 1 -> 1))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (not be (scala.collection.mutable.LinkedHashMap(1 -> 1)) or not contain atLeastOneOf (2 -> 2, 3 -> 3, 1 -> 1))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap(1 -> 1)) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(1 -> 1)) + ", and " + decorateToStringValue(scala.collection.mutable.LinkedHashMap(1 -> 1)) + " contained at least one of ((2,2), (3,3), (1,1))", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = invertedStringEquality
        
        all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) or not contain atLeastOneOf ("hi" -> "hi"))
        all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("hi" -> "hi")) or not contain atLeastOneOf ("hi" -> "hi"))
        all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) or not contain atLeastOneOf ("ho" -> "ho"))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("hi" -> "hi")) or not contain atLeastOneOf ("ho" -> "ho"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + ", and " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " contained at least one of ((ho,ho))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) or not contain atLeastOneOf ("hi" -> "hi"))) (decided by invertedStringEquality)
        (all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("hi" -> "hi")) or not contain atLeastOneOf ("hi" -> "hi"))) (decided by invertedStringEquality)
        (all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("ho" -> "ho")) or not contain atLeastOneOf ("ho" -> "ho"))) (decided by invertedStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (not be (scala.collection.mutable.LinkedHashMap("hi" -> "hi")) or not contain atLeastOneOf ("ho" -> "ho"))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " was equal to " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + ", and " + decorateToStringValue(scala.collection.mutable.LinkedHashMap("hi" -> "hi")) + " contained at least one of ((ho,ho))", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
    }
  }
}
