/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Normalization
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue
import scala.collection.JavaConverters._

class OptionShouldContainNoneOfLogicalAndSpec extends Spec with Matchers {
  
  val invertedStringEquality =
    new Equality[String] {
      def areEqual(a: String, b: Any): Boolean = a != b
    }
  
  val invertedOptionOfStringEquality = 
    new Equality[Option[String]] {
      def areEqual(a: Option[String], b: Any): Boolean = a != b
    }
  
  val upperCaseStringEquality =
    new Equality[String] {
      def areEqual(a: String, b: Any): Boolean = a.toUpperCase == b
    }
  
  val upperCaseOptionOfStringEquality = 
    new Equality[Option[String]] {
      def areEqual(a: Option[String], b: Any): Boolean = a.map(_.toUpperCase) == b
    }
  
  //ADDITIONAL//
  
  val fileName: String = "OptionShouldContainNoneOfLogicalAndSpec.scala"
  
  object `a Option` {
    
    val fumOption: Option[String] = Option("fum")
    val toOption: Option[String] = Option("to")
    
    object `when used with (contain noneOf (..) and contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumOption should (contain noneOf ("fee", "fie", "foe", "fam") and contain noneOf("fie", "fee", "fam", "foe"))
        val e1 = intercept[TestFailedException] {
          fumOption should (contain noneOf ("fee", "fie", "foe", "fum") and contain noneOf ("fie", "fee", "fam", "foe"))
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumOption), "\"fee\", \"fie\", \"foe\", \"fum\""), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumOption should (contain noneOf ("fee", "fie", "foe", "fam") and contain noneOf ("fie", "fee", "fum", "foe"))
        }
        checkMessageStackDepth(e2, Resources("didNotContainOneOfElements", decorateToStringValue(fumOption), "\"fee\", \"fie\", \"foe\", \"fam\"") + ", but " + Resources("containedOneOfElements", decorateToStringValue(fumOption), "\"fie\", \"fee\", \"fum\", \"foe\""), fileName, thisLineNumber - 2)
      }

      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumOption should (contain noneOf ("fee", "fie", "foe", "fum") and contain noneOf ("fee", "fie", "fum", "foe"))
        val e1 = intercept[TestFailedException] {
          fumOption should (contain noneOf ("FEE", "FIE", "FOE", "FUM") and contain noneOf ("fee", "fie", "fum", "foe"))
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumOption), "\"FEE\", \"FIE\", \"FOE\", \"FUM\""), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumOption should (contain noneOf ("fee", "fie", "foe", "fum") and (contain noneOf ("FEE", "FIE", "FUM", "FOE")))
        }
        checkMessageStackDepth(e2, Resources("didNotContainOneOfElements", decorateToStringValue(fumOption), "\"fee\", \"fie\", \"foe\", \"fum\"") + ", but " + Resources("containedOneOfElements", decorateToStringValue(fumOption), "\"FEE\", \"FIE\", \"FUM\", \"FOE\""), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumOption should (contain noneOf ("fee", "fie", "foe", "fum") and contain noneOf ("fee", "fie", "fum", "foe"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumOption should (contain noneOf ("FEE", "FIE", "FOE", "FUM") and contain noneOf ("fee", "fie", "fum", "foe"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumOption), "\"FEE\", \"FIE\", \"FOE\", \"FUM\""), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumOption should (contain noneOf ("fee", "fie", "foe", "fum") and contain noneOf ("FEE", "FIE", "FUM", "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotContainOneOfElements", decorateToStringValue(fumOption), "\"fee\", \"fie\", \"foe\", \"fum\"") + ", but " + Resources("containedOneOfElements", decorateToStringValue(fumOption), "\"FEE\", \"FIE\", \"FUM\", \"FOE\""), fileName, thisLineNumber - 2)
        (fumOption should (contain noneOf (" FEE ", " FIE ", " FOE ", " FAM ") and contain noneOf (" FEE ", " FIE ", " FOE ", " FAM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (equal (..) and contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumOption should (equal (fumOption) and contain noneOf("fie", "fee", "fam", "foe"))
        val e1 = intercept[TestFailedException] {
          fumOption should (equal (toOption) and contain noneOf ("fee", "fie", "foe", "fam"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumOption), decorateToStringValue(toOption)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumOption should (equal (fumOption) and contain noneOf ("fee", "fie", "foe", "fum"))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumOption), decorateToStringValue(fumOption)) + ", but " + Resources("containedOneOfElements", decorateToStringValue(fumOption), "\"fee\", \"fie\", \"foe\", \"fum\""), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumOption should (equal (fumOption) and contain noneOf ("fee", "fie", "foe", "fum"))
        val e1 = intercept[TestFailedException] {
          fumOption should (equal (toOption) and contain noneOf ("fee", "fie", "foe", "fum"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumOption), decorateToStringValue(toOption)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumOption should (equal (fumOption) and (contain noneOf ("FEE", "FIE", "FOE", "FUM")))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumOption), decorateToStringValue(fumOption)) + ", but " + Resources("containedOneOfElements", decorateToStringValue(fumOption), "\"FEE\", \"FIE\", \"FOE\", \"FUM\""), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumOption should (equal (toOption) and contain noneOf ("fee", "fie", "foe", "fum"))) (decided by invertedOptionOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumOption should (equal (fumOption) and contain noneOf ("fee", "fie", "foe", "fum"))) (decided by invertedOptionOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumOption), decorateToStringValue(fumOption)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumOption should (equal (toOption) and contain noneOf ("FEE", "FIE", "FOE", "FUM"))) (decided by invertedOptionOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumOption), decorateToStringValue(toOption)) + ", but " + Resources("containedOneOfElements", decorateToStringValue(fumOption), "\"FEE\", \"FIE\", \"FOE\", \"FUM\""), fileName, thisLineNumber - 2)
        (fumOption should (equal (toOption) and contain noneOf (" FEE ", " FIE ", " FOE ", " FAM "))) (decided by invertedOptionOfStringEquality, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (legacyEqual (..) and contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumOption should (legacyEqual (fumOption) and contain noneOf("fie", "fee", "fam", "foe"))
        val e1 = intercept[TestFailedException] {
          fumOption should (legacyEqual (toOption) and contain noneOf ("fee", "fie", "foe", "fam"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumOption), decorateToStringValue(toOption)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumOption should (legacyEqual (fumOption) and contain noneOf ("fee", "fie", "foe", "fum"))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumOption), decorateToStringValue(fumOption)) + ", but " + Resources("containedOneOfElements", decorateToStringValue(fumOption), "\"fee\", \"fie\", \"foe\", \"fum\""), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumOption should (legacyEqual (fumOption) and contain noneOf ("fee", "fie", "foe", "fum"))
        val e1 = intercept[TestFailedException] {
          fumOption should (legacyEqual (toOption) and contain noneOf ("fee", "fie", "foe", "fum"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumOption), decorateToStringValue(toOption)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumOption should (legacyEqual (fumOption) and (contain noneOf ("FEE", "FIE", "FOE", "FUM")))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumOption), decorateToStringValue(fumOption)) + ", but " + Resources("containedOneOfElements", decorateToStringValue(fumOption), "\"FEE\", \"FIE\", \"FOE\", \"FUM\""), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumOption should (legacyEqual (fumOption) and contain noneOf ("fee", "fie", "foe", "fum"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumOption should (legacyEqual (fumOption) and contain noneOf ("FEE", "FIE", "FOE", "FUM"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumOption), decorateToStringValue(fumOption)) + ", but " + Resources("containedOneOfElements", decorateToStringValue(fumOption), "\"FEE\", \"FIE\", \"FOE\", \"FUM\""), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumOption should (legacyEqual (toOption) and contain noneOf ("fee", "fie", "foe", "fum"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumOption), decorateToStringValue(toOption)), fileName, thisLineNumber - 2)
        (fumOption should (legacyEqual (fumOption) and contain noneOf (" FEE ", " FIE ", " FOE ", " FAM "))) (after being lowerCased and trimmed)
      }
    }

    object `when used with (contain noneOf (..) and legacyEqual (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumOption should (contain noneOf("fie", "fee", "fam", "foe") and legacyEqual (fumOption))
        val e1 = intercept[TestFailedException] {
          fumOption should (contain noneOf ("fee", "fie", "foe", "fam") and legacyEqual (toOption))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumOption), "\"fee\", \"fie\", \"foe\", \"fam\"") + ", but " + Resources("didNotEqual", decorateToStringValue(fumOption), decorateToStringValue(toOption)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumOption should (contain noneOf ("fee", "fie", "foe", "fum") and legacyEqual (fumOption))
        }
        checkMessageStackDepth(e2, Resources("containedOneOfElements", decorateToStringValue(fumOption), "\"fee\", \"fie\", \"foe\", \"fum\""), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumOption should (contain noneOf ("fee", "fie", "foe", "fum") and legacyEqual (fumOption))
        val e1 = intercept[TestFailedException] {
          fumOption should (contain noneOf ("FEE", "FIE", "FOE", "FUM") and legacyEqual (toOption))
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumOption), "\"FEE\", \"FIE\", \"FOE\", \"FUM\""), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumOption should (contain noneOf ("FEE", "FIE", "FOE", "FUM") and (legacyEqual (fumOption)))
        }
        checkMessageStackDepth(e2, Resources("containedOneOfElements", decorateToStringValue(fumOption), "\"FEE\", \"FIE\", \"FOE\", \"FUM\""), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumOption should (contain noneOf ("fee", "fie", "foe", "fum") and legacyEqual (fumOption))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumOption should (contain noneOf ("FEE", "FIE", "FOE", "FUM") and legacyEqual (fumOption))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumOption), "\"FEE\", \"FIE\", \"FOE\", \"FUM\""), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumOption should (contain noneOf ("fee", "fie", "foe", "fum") and legacyEqual (toOption))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotContainOneOfElements", decorateToStringValue(fumOption), "\"fee\", \"fie\", \"foe\", \"fum\"") + ", but " + Resources("didNotEqual", decorateToStringValue(fumOption), decorateToStringValue(toOption)), fileName, thisLineNumber - 2)
        (fumOption should (contain noneOf (" FEE ", " FIE ", " FOE ", " FAM ") and legacyEqual (fumOption))) (after being lowerCased and trimmed)
      }
    }
    
    object `when used with (not contain noneOf (..) and not contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumOption should (not contain noneOf ("fee", "fie", "foe", "fum") and not contain noneOf("fee", "fie", "fum", "foe"))
        val e1 = intercept[TestFailedException] {
          fumOption should (not contain noneOf ("FEE", "FIE", "FOE", "FUM") and not contain noneOf ("fee", "fie", "fum", "foe"))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumOption), "\"FEE\", \"FIE\", \"FOE\", \"FUM\""), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumOption should (not contain noneOf ("fee", "fie", "foe", "fum") and not contain noneOf ("FEE", "FIE", "FOE", "FUM"))
        }
        checkMessageStackDepth(e2, Resources("containedOneOfElements", decorateToStringValue(fumOption), "\"fee\", \"fie\", \"foe\", \"fum\"") + ", but " + Resources("didNotContainOneOfElements", decorateToStringValue(fumOption), "\"FEE\", \"FIE\", \"FOE\", \"FUM\""), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumOption should (not contain noneOf ("FEE", "FIE", "FOE", "FUM") and not contain noneOf ("FEE", "FIE", "FUM", "FOE"))
        val e1 = intercept[TestFailedException] {
          fumOption should (not contain noneOf ("fee", "fie", "foe", "fum") and not contain noneOf ("FEE", "FIE", "FUM", "FOE"))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumOption), "\"fee\", \"fie\", \"foe\", \"fum\""), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          fumOption should (not contain noneOf ("FEE", "FIE", "FOE", "FUM") and (not contain noneOf ("fee", "fie", "fum", "foe")))
        }
        checkMessageStackDepth(e2, Resources("containedOneOfElements", decorateToStringValue(fumOption), "\"FEE\", \"FIE\", \"FOE\", \"FUM\"") + ", but " + Resources("didNotContainOneOfElements", decorateToStringValue(fumOption), "\"fee\", \"fie\", \"fum\", \"foe\""), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumOption should (not contain noneOf ("FEE", "FIE", "FOE", "FUM") and not contain noneOf ("FEE", "FIE", "FUM", "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumOption should (not contain noneOf ("fee", "fie", "foe", "fum") and not contain noneOf ("FEE", "FIE", "FUM", "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumOption), "\"fee\", \"fie\", \"foe\", \"fum\""), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumOption should (not contain noneOf ("FEE", "FIE", "FOE", "FUM") and not contain noneOf ("fee", "fie", "fum", "foe"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedOneOfElements", decorateToStringValue(fumOption), "\"FEE\", \"FIE\", \"FOE\", \"FUM\"") + ", but " + Resources("didNotContainOneOfElements", decorateToStringValue(fumOption), "\"fee\", \"fie\", \"fum\", \"foe\""), fileName, thisLineNumber - 2)
        (fumOption should (contain noneOf (" FEE ", " FIE ", " FOE ", " FAM ") and contain noneOf (" FEE ", " FIE ", " FOE ", " FAM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (not equal (..) and not contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumOption should (not equal (toOption) and not contain noneOf("fee", "fie", "foe", "fum"))
        val e1 = intercept[TestFailedException] {
          fumOption should (not equal (fumOption) and not contain noneOf ("fee", "fie", "foe", "fum"))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumOption), decorateToStringValue(fumOption)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumOption should (not equal (toOption) and not contain noneOf ("FEE", "FIE", "FOE", "FUM"))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumOption), decorateToStringValue(toOption)) + ", but " + Resources("didNotContainOneOfElements", decorateToStringValue(fumOption), "\"FEE\", \"FIE\", \"FOE\", \"FUM\""), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumOption should (not equal (toOption) and not contain noneOf ("FEE", "FIE", "FOE", "FUM"))
        val e1 = intercept[TestFailedException] {
          fumOption should (not equal (fumOption) and not contain noneOf ("FEE", "FIE", "FOE", "FUM"))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumOption), decorateToStringValue(fumOption)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumOption should (not equal (toOption) and (not contain noneOf ("fee", "fie", "foe", "fum")))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumOption), decorateToStringValue(toOption)) + ", but " + Resources("didNotContainOneOfElements", decorateToStringValue(fumOption), "\"fee\", \"fie\", \"foe\", \"fum\""), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumOption should (not equal (fumOption) and not contain noneOf ("FEE", "FIE", "FOE", "FUM"))) (decided by invertedOptionOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumOption should (not equal (fumOption) and not contain noneOf ("fee", "fie", "foe", "fum"))) (decided by invertedOptionOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumOption), decorateToStringValue(fumOption)) + ", but " + Resources("didNotContainOneOfElements", decorateToStringValue(fumOption), "\"fee\", \"fie\", \"foe\", \"fum\""), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumOption should (not equal (toOption) and not contain noneOf ("FEE", "FIE", "FOE", "FUM"))) (decided by invertedOptionOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumOption), decorateToStringValue(toOption)), fileName, thisLineNumber - 2)
        (fumOption should (not contain noneOf (" FEE ", " FIE ", " FOE ", " FUM ") and not contain noneOf (" FEE ", " FIE ", " FOE ", " FUM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (not be (..) and not contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumOption should (not be (toOption) and not contain noneOf("fee", "fie", "foe", "fum"))
        val e1 = intercept[TestFailedException] {
          fumOption should (not be (fumOption) and not contain noneOf ("fee", "fie", "foe", "fum"))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumOption), decorateToStringValue(fumOption)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumOption should (not be (toOption) and not contain noneOf ("FEE", "FIE", "FOE", "FUM"))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue(fumOption), decorateToStringValue(toOption)) + ", but " + Resources("didNotContainOneOfElements", decorateToStringValue(fumOption), "\"FEE\", \"FIE\", \"FOE\", \"FUM\""), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumOption should (not be (toOption) and not contain noneOf ("FEE", "FIE", "FOE", "FUM"))
        val e1 = intercept[TestFailedException] {
          fumOption should (not be (fumOption) and not contain noneOf ("FEE", "FIE", "FOE", "FUM"))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumOption), decorateToStringValue(fumOption)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumOption should (not be (toOption) and (not contain noneOf ("fee", "fie", "foe", "fum")))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue(fumOption), decorateToStringValue(toOption)) + ", but " + Resources("didNotContainOneOfElements", decorateToStringValue(fumOption), "\"fee\", \"fie\", \"foe\", \"fum\""), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumOption should (not be (toOption) and not contain noneOf ("FEE", "FIE", "FOE", "FUM"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumOption should (not be (toOption) and not contain noneOf ("fee", "fie", "foe", "fum"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasNotEqualTo", decorateToStringValue(fumOption), decorateToStringValue(toOption)) + ", but " + Resources("didNotContainOneOfElements", decorateToStringValue(fumOption), "\"fee\", \"fie\", \"foe\", \"fum\""), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumOption should (not be (fumOption) and not contain noneOf ("FEE", "FIE", "FOE", "FUM"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, Resources("wasEqualTo", decorateToStringValue(fumOption), decorateToStringValue(fumOption)), fileName, thisLineNumber - 2)
        (fumOption should (not contain noneOf (" FEE ", " FIE ", " FOE ", " FUM ") and not contain noneOf (" FEE ", " FIE ", " FOE ", " FUM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
  }
  
  object `collection of Options` {
    
    val list1s: Vector[Option[Int]] = Vector(Option(1), Option(1), Option(1))
    val lists: Vector[Option[Int]] = Vector(Option(1), Option(1), Option(2))
    val nils: Vector[Option[Int]] = Vector(None, None, None)
    val listsOption: Vector[Option[Int]] = Vector(Option(1), Option(1), None)
    val hiOptions: Vector[Option[String]] = Vector(Option("hi"), Option("hi"), Option("hi"))
    val toOptions: Vector[Option[String]] = Vector(Option("to"), Option("to"), Option("to"))
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + left
    
    object `when used with (contain noneOf (..) and contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain noneOf (3, 2, 8) and contain noneOf (2, 6, 8))
        atLeast (2, lists) should (contain noneOf (3, 2, 5) and contain noneOf (2, 3, 4))
        atMost (2, lists) should (contain noneOf (3, 2, 8) and contain noneOf (2, 3, 4))
        no (lists) should (contain noneOf (1, 2, 8) and contain noneOf (1, 2, 3))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain noneOf (2, 6, 8) and contain noneOf (3, 6, 9))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(Option(2)) + " contained one of (2, 6, 8)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (contain noneOf (3, 6, 9) and contain noneOf (2, 6, 8))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(Option(2)) + " did not contain one of (3, 6, 9), but " + decorateToStringValue(Option(2)) + " contained one of (2, 6, 8)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hiOptions) should (contain noneOf ("ho", "hello") and contain noneOf ("hi", "hey", "howdy"))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(Option("hi")) + " did not contain one of (\"ho\", \"hello\"), but " + decorateToStringValue(Option("hi")) + " contained one of (\"hi\", \"hey\", \"howdy\")", thisLineNumber - 2, hiOptions), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (lists) should (contain noneOf (3, 6, 9) and contain noneOf (2, 6, 8))
        }
        checkMessageStackDepth(e4, allErrMsg(2, decorateToStringValue(Option(2)) + " did not contain one of (3, 6, 9), but " + decorateToStringValue(Option(2)) + " contained one of (2, 6, 8)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiOptions) should (contain noneOf ("hi") and contain noneOf ("he"))
        
        val e1 = intercept[TestFailedException] {
          all (hiOptions) should (contain noneOf ("HI") and contain noneOf ("he"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Option("hi")) + " contained one of (\"HI\")", thisLineNumber - 2, hiOptions), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiOptions) should (contain noneOf ("he") and contain noneOf ("HI"))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(Option("hi")) + " did not contain one of (\"he\"), but " + decorateToStringValue(Option("hi")) + " contained one of (\"HI\")", thisLineNumber - 2, hiOptions), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiOptions) should (contain noneOf ("hi") and contain noneOf ("he"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiOptions) should (contain noneOf ("HI") and contain noneOf ("he"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Option("hi")) + " contained one of (\"HI\")", thisLineNumber - 2, hiOptions), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiOptions) should (contain noneOf ("he") and contain noneOf ("HI"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(Option("hi")) + " did not contain one of (\"he\"), but " + decorateToStringValue(Option("hi")) + " contained one of (\"HI\")", thisLineNumber - 2, hiOptions), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (be (..) and contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (be (Option(1)) and contain noneOf (2, 3, 4))
        atLeast (2, lists) should (be (Option(1)) and contain noneOf (2, 3, 4))
        atMost (2, lists) should (be (Option(1)) and contain noneOf (2, 3, 4))
        no (lists) should (be (Option(8)) and contain noneOf (1, 2, 3))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (be (Option(1)) and contain noneOf (3, 6, 9))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(Option(2)) + " was not equal to " + decorateToStringValue(Option(1)), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (list1s) should (be (Option(1)) and contain noneOf (1, 2, 3))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(Option(1)) + " was equal to " + decorateToStringValue(Option(1)) + ", but " + decorateToStringValue(Option(1)) + " contained one of (1, 2, 3)", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hiOptions) should (be (Option("hi")) and contain noneOf ("hi", "hey", "howdy"))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(Option("hi")) + " was equal to " + decorateToStringValue(Option("hi")) + ", but " + decorateToStringValue(Option("hi")) + " contained one of (\"hi\", \"hey\", \"howdy\")", thisLineNumber - 2, hiOptions), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (list1s) should (be (Option(1)) and contain noneOf (1, 2, 3))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(Option(1)) + " was equal to " + decorateToStringValue(Option(1)) + ", but " + decorateToStringValue(Option(1)) + " contained one of (1, 2, 3)", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiOptions) should (be (Option("hi")) and contain noneOf ("hi"))
        
        val e1 = intercept[TestFailedException] {
          all (hiOptions) should (be (Option("ho")) and contain noneOf ("hi"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Option("hi")) + " was not equal to " + decorateToStringValue(Option("ho")), thisLineNumber - 2, hiOptions), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiOptions) should (be (Option("hi")) and contain noneOf ("HI"))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(Option("hi")) + " was equal to " + decorateToStringValue(Option("hi")) + ", but " + decorateToStringValue(Option("hi")) + " contained one of (\"HI\")", thisLineNumber - 2, hiOptions), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiOptions) should (be (Option("hi")) and contain noneOf ("hi"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiOptions) should (be (Option("ho")) and contain noneOf ("hi"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Option("hi")) + " was not equal to " + decorateToStringValue(Option("ho")), thisLineNumber - 2, hiOptions), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiOptions) should (be (Option("hi")) and contain noneOf ("HI"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(Option("hi")) + " was equal to " + decorateToStringValue(Option("hi")) + ", but " + decorateToStringValue(Option("hi")) + " contained one of (\"HI\")", thisLineNumber - 2, hiOptions), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not contain noneOf (..) and not contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not contain noneOf (1, 2, 8) and not contain noneOf (1, 6, 8))
        atLeast (2, lists) should (not contain noneOf (1, 6, 8) and not contain noneOf (1, 3, 8))
        atMost (2, lists) should (not contain noneOf (1, 3, 8) and contain noneOf (1, 6, 8))
        no (lists) should (not contain noneOf (3, 6, 9) and not contain noneOf (5, 7, 9))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain noneOf (1, 6, 8) and not contain noneOf (1, 2, 3))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(Option(2)) + " did not contain one of (1, 6, 8)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not contain noneOf (1, 2, 3) and not contain noneOf (1, 6, 8))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(Option(2)) + " contained one of (1, 2, 3), but " + decorateToStringValue(Option(2)) + " did not contain one of (1, 6, 8)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hiOptions) should (not contain noneOf ("ho", "hello") and not contain noneOf ("hi", "hey", "howdy"))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(Option("hi")) + " did not contain one of (\"ho\", \"hello\")", thisLineNumber - 2, hiOptions), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiOptions) should (not contain noneOf ("hi", "hey", "howdy") and not contain noneOf ("ho", "hello"))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(Option("hi")) + " contained one of (\"hi\", \"hey\", \"howdy\"), but " + decorateToStringValue(Option("hi")) + " did not contain one of (\"ho\", \"hello\")", thisLineNumber - 2, hiOptions), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiOptions) should (not contain noneOf ("HI") and not contain noneOf ("HI"))
        
        val e1 = intercept[TestFailedException] {
          all (hiOptions) should (not contain noneOf ("hi") and not contain noneOf ("HI"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Option("hi")) + " did not contain one of (\"hi\")", thisLineNumber - 2, hiOptions), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiOptions) should (not contain noneOf ("HI") and not contain noneOf ("hi"))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(Option("hi")) + " contained one of (\"HI\"), but " + decorateToStringValue(Option("hi")) + " did not contain one of (\"hi\")", thisLineNumber - 2, hiOptions), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiOptions) should (not contain noneOf ("HI") and not contain noneOf ("HI"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiOptions) should (not contain noneOf ("hi") and not contain noneOf ("HI"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Option("hi")) + " did not contain one of (\"hi\")", thisLineNumber - 2, hiOptions), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiOptions) should (not contain noneOf ("HI") and not contain noneOf ("hi"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(Option("hi")) + " contained one of (\"HI\"), but " + decorateToStringValue(Option("hi")) + " did not contain one of (\"hi\")", thisLineNumber - 2, hiOptions), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not be (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not be (Option(2)) and not contain noneOf (1, 2, 3))
        atLeast (2, lists) should (not be (Option(3)) and not contain noneOf (1, 6, 8))
        atMost (2, lists) should (not be (Option(3)) and contain noneOf (1, 6, 8))
        no (list1s) should (not be (Option(1)) and not contain noneOf (3, 6, 9))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not be (Option(2)) and not contain noneOf (1, 2, 3))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(Option(2)) + " was equal to " + decorateToStringValue(Option(2)), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not be (Option(3)) and not contain noneOf (1, 6, 8))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue(Option(2)) + " was not equal to " + decorateToStringValue(Option(3)) + ", but " + decorateToStringValue(Option(2)) + " did not contain one of (1, 6, 8)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hiOptions) should (not be (Option("hi")) and not contain noneOf ("hi", "hey", "howdy"))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue(Option("hi")) + " was equal to " + decorateToStringValue(Option("hi")), thisLineNumber - 2, hiOptions), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiOptions) should (not be (Option("ho")) and not contain noneOf ("ho", "hello"))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue(Option("hi")) + " was not equal to " + decorateToStringValue(Option("ho")) + ", but " + decorateToStringValue(Option("hi")) + " did not contain one of (\"ho\", \"hello\")", thisLineNumber - 2, hiOptions), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiOptions) should (not be (Option("ho")) and not contain noneOf ("HI"))
        
        val e1 = intercept[TestFailedException] {
          all (hiOptions) should (not be (Option("hi")) and not contain noneOf ("HI"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Option("hi")) + " was equal to " + decorateToStringValue(Option("hi")), thisLineNumber - 2, hiOptions), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiOptions) should (not be (Option("ho")) and not contain noneOf ("hi"))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(Option("hi")) + " was not equal to " + decorateToStringValue(Option("ho")) + ", but " + decorateToStringValue(Option("hi")) + " did not contain one of (\"hi\")", thisLineNumber - 2, hiOptions), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiOptions) should (not be (Option("ho")) and not contain noneOf ("HI"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiOptions) should (not be (Option("hi")) and not contain noneOf ("HI"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Option("hi")) + " was equal to " + decorateToStringValue(Option("hi")), thisLineNumber - 2, hiOptions), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiOptions) should (not be (Option("ho")) and not contain noneOf ("hi"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue(Option("hi")) + " was not equal to " + decorateToStringValue(Option("ho")) + ", but " + decorateToStringValue(Option("hi")) + " did not contain one of (\"hi\")", thisLineNumber - 2, hiOptions), fileName, thisLineNumber - 2)
      }
    }
  }
}
