/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Normalization
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue
import scala.collection.JavaConverters._

class OptionShouldContainNoneOfLogicalOrSpec extends Spec with Matchers {
  
  val invertedStringEquality =
    new Equality[String] {
      def areEqual(a: String, b: Any): Boolean = a != b
    }
  
  val invertedOptionOfStringEquality = 
    new Equality[Option[String]] {
      def areEqual(a: Option[String], b: Any): Boolean = a != b
    }
  
  val upperCaseStringEquality =
    new Equality[String] {
      def areEqual(a: String, b: Any): Boolean = a.toUpperCase == b
    }
  
  val upperCaseOptionOfStringEquality = 
    new Equality[Option[String]] {
      def areEqual(a: Option[String], b: Any): Boolean = a.map(_.toUpperCase) == b
    }
  
  //ADDITIONAL//
  
  val fileName: String = "OptionShouldContainNoneOfLogicalOrSpec.scala"
  
  object `a Option` {
    
    val fumOption: Option[String] = Option("fum")
    val toOption: Option[String] = Option("to")
    
    object `when used with (contain noneOf (..) or contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumOption should (contain noneOf ("fee", "fie", "foe", "fam") or contain noneOf("fie", "fee", "fam", "foe"))
        fumOption should (contain noneOf ("fee", "fie", "foe", "fam") or contain noneOf("fie", "fee", "fum", "foe"))
        fumOption should (contain noneOf ("fee", "fie", "foe", "fum") or contain noneOf("fie", "fee", "fam", "foe"))
        val e1 = intercept[TestFailedException] {
          fumOption should (contain noneOf ("fee", "fie", "foe", "fum") or contain noneOf ("fie", "fee", "fum", "foe"))
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumOption), "\"fee\", \"fie\", \"foe\", \"fum\"") + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumOption), "\"fie\", \"fee\", \"fum\", \"foe\""), fileName, thisLineNumber - 2)
      }

      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumOption should (contain noneOf ("fee", "fie", "foe", "fum") or contain noneOf ("fee", "fie", "fum", "foe"))
        fumOption should (contain noneOf ("FEE", "FIE", "FOE", "FUM") or contain noneOf ("fee", "fie", "fum", "foe"))
        fumOption should (contain noneOf ("fee", "fie", "foe", "fum") or contain noneOf ("FEE", "FIE", "FUM", "FOE"))
        val e1 = intercept[TestFailedException] {
          fumOption should (contain noneOf ("FEE", "FIE", "FOE", "FUM") or (contain noneOf ("FEE", "FIE", "FUM", "FOE")))
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumOption), "\"FEE\", \"FIE\", \"FOE\", \"FUM\"") + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumOption), "\"FEE\", \"FIE\", \"FUM\", \"FOE\""), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumOption should (contain noneOf ("fee", "fie", "foe", "fum") or contain noneOf ("fee", "fie", "fum", "foe"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumOption should (contain noneOf ("FEE", "FIE", "FOE", "FUM") or contain noneOf ("fee", "fie", "fum", "foe"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumOption should (contain noneOf ("fee", "fie", "foe", "fum") or contain noneOf ("FEE", "FIE", "FUM", "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumOption should (contain noneOf ("FEE", "FIE", "FOE", "FUM") or contain noneOf ("FEE", "FIE", "FUM", "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumOption), "\"FEE\", \"FIE\", \"FOE\", \"FUM\"") + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumOption), "\"FEE\", \"FIE\", \"FUM\", \"FOE\""), fileName, thisLineNumber - 2)
        (fumOption should (contain noneOf (" FEE ", " FIE ", " FOE ", " FAM ") or contain noneOf (" FEE ", " FIE ", " FOE ", " FAM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (equal (..) and contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumOption should (equal (fumOption) or contain noneOf("fie", "fee", "fam", "foe"))
        fumOption should (equal (toOption) or contain noneOf("fie", "fee", "fam", "foe"))
        fumOption should (equal (fumOption) or contain noneOf("fee", "fie", "foe", "fum"))
        val e1 = intercept[TestFailedException] {
          fumOption should (equal (toOption) or contain noneOf ("fee", "fie", "foe", "fum"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumOption), decorateToStringValue(toOption)) + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumOption), "\"fee\", \"fie\", \"foe\", \"fum\""), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumOption should (equal (fumOption) or contain noneOf ("fie", "fee", "fum", "foe"))
        fumOption should (equal (toOption) or contain noneOf ("fie", "fee", "fum", "foe"))
        fumOption should (equal (fumOption) or contain noneOf ("FEE", "FIE", "FOE", "FUM"))
        val e1 = intercept[TestFailedException] {
          fumOption should (equal (toOption) or (contain noneOf ("FEE", "FIE", "FOE", "FUM")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumOption), decorateToStringValue(toOption)) + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumOption), "\"FEE\", \"FIE\", \"FOE\", \"FUM\""), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumOption should (equal (toOption) or contain noneOf ("fie", "fee", "fum", "foe"))) (decided by invertedOptionOfStringEquality, decided by upperCaseStringEquality)
        (fumOption should (equal (fumOption) or contain noneOf ("fie", "fee", "fum", "foe"))) (decided by invertedOptionOfStringEquality, decided by upperCaseStringEquality)
        (fumOption should (equal (toOption) or contain noneOf ("FEE", "FIE", "FOE", "FUM"))) (decided by invertedOptionOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumOption should (equal (fumOption) or contain noneOf ("FEE", "FIE", "FOE", "FUM"))) (decided by invertedOptionOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumOption), decorateToStringValue(fumOption)) + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumOption), "\"FEE\", \"FIE\", \"FOE\", \"FUM\""), fileName, thisLineNumber - 2)
        (fumOption should (equal (toOption) or contain noneOf (" FEE ", " FIE ", " FOE ", " FAM "))) (decided by invertedOptionOfStringEquality, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (legacyEqual (..) and contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumOption should (legacyEqual (fumOption) or contain noneOf("fie", "fee", "fam", "foe"))
        fumOption should (legacyEqual (toOption) or contain noneOf("fie", "fee", "fam", "foe"))
        fumOption should (legacyEqual (fumOption) or contain noneOf("fie", "fee", "fum", "foe"))
        val e1 = intercept[TestFailedException] {
          fumOption should (legacyEqual (toOption) or contain noneOf ("fie", "fee", "fum", "foe"))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumOption), decorateToStringValue(toOption)) + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumOption), "\"fie\", \"fee\", \"fum\", \"foe\""), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumOption should (legacyEqual (fumOption) or contain noneOf ("fee", "fie", "foe", "fum"))
        fumOption should (legacyEqual (toOption) or contain noneOf ("fee", "fie", "foe", "fum"))
        fumOption should (legacyEqual (fumOption) or contain noneOf ("FEE", "FIE", "FOE", "FUM"))
        val e1 = intercept[TestFailedException] {
          fumOption should (legacyEqual (toOption) or (contain noneOf ("FEE", "FIE", "FOE", "FUM")))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumOption), decorateToStringValue(toOption)) + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumOption), "\"FEE\", \"FIE\", \"FOE\", \"FUM\""), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumOption should (legacyEqual (fumOption) or contain noneOf ("fee", "fie", "foe", "fum"))) (decided by upperCaseStringEquality)
        (fumOption should (legacyEqual (toOption) or contain noneOf ("fee", "fie", "foe", "fum"))) (decided by upperCaseStringEquality)
        (fumOption should (legacyEqual (fumOption) or contain noneOf ("FEE", "FIE", "FOE", "FUM"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumOption should (legacyEqual (toOption) or contain noneOf ("FEE", "FIE", "FOE", "FUM"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumOption), decorateToStringValue(toOption)) + ", and " + Resources("containedOneOfElements", decorateToStringValue(fumOption), "\"FEE\", \"FIE\", \"FOE\", \"FUM\""), fileName, thisLineNumber - 2)
        (fumOption should (legacyEqual (fumOption) or contain noneOf (" FEE ", " FIE ", " FOE ", " FAM "))) (after being lowerCased and trimmed)
      }
    }

    object `when used with (contain noneOf (..) and legacyEqual (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumOption should (contain noneOf("fie", "fee", "fam", "foe") or legacyEqual (fumOption))
        fumOption should (contain noneOf("fie", "fee", "fum", "foe") or legacyEqual (fumOption))
        fumOption should (contain noneOf("fie", "fee", "fam", "foe") or legacyEqual (toOption))
        val e1 = intercept[TestFailedException] {
          fumOption should (contain noneOf ("fie", "fee", "fum", "foe") or legacyEqual (toOption))
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumOption), "\"fie\", \"fee\", \"fum\", \"foe\"") + ", and " + Resources("didNotEqual", decorateToStringValue(fumOption), decorateToStringValue(toOption)), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumOption should (contain noneOf ("fee", "fie", "foe", "fum") or legacyEqual (fumOption))
        fumOption should (contain noneOf ("FEE", "FIE", "FOE", "FUM") or legacyEqual (fumOption))
        fumOption should (contain noneOf ("fee", "fie", "foe", "fum") or legacyEqual (toOption))
        val e1 = intercept[TestFailedException] {
          fumOption should (contain noneOf ("FEE", "FIE", "FOE", "FUM") or legacyEqual (toOption))
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumOption), "\"FEE\", \"FIE\", \"FOE\", \"FUM\"") + ", and " + Resources("didNotEqual", decorateToStringValue(fumOption), decorateToStringValue(toOption)), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumOption should (contain noneOf ("fee", "fie", "foe", "fum") or legacyEqual (fumOption))) (decided by upperCaseStringEquality)
        (fumOption should (contain noneOf ("FEE", "FIE", "FOE", "FUM") or legacyEqual (fumOption))) (decided by upperCaseStringEquality)
        (fumOption should (contain noneOf ("fee", "fie", "foe", "fum") or legacyEqual (toOption))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumOption should (contain noneOf ("FEE", "FIE", "FOE", "FUM") or legacyEqual (toOption))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("containedOneOfElements", decorateToStringValue(fumOption), "\"FEE\", \"FIE\", \"FOE\", \"FUM\"") + ", and " + Resources("didNotEqual", decorateToStringValue(fumOption), decorateToStringValue(toOption)), fileName, thisLineNumber - 2)
        (fumOption should (contain noneOf (" FEE ", " FIE ", " FOE ", " FUM ") or legacyEqual (fumOption))) (after being lowerCased and trimmed)
      }
    }
    
    object `when used with (not contain noneOf (..) and not contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumOption should (not contain noneOf ("fee", "fie", "foe", "fum") or not contain noneOf("fee", "fie", "fum", "foe"))
        fumOption should (not contain noneOf ("fee", "fie", "foe", "fam") or not contain noneOf("fee", "fie", "fum", "foe"))
        fumOption should (not contain noneOf ("fee", "fie", "foe", "fum") or not contain noneOf("fee", "fie", "fam", "foe"))
        val e1 = intercept[TestFailedException] {
          fumOption should (not contain noneOf ("fee", "fie", "foe", "fam") or not contain noneOf ("fee", "fie", "fam", "foe"))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumOption), "\"fee\", \"fie\", \"foe\", \"fam\"") + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumOption), "\"fee\", \"fie\", \"fam\", \"foe\""), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumOption should (not contain noneOf ("FEE", "FIE", "FOE", "FUM") or not contain noneOf ("FEE", "FIE", "FUM", "FOE"))
        fumOption should (not contain noneOf ("fee", "fie", "foe", "fum") or not contain noneOf ("FEE", "FIE", "FUM", "FOE"))
        fumOption should (not contain noneOf ("FEE", "FIE", "FOE", "FUM") or not contain noneOf ("fee", "fie", "fum", "foe"))
        val e1 = intercept[TestFailedException] {
          fumOption should (not contain noneOf ("fee", "fie", "foe", "fum") or not contain noneOf ("fee", "fie", "fum", "foe"))
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumOption), "\"fee\", \"fie\", \"foe\", \"fum\"") + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumOption), "\"fee\", \"fie\", \"fum\", \"foe\""), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumOption should (not contain noneOf ("FEE", "FIE", "FOE", "FUM") or not contain noneOf ("FEE", "FIE", "FUM", "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumOption should (not contain noneOf ("fee", "fie", "foe", "fum") or not contain noneOf ("FEE", "FIE", "FUM", "FOE"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumOption should (not contain noneOf ("FEE", "FIE", "FOE", "FUM") or not contain noneOf ("fee", "fie", "fum", "foe"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumOption should (not contain noneOf ("fee", "fie", "foe", "fum") or not contain noneOf ("fee", "fie", "fum", "foe"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainOneOfElements", decorateToStringValue(fumOption), "\"fee\", \"fie\", \"foe\", \"fum\"") + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumOption), "\"fee\", \"fie\", \"fum\", \"foe\""), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not equal (..) and not contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumOption should (not equal (toOption) or not contain noneOf("fee", "fie", "foe", "fum"))
        fumOption should (not equal (fumOption) or not contain noneOf("fee", "fie", "foe", "fum"))
        fumOption should (not equal (toOption) or not contain noneOf("fee", "fie", "foe", "fam"))
        val e1 = intercept[TestFailedException] {
          fumOption should (not equal (fumOption) or not contain noneOf ("fee", "fie", "foe", "fam"))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumOption), decorateToStringValue(fumOption)) + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumOption), "\"fee\", \"fie\", \"foe\", \"fam\""), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumOption should (not equal (toOption) or not contain noneOf ("FIE", "FEE", "FUM", "FOE"))
        fumOption should (not equal (fumOption) or not contain noneOf ("FIE", "FEE", "FUM", "FOE"))
        fumOption should (not equal (toOption) or not contain noneOf ("fie", "fee", "fum", "foe"))
        val e2 = intercept[TestFailedException] {
          fumOption should (not equal (fumOption) or (not contain noneOf ("fie", "fee", "fum", "foe")))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumOption), decorateToStringValue(fumOption)) + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumOption), "\"fie\", \"fee\", \"fum\", \"foe\""), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumOption should (not equal (fumOption) or not contain noneOf ("FIE", "FEE", "FUM", "FOE"))) (decided by invertedOptionOfStringEquality, decided by upperCaseStringEquality)
        (fumOption should (not equal (toOption) or not contain noneOf ("FIE", "FEE", "FUM", "FOE"))) (decided by invertedOptionOfStringEquality, decided by upperCaseStringEquality)
        (fumOption should (not equal (fumOption) or not contain noneOf ("fie", "fee", "fum", "foe"))) (decided by invertedOptionOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumOption should (not equal (toOption) or not contain noneOf ("fie", "fee", "fum", "foe"))) (decided by invertedOptionOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumOption), decorateToStringValue(toOption)) + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumOption), "\"fie\", \"fee\", \"fum\", \"foe\""), fileName, thisLineNumber - 2)
        (fumOption should (not contain noneOf (" FEE ", " FIE ", " FOE ", " FUM ") or not contain noneOf (" FEE ", " FIE ", " FOE ", " FUM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
    object `when used with (not be (..) and not contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumOption should (not be (toOption) or not contain noneOf("fee", "fie", "foe", "fum"))
        fumOption should (not be (fumOption) or not contain noneOf("fee", "fie", "foe", "fum"))
        fumOption should (not be (toOption) or not contain noneOf("FEE", "FIE", "FOE", "FUM"))
        val e1 = intercept[TestFailedException] {
          fumOption should (not be (fumOption) or not contain noneOf ("FEE", "FIE", "FOE", "FUM"))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumOption), decorateToStringValue(fumOption)) + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumOption), "\"FEE\", \"FIE\", \"FOE\", \"FUM\""), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumOption should (not be (toOption) or not contain noneOf ("FEE", "FIE", "FOE", "FUM"))
        fumOption should (not be (fumOption) or not contain noneOf ("FEE", "FIE", "FOE", "FUM"))
        fumOption should (not be (toOption) or not contain noneOf ("fee", "fie", "foe", "fum"))
        val e1 = intercept[TestFailedException] {
          fumOption should (not be (fumOption) or (not contain noneOf ("fee", "fie", "foe", "fum")))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumOption), decorateToStringValue(fumOption)) + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumOption), "\"fee\", \"fie\", \"foe\", \"fum\""), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumOption should (not be (toOption) or not contain noneOf ("FEE", "FIE", "FOE", "FUM"))) (decided by upperCaseStringEquality)
        (fumOption should (not be (fumOption) or not contain noneOf ("FEE", "FIE", "FOE", "FUM"))) (decided by upperCaseStringEquality)
        (fumOption should (not be (toOption) or not contain noneOf ("fee", "fie", "foe", "fum"))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumOption should (not be (fumOption) or not contain noneOf ("fee", "fie", "foe", "fum"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumOption), decorateToStringValue(fumOption)) + ", and " + Resources("didNotContainOneOfElements", decorateToStringValue(fumOption), "\"fee\", \"fie\", \"foe\", \"fum\""), fileName, thisLineNumber - 2)
        (fumOption should (not contain noneOf (" FEE ", " FIE ", " FOE ", " FUM ") or not contain noneOf (" FEE ", " FIE ", " FOE ", " FUM "))) (after being lowerCased and trimmed, after being lowerCased and trimmed)
      }
    }
    
  }
  
  object `collection of Options` {
    
    val list1s: Vector[Option[Int]] = Vector(Option(1), Option(1), Option(1))
    val lists: Vector[Option[Int]] = Vector(Option(1), Option(1), Option(2))
    val nils: Vector[Option[Int]] = Vector(None, None, None)
    val listsOption: Vector[Option[Int]] = Vector(Option(1), Option(1), None)
    val hiOptions: Vector[Option[String]] = Vector(Option("hi"), Option("hi"), Option("hi"))
    val toOptions: Vector[Option[String]] = Vector(Option("to"), Option("to"), Option("to"))
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + left
    
    object `when used with (contain noneOf (..) and contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain noneOf (3, 6, 9) or contain noneOf (2, 6, 8))
        all (list1s) should (contain noneOf (1, 2, 3) or contain noneOf (2, 6, 8))
        all (list1s) should (contain noneOf (3, 6, 9) or contain noneOf (1, 2, 3))
        
        atLeast (2, lists) should (contain noneOf (2, 6, 8) or contain noneOf (3, 6, 9))
        atLeast (2, lists) should (contain noneOf (1, 2, 3) or contain noneOf (3, 6, 9))
        atLeast (2, lists) should (contain noneOf (2, 6, 8) or contain noneOf (1, 2, 3))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain noneOf (2, 6, 8) or contain noneOf (2, 3, 5))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(Option(2)) + " contained one of (2, 6, 8), and " + decorateToStringValue(Option(2)) + " contained one of (2, 3, 5)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiOptions) should (contain noneOf ("hi") or contain noneOf ("hi"))
        all (hiOptions) should (contain noneOf ("hi") or contain noneOf ("HI"))
        all (hiOptions) should (contain noneOf ("HI") or contain noneOf ("hi"))
        
        val e1 = intercept[TestFailedException] {
          all (hiOptions) should (contain noneOf ("HI") or contain noneOf ("HI"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Option("hi")) + " contained one of (\"HI\"), and " + decorateToStringValue(Option("hi")) + " contained one of (\"HI\")", thisLineNumber - 2, hiOptions), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiOptions) should (contain noneOf ("hi") or contain noneOf ("hi"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hiOptions) should (contain noneOf ("hi") or contain noneOf ("HI"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hiOptions) should (contain noneOf ("HI") or contain noneOf ("hi"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiOptions) should (contain noneOf ("HI") or contain noneOf ("HI"))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Option("hi")) + " contained one of (\"HI\"), and " + decorateToStringValue(Option("hi")) + " contained one of (\"HI\")", thisLineNumber - 2, hiOptions), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (be (..) and contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (be (Option(1)) or contain noneOf (2, 6, 8))
        all (list1s) should (be (Option(2)) or contain noneOf (2, 6, 8))
        all (list1s) should (be (Option(1)) or contain noneOf (1, 2, 3))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (be (Option(2)) or contain noneOf (1, 2, 3))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Option(1)) + " was not equal to " + decorateToStringValue(Option(2)) + ", and " + decorateToStringValue(Option(1)) + " contained one of (1, 2, 3)", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiOptions) should (be (Option("hi")) or contain noneOf ("hi"))
        all (hiOptions) should (be (Option("ho")) or contain noneOf ("hi"))
        all (hiOptions) should (be (Option("hi")) or contain noneOf ("HI"))
        
        val e1 = intercept[TestFailedException] {
          all (hiOptions) should (be (Option("ho")) or contain noneOf ("HI"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Option("hi")) + " was not equal to " + decorateToStringValue(Option("ho")) + ", and " + decorateToStringValue(Option("hi")) + " contained one of (\"HI\")", thisLineNumber - 2, hiOptions), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiOptions) should (be (Option("hi")) or contain noneOf ("hi"))) (decided by upperCaseStringEquality)
        (all (hiOptions) should (be (Option("ho")) or contain noneOf ("hi"))) (decided by upperCaseStringEquality)
        (all (hiOptions) should (be (Option("hi")) or contain noneOf ("HI"))) (decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiOptions) should (be (Option("ho")) or contain noneOf ("HI"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Option("hi")) + " was not equal to " + decorateToStringValue(Option("ho")) + ", and " + decorateToStringValue(Option("hi")) + " contained one of (\"HI\")", thisLineNumber - 2, hiOptions), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not contain noneOf (..) and not contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not contain noneOf (1, 2, 3) or not contain noneOf (1, 6, 8))
        all (list1s) should (not contain noneOf (2, 6, 8) or not contain noneOf (1, 6, 8))
        all (list1s) should (not contain noneOf (1, 2, 3) or not contain noneOf (2, 6, 8))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain noneOf (1, 6, 8) or not contain noneOf (1, 3, 5))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue(Option(2)) + " did not contain one of (1, 6, 8), and " + decorateToStringValue(Option(2)) + " did not contain one of (1, 3, 5)", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiOptions) should (not contain noneOf ("HI") or not contain noneOf ("HI"))
        all (hiOptions) should (not contain noneOf ("hi") or not contain noneOf ("HI"))
        all (hiOptions) should (not contain noneOf ("HI") or not contain noneOf ("hi"))
        
        val e1 = intercept[TestFailedException] {
          all (hiOptions) should (not contain noneOf ("hi") or not contain noneOf ("hi"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Option("hi")) + " did not contain one of (\"hi\"), and " + decorateToStringValue(Option("hi")) + " did not contain one of (\"hi\")", thisLineNumber - 2, hiOptions), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiOptions) should (not contain noneOf ("HI") or not contain noneOf ("HI"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hiOptions) should (not contain noneOf ("hi") or not contain noneOf ("HI"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hiOptions) should (not contain noneOf ("HI") or not contain noneOf ("hi"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiOptions) should (not contain noneOf ("hi") or not contain noneOf ("hi"))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Option("hi")) + " did not contain one of (\"hi\"), and " + decorateToStringValue(Option("hi")) + " did not contain one of (\"hi\")", thisLineNumber - 2, hiOptions), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not be (..) and not contain noneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not be (Option(2)) or not contain noneOf (1, 6, 8))
        all (list1s) should (not be (Option(1)) or not contain noneOf (1, 6, 8))
        all (list1s) should (not be (Option(2)) or not contain noneOf (2, 6, 8))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (not be (Option(1)) or not contain noneOf (2, 6, 8))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Option(1)) + " was equal to " + decorateToStringValue(Option(1)) + ", and " + decorateToStringValue(Option(1)) + " did not contain one of (2, 6, 8)", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        
        all (hiOptions) should (not be (Option("ho")) or not contain noneOf ("HI"))
        all (hiOptions) should (not be (Option("hi")) or not contain noneOf ("HI"))
        all (hiOptions) should (not be (Option("ho")) or not contain noneOf ("hi"))
        
        val e1 = intercept[TestFailedException] {
          all (hiOptions) should (not be (Option("hi")) or not contain noneOf ("hi"))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Option("hi")) + " was equal to " + decorateToStringValue(Option("hi")) + ", and " + decorateToStringValue(Option("hi")) + " did not contain one of (\"hi\")", thisLineNumber - 2, hiOptions), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiOptions) should (not be (Option("ho")) or not contain noneOf ("HI"))) (decided by upperCaseStringEquality)
        (all (hiOptions) should (not be (Option("hi")) or not contain noneOf ("HI"))) (decided by upperCaseStringEquality)
        (all (hiOptions) should (not be (Option("ho")) or not contain noneOf ("hi"))) (decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiOptions) should (not be (Option("hi")) or not contain noneOf ("hi"))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue(Option("hi")) + " was equal to " + decorateToStringValue(Option("hi")) + ", and " + decorateToStringValue(Option("hi")) + " did not contain one of (\"hi\")", thisLineNumber - 2, hiOptions), fileName, thisLineNumber - 2)
      }
    }
  }
}
