/*
 * Copyright 2001-2013 Artima, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Normalization
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class OptionShouldContainNoneOfSpec extends Spec with Matchers {

  val upperCaseEquality =
    new Equality[String] {
      def areEqual(a: String, b: Any): Boolean = a.toUpperCase == b
    }
  
  //ADDITIONAL//

  object `a Option` {

    val fumOption: Option[String] = Option("fum")
    val toOption: Option[String] = Option("to")

    object `when used with contain noneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumOption should contain noneOf ("fee", "fie", "foe", "fam")
        val e1 = intercept[TestFailedException] {
          fumOption should contain noneOf ("fee", "fie", "foe", "fum")
        }
        e1.failedCodeFileName.get should be ("OptionShouldContainNoneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedOneOfElements", decorateToStringValue(fumOption), "\"fee\", \"fie\", \"foe\", \"fum\""))
        // Contains duplicate elements in the right list
        val e2 = intercept[IllegalArgumentException] {
          fumOption should contain noneOf ("fee", "fam", "foe", "fam")
        }
        e2.getMessage should be (Resources("noneOfDuplicate", "\"fam\""))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseEquality
        fumOption should contain noneOf ("FEE", "FAM", "FOE", "FU")
        intercept[TestFailedException] {
          fumOption should contain noneOf ("FEE", "FUM", "FOE", "FU")
        }
      }
      def `should use an explicitly provided Equality` {
        (fumOption should contain noneOf ("FEE", "FAM", "FOE", "FU")) (decided by upperCaseEquality)
        intercept[TestFailedException] {
          (fumOption should contain noneOf ("FEE", "FUM", "FOE", "FU")) (decided by upperCaseEquality)
        }
        intercept[TestFailedException] {
          (fumOption should contain noneOf (" FEE ", " FIE ", " FOE ", " FUM ")) (after being lowerCased and trimmed)
        }
        (fumOption should contain noneOf (" FEE ", " FIE ", " FOE ", " FAM ")) (after being lowerCased and trimmed)
      }
    }

    object `when used with (contain noneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {

        fumOption should (contain noneOf ("fee", "fie", "foe", "fam"))
        val e1 = intercept[TestFailedException] {
          fumOption should (contain noneOf ("fee", "fie", "foe", "fum"))
        }
        e1.failedCodeFileName.get should be ("OptionShouldContainNoneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedOneOfElements", decorateToStringValue(fumOption), "\"fee\", \"fie\", \"foe\", \"fum\""))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseEquality
        fumOption should (contain noneOf ("FEE", "FAM", "FOE", "FU"))
        intercept[TestFailedException] {
          fumOption should (contain noneOf ("FEE", "FUM", "FOE", "FU"))
        }
      }
      def `should use an explicitly provided Equality` {
        (fumOption should (contain noneOf ("FEE", "FAM", "FOE", "FU"))) (decided by upperCaseEquality)
        intercept[TestFailedException] {
          (fumOption should (contain noneOf ("FEE", "FUM", "FOE", "FU"))) (decided by upperCaseEquality)
        }
        fumOption should (contain noneOf (" FEE ", " FIE ", " FOE ", " FUM "))
        intercept[TestFailedException] {
          (fumOption should (contain noneOf (" FEE ", " FIE ", " FOE ", " FUM "))) (after being lowerCased and trimmed)
        }
      }
    }

    object `when used with not contain noneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumOption should not contain noneOf ("fee", "fie", "foe", "fum")
        val e1 = intercept[TestFailedException] {
          fumOption should not contain noneOf ("happy", "birthday", "to", "you")
        }
        e1.failedCodeFileName.get should be ("OptionShouldContainNoneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainOneOfElements", decorateToStringValue(fumOption), "\"happy\", \"birthday\", \"to\", \"you\""))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseEquality
        fumOption should not contain noneOf ("FEE", "FIE", "FOE", "FUM")
        intercept[TestFailedException] {
          fumOption should not contain noneOf ("HAPPY", "BIRTHDAY", "TO", "YOU")
        }
      }
      def `should use an explicitly provided Equality` {
        (fumOption should not contain noneOf ("FEE", "FIE", "FOE", "FUM")) (decided by upperCaseEquality)
        intercept[TestFailedException] {
          (fumOption should not contain noneOf ("HAPPY", "BIRTHDAY", "TO", "YOU")) (decided by upperCaseEquality)
        }
        (fumOption should not contain noneOf (" FEE ", " FIE ", " FOE ", " FUM ")) (after being lowerCased and trimmed)
        intercept[TestFailedException] {
          fumOption should not contain noneOf (" FEE ", " FIE ", " FOE ", " FUM ")
        }
      }
    }

    object `when used with (not contain noneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        toOption should (not contain noneOf ("happy", "birthday", "to", "you"))
        val e1 = intercept[TestFailedException] {
          toOption should (not contain noneOf ("fee", "fie", "foe", "fum"))
        }
        e1.failedCodeFileName.get should be ("OptionShouldContainNoneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainOneOfElements", decorateToStringValue(toOption), "\"fee\", \"fie\", \"foe\", \"fum\""))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseEquality
        toOption should (not contain noneOf ("HAPPY", "BIRTHDAY", "TO", "YOU"))
        intercept[TestFailedException] {
          toOption should (not contain noneOf ("happy", "birthday", "to", "you"))
        }
      }
      def `should use an explicitly provided Equality` {
        (toOption should (not contain noneOf ("HAPPY", "BIRTHDAY", "TO", "YOU"))) (decided by upperCaseEquality)
        intercept[TestFailedException] {
          (toOption should (not contain noneOf ("happy", "birthday", "to", "you"))) (decided by upperCaseEquality)
        }
        (toOption should (not contain noneOf (" HAPPY ", " BIRTHDAY ", " TO ", " YOU "))) (after being lowerCased and trimmed)
        intercept[TestFailedException] {
          toOption should (not contain noneOf (" HAPPY ", " BIRTHDAY ", " TO ", " YOU "))
        }
      }
    }
  }

  object `a collection of Options` {

    val list1s: Vector[Option[Int]] = Vector(Option(1), Option(1), Option(1))
    val lists: Vector[Option[Int]] = Vector(Option(1), Option(1), Option(2))
    val nils: Vector[Option[Int]] = Vector(None, None, None)
    val listsOption: Vector[Option[Int]] = Vector(Option(1), Option(1), None)
    val hiOptions: Vector[Option[String]] = Vector(Option("hi"), Option("hi"), Option("hi"))
    val toOptions: Vector[Option[String]] = Vector(Option("to"), Option("to"), Option("to"))

    object `when used with contain noneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should contain noneOf (2, 3, 4)
        atLeast (2, lists) should contain noneOf (8, 3, 4)
        atMost (2, lists) should contain noneOf (2, 3, 4)
        no (lists) should contain noneOf (1, 2, 5)

        val e1 = intercept[TestFailedException] {
          all (lists) should contain noneOf (2, 3, 4)
        }
        e1.failedCodeFileName.get should be ("OptionShouldContainNoneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(Option(2)) + " contained one of (2, 3, 4) (OptionShouldContainNoneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))
      }

      def `should use the implicit Equality in scope` {
        all (hiOptions) should contain noneOf ("ho")
        intercept[TestFailedException] {
          all (hiOptions) should contain noneOf ("hi")
        }
        implicit val ise = upperCaseEquality
        all (hiOptions) should contain noneOf ("hi")
        intercept[TestFailedException] {
          all (hiOptions) should contain noneOf ("HI")
        }
      }
      def `should use an explicitly provided Equality` {
        (all (hiOptions) should contain noneOf ("hi")) (decided by upperCaseEquality)
        intercept[TestFailedException] {
          (all (hiOptions) should contain noneOf ("HI")) (decided by upperCaseEquality)
        }
        implicit val ise = upperCaseEquality
        (all (hiOptions) should contain noneOf ("ho")) (decided by defaultEquality[String])
        intercept[TestFailedException] {
          (all (hiOptions) should contain noneOf ("hi")) (decided by defaultEquality[String])
        }
      }
    }

    object `when used with (contain noneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain noneOf (2, 3, 4))
        atLeast (2, lists) should (contain noneOf (2, 3, 4))
        atMost (2, lists) should (contain noneOf (2, 3, 4))
        no (lists) should (contain noneOf (1, 2, 5))

        val e1 = intercept[TestFailedException] {
          all (lists) should (contain noneOf (2, 3, 4))
        }
        e1.failedCodeFileName.get should be ("OptionShouldContainNoneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue(Option(2)) + " contained one of (2, 3, 4) (OptionShouldContainNoneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))
      }

      def `should use the implicit Equality in scope` {
        all (hiOptions) should (contain noneOf ("ho"))
        intercept[TestFailedException] {
          all (hiOptions) should (contain noneOf ("hi"))
        }
        implicit val ise = upperCaseEquality
        all (hiOptions) should (contain noneOf ("hi"))
        intercept[TestFailedException] {
          all (hiOptions) should (contain noneOf ("HI"))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (hiOptions) should (contain noneOf ("hi"))) (decided by upperCaseEquality)
        intercept[TestFailedException] {
          (all (hiOptions) should (contain noneOf ("HI"))) (decided by upperCaseEquality)
        }
        implicit val ise = upperCaseEquality
        (all (hiOptions) should (contain noneOf ("ho"))) (decided by defaultEquality[String])
        intercept[TestFailedException] {
          (all (hiOptions) should (contain noneOf ("hi"))) (decided by defaultEquality[String])
        }
      }
    }

    object `when used with not contain noneOf (...) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (toOptions) should not contain noneOf ("happy", "birthday", "to", "you")
        val e1 = intercept[TestFailedException] {
          all (toOptions) should not contain noneOf ("fee", "fie", "foe", "fum")
        }
        e1.failedCodeFileName.get should be ("OptionShouldContainNoneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(Option("to")) + " did not contain one of (\"fee\", \"fie\", \"foe\", \"fum\") (OptionShouldContainNoneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(toOptions)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseEquality
        all (toOptions) should not contain noneOf ("HAPPY", "BIRTHDAY", "TO", "YOU")
        intercept[TestFailedException] {
          all (toOptions) should not contain noneOf ("happy", "birthday", "to", "you")
        }
      }
      def `should use an explicitly provided Equality` {
        (all (toOptions) should not contain noneOf ("HAPPY", "BIRTHDAY", "TO", "YOU")) (decided by upperCaseEquality)
        intercept[TestFailedException] {
          (all (toOptions) should not contain noneOf ("happy", "birthday", "to", "you")) (decided by upperCaseEquality)
        }
        (all (toOptions) should not contain noneOf (" HAPPY ", " BIRTHDAY ", " TO ", " YOU ")) (after being lowerCased and trimmed)
        intercept[TestFailedException] {
          all (toOptions) should not contain noneOf (" HAPPY ", " BIRTHDAY ", " TO ", " YOU ")
        }
      }
    }

    object `when used with (not contain noneOf (...)) syntax` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (toOptions) should (not contain noneOf ("happy", "birthday", "to", "you"))
        val e1 = intercept[TestFailedException] {
          all (toOptions) should (not contain noneOf ("fee", "fie", "foe", "fum"))
        }
        e1.failedCodeFileName.get should be ("OptionShouldContainNoneOfSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue(Option("to")) + " did not contain one of (\"fee\", \"fie\", \"foe\", \"fum\") (OptionShouldContainNoneOfSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(toOptions)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseEquality
        all (toOptions) should (not contain noneOf ("HAPPY", "BIRTHDAY", "TO", "YOU"))
        intercept[TestFailedException] {
          all (toOptions) should (not contain noneOf ("happy", "birthday", "to", "you"))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (toOptions) should (not contain noneOf ("HAPPY", "BIRTHDAY", "TO", "YOU"))) (decided by upperCaseEquality)
        intercept[TestFailedException] {
          (all (toOptions) should (not contain noneOf ("happy", "birthday", "to", "you"))) (decided by upperCaseEquality)
        }
        (all (toOptions) should (not contain noneOf (" HAPPY ", " BIRTHDAY ", " TO ", " YOU "))) (after being lowerCased and trimmed)
        intercept[TestFailedException] {
          all (toOptions) should (not contain noneOf (" HAPPY ", " BIRTHDAY ", " TO ", " YOU "))
        }
      }
    }
  }
}
