/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Normalization
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue

class StringShouldContainAtLeastOneOfLogicalAndSpec extends Spec with Matchers {
  
  val invertedStringEquality =
    new Equality[Char] {
      def areEqual(a: Char, b: Any): Boolean = a != b
    }
  
  val invertedListOfStringEquality = 
    new Equality[String] {
      def areEqual(a: String, b: Any): Boolean = a != b
    }
  
  val lowerCased: Normalization[Char] = new Normalization[Char] {
    def normalized(c: Char): Char = c.toString.toLowerCase.toCharArray()(0)
    def normalizedIfInstanceOfA(b: Any) =
      b match {
        case c: Char => normalized(c)
        case _ => b
      }
    }
  
  val fileName: String = "StringShouldContainAtLeastOneOfLogicalAndSpec.scala"
  
  object `a List` {
    
    val fumList: String = "u"
    val toList: String = "o"
    
    object `when used with (contain oneOf (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (contain atLeastOneOf ('e', 'i', 'p', 'u') and contain atLeastOneOf('i', 'e', 'u', 'p'))
        val e1 = intercept[TestFailedException] {
          fumList should (contain atLeastOneOf ('h', 'b', 'o', 'y') and contain atLeastOneOf ('e', 'i', 'p', 'u'))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "'h', 'b', 'o', 'y'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (contain atLeastOneOf ('e', 'i', 'p', 'u') and contain atLeastOneOf ('h', 'b', 'o', 'y'))
        }
        checkMessageStackDepth(e2, Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'") + ", but " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "'h', 'b', 'o', 'y'"), fileName, thisLineNumber - 2)
      }

      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        fumList should (contain atLeastOneOf ('h', 'b', 'o', 'y') and contain atLeastOneOf ('h', 'a', 'n', 'd'))
        val e1 = intercept[TestFailedException] {
          fumList should (contain atLeastOneOf ('u', 'u', 'u', 'u') and contain atLeastOneOf ('u', 'u', 'u'))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "'u', 'u', 'u', 'u'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (contain atLeastOneOf ('h', 'b', 'o', 'y') and (contain atLeastOneOf ('u', 'u', 'u')))
        }
        checkMessageStackDepth(e2, Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "'h', 'b', 'o', 'y'") + ", but " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "'u', 'u', 'u'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (contain atLeastOneOf ('h', 'b', 'o', 'y') and contain atLeastOneOf ('h', 'a', 'n', 'd'))) (decided by invertedStringEquality, decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (contain atLeastOneOf ('h', 'b', 'o', 'y') and contain atLeastOneOf ('u', 'u', 'u'))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "'h', 'b', 'o', 'y'") + ", but " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "'u', 'u', 'u'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (contain atLeastOneOf ('u', 'u', 'u') and contain atLeastOneOf ('h', 'b', 'o', 'y'))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "'u', 'u', 'u'"), fileName, thisLineNumber - 2)
        (fumList should (contain atLeastOneOf ('E', 'I', 'P', 'U') and contain atLeastOneOf ('E', 'I', 'P', 'U'))) (after being lowerCased, after being lowerCased)
      }
    }
    
    object `when used with (equal (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (equal (fumList) and contain atLeastOneOf('i', 'e', 'u', 'p'))
        val e1 = intercept[TestFailedException] {
          fumList should (equal (toList) and contain atLeastOneOf ('e', 'i', 'p', 'u'))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (equal (fumList) and contain atLeastOneOf ('h', 'b', 'o', 'y'))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "'h', 'b', 'o', 'y'"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        fumList should (equal (fumList) and contain atLeastOneOf ('h', 'a', 'n', 'd'))
        val e1 = intercept[TestFailedException] {
          fumList should (equal (toList) and contain atLeastOneOf ('u', 'u', 'u'))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (equal (fumList) and (contain atLeastOneOf ('u', 'u', 'u')))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "'u', 'u', 'u'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (equal (toList) and contain atLeastOneOf ('h', 'a', 'n', 'd'))) (decided by invertedListOfStringEquality, decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (equal (toList) and contain atLeastOneOf ('u', 'u', 'u'))) (decided by invertedListOfStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", but " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "'u', 'u', 'u'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (equal (fumList) and contain atLeastOneOf ('h', 'b', 'o', 'y'))) (decided by invertedListOfStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        (fumList should (equal (toList) and contain atLeastOneOf ('E', 'I', 'P', 'U'))) (decided by invertedListOfStringEquality, after being lowerCased)
      }
    }
    
    object `when used with (legacyEqual (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (legacyEqual (fumList) and contain atLeastOneOf('i', 'e', 'u', 'p'))
        val e1 = intercept[TestFailedException] {
          fumList should (legacyEqual (toList) and contain atLeastOneOf ('e', 'i', 'p', 'u'))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (legacyEqual (fumList) and contain atLeastOneOf ('h', 'b', 'o', 'y'))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "'h', 'b', 'o', 'y'"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        fumList should (legacyEqual (fumList) and contain atLeastOneOf ('h', 'a', 'n', 'd'))
        val e1 = intercept[TestFailedException] {
          fumList should (legacyEqual (toList) and contain atLeastOneOf ('u', 'u', 'u'))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (legacyEqual (fumList) and (contain atLeastOneOf ('u', 'u', 'u')))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "'u', 'u', 'u'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (legacyEqual (fumList) and contain atLeastOneOf ('h', 'a', 'n', 'd'))) (decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (legacyEqual (fumList) and contain atLeastOneOf ('u', 'u', 'u'))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "'u', 'u', 'u'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (legacyEqual (toList) and contain atLeastOneOf ('h', 'b', 'o', 'y'))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
        (fumList should (legacyEqual (fumList) and contain atLeastOneOf ('E', 'I', 'P', 'U'))) (after being lowerCased)
      }
    }

    object `when used with (contain oneOf (..) and legacyEqual (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (contain atLeastOneOf('i', 'e', 'u', 'p') and legacyEqual (fumList))
        val e1 = intercept[TestFailedException] {
          fumList should (contain atLeastOneOf ('e', 'i', 'p', 'u') and legacyEqual (toList))
        }
        checkMessageStackDepth(e1, Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'") + ", but " + Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (contain atLeastOneOf ('h', 'b', 'o', 'y') and legacyEqual (fumList))
        }
        checkMessageStackDepth(e2, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "'h', 'b', 'o', 'y'"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        fumList should (contain atLeastOneOf ('h', 'a', 'n', 'd') and legacyEqual (fumList))
        val e1 = intercept[TestFailedException] {
          fumList should (contain atLeastOneOf ('u', 'u', 'u') and legacyEqual (toList))
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "'u', 'u', 'u'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (contain atLeastOneOf ('u', 'u', 'u') and (legacyEqual (fumList)))
        }
        checkMessageStackDepth(e2, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "'u', 'u', 'u'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (contain atLeastOneOf ('h', 'a', 'n', 'd') and legacyEqual (fumList))) (decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (contain atLeastOneOf ('u', 'u', 'u') and legacyEqual (fumList))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "'u', 'u', 'u'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (contain atLeastOneOf ('h', 'b', 'o', 'y') and legacyEqual (toList))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "'h', 'b', 'o', 'y'") + ", but " + Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
        (fumList should (contain atLeastOneOf ('E', 'I', 'P', 'U') and legacyEqual (fumList))) (after being lowerCased)
      }
    }
    
    object `when used with (not contain oneOf (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not contain atLeastOneOf ("fee", "fie", "foe", "fuu") and not contain atLeastOneOf("fie", "fee", "fuu", "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (not contain atLeastOneOf ('e', 'i', 'p', 'u') and not contain atLeastOneOf ('h', 'b', 'o', 'y'))
        }
        checkMessageStackDepth(e1, Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not contain atLeastOneOf ('h', 'b', 'o', 'y') and not contain atLeastOneOf ('e', 'i', 'p', 'u'))
        }
        checkMessageStackDepth(e2, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "'h', 'b', 'o', 'y'") + ", but " + Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        fumList should (not contain atLeastOneOf ('u', 'u', 'u', 'u') and not contain atLeastOneOf ('u', 'u', 'u'))
        val e1 = intercept[TestFailedException] {
          fumList should (not contain atLeastOneOf ('h', 'b', 'o', 'y') and not contain atLeastOneOf ('u', 'u', 'u'))
        }
        checkMessageStackDepth(e1, Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "'h', 'b', 'o', 'y'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not contain atLeastOneOf ('u', 'u', 'u') and (not contain atLeastOneOf ('h', 'b', 'o', 'y')))
        }
        checkMessageStackDepth(e2, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "'u', 'u', 'u'") + ", but " + Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "'h', 'b', 'o', 'y'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (not contain atLeastOneOf ('u', 'u', 'u', 'u') and not contain atLeastOneOf ('u', 'u', 'u'))) (decided by invertedStringEquality, decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not contain atLeastOneOf ('u', 'u', 'u') and not contain atLeastOneOf ('h', 'b', 'o', 'y'))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainAtLeastOneOf", decorateToStringValue(fumList), "'u', 'u', 'u'") + ", but " + Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "'h', 'b', 'o', 'y'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (not contain atLeastOneOf ('h', 'b', 'o', 'y') and not contain atLeastOneOf ('u', 'u', 'u'))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "'h', 'b', 'o', 'y'"), fileName, thisLineNumber - 2)
        (fumList should (contain atLeastOneOf ('E', 'I', 'P', 'U') and contain atLeastOneOf ('E', 'I', 'P', 'U'))) (after being lowerCased, after being lowerCased)
      }
    }
    
    object `when used with (not equal (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not equal (toList) and not contain atLeastOneOf("fie", "fee", "fuu", "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (not equal (fumList) and not contain atLeastOneOf ('h', 'b', 'o', 'y'))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not equal (toList) and not contain atLeastOneOf ('e', 'i', 'p', 'u'))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", but " + Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        fumList should (not equal (toList) and not contain atLeastOneOf ('u', 'u', 'u'))
        val e1 = intercept[TestFailedException] {
          fumList should (not equal (fumList) and not contain atLeastOneOf ('u', 'u', 'u'))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not equal (toList) and (not contain atLeastOneOf ('h', 'b', 'o', 'y')))
        }
        checkMessageStackDepth(e2, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", but " + Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "'h', 'b', 'o', 'y'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (not equal (fumList) and not contain atLeastOneOf ('u', 'u', 'u'))) (decided by invertedListOfStringEquality, decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not equal (fumList) and not contain atLeastOneOf ('h', 'b', 'o', 'y'))) (decided by invertedListOfStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", but " + Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "'h', 'b', 'o', 'y'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (not equal (toList) and not contain atLeastOneOf ('u', 'u', 'u'))) (decided by invertedListOfStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(toList)), fileName, thisLineNumber - 2)
        (fumList should (not contain atLeastOneOf ('E', 'I', 'P', 'D') and not contain atLeastOneOf ('E', 'I', 'P', 'D'))) (after being lowerCased, after being lowerCased)
      }
    }
    
    object `when used with (not be (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should (not be (toList) and not contain atLeastOneOf("fie", "fee", "fuu", "foe"))
        val e1 = intercept[TestFailedException] {
          fumList should (not be (fumList) and not contain atLeastOneOf ('h', 'b', 'o', 'y'))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not be (toList) and not contain atLeastOneOf ('e', 'i', 'p', 'u'))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", but " + Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "'e', 'i', 'p', 'u'"), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        fumList should (not be (toList) and not contain atLeastOneOf ('u', 'u', 'u'))
        val e1 = intercept[TestFailedException] {
          fumList should (not be (fumList) and not contain atLeastOneOf ('u', 'u', 'u'))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          fumList should (not be (toList) and (not contain atLeastOneOf ('h', 'b', 'o', 'y')))
        }
        checkMessageStackDepth(e2, Resources("wasNotEqualTo", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", but " + Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "'h', 'b', 'o', 'y'"), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (fumList should (not be (toList) and not contain atLeastOneOf ('u', 'u', 'u'))) (decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not be (toList) and not contain atLeastOneOf ('h', 'b', 'o', 'y'))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasNotEqualTo", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", but " + Resources("containedAtLeastOneOf", decorateToStringValue(fumList), "'h', 'b', 'o', 'y'"), fileName, thisLineNumber - 2)
        val e2 = intercept[TestFailedException] {
          (fumList should (not be (fumList) and not contain atLeastOneOf ('u', 'u', 'u'))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)), fileName, thisLineNumber - 2)
        (fumList should (not contain atLeastOneOf ('E', 'I', 'P', 'D') and not contain atLeastOneOf ('E', 'I', 'P', 'D'))) (after being lowerCased, after being lowerCased)
      }
    }
    
  }
  
  object `collection of Lists` {
    
    val list1s: Vector[String] = Vector("1", "1", "1")
    val lists: Vector[String] = Vector("1", "1", "2")
    val nils: Vector[String] = Vector("", "", "")
    val listsString: Vector[String] = Vector("1", "1", "")
    val hiLists: Vector[String] = Vector("i", "i", "i")
    val toLists: Vector[String] = Vector("o", "o", "o")
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + decorateToStringValue(left)
    
    object `when used with (contain oneOf (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain atLeastOneOf ('3', '2', '1') and contain atLeastOneOf ('1', '3', '4'))
        atLeast (2, lists) should (contain atLeastOneOf ('3', '1', '5') and contain atLeastOneOf ('1', '3', '4'))
        atMost (2, lists) should (contain atLeastOneOf ('3', '2', '8') and contain atLeastOneOf ('2', '3', '4'))
        no (lists) should (contain atLeastOneOf ('3', '6', '9') and contain atLeastOneOf ('3', '4', '5'))
        no (nils) should (contain atLeastOneOf ('1', '2', '8') and contain atLeastOneOf ('1', '3', '4'))
        no (listsString) should (contain atLeastOneOf('3', '8', '5') and contain atLeastOneOf ('3', '4', '5'))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain atLeastOneOf ('1', '6', '8') and contain atLeastOneOf ('1', '3', '4'))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue("2") + " did not contain at least one of ('1', '6', '8')", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (contain atLeastOneOf ('1', '2', '8') and contain atLeastOneOf ('1', '3', '4'))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue("2") + " contained at least one of ('1', '2', '8'), but " + decorateToStringValue("2") + " did not contain at least one of ('1', '3', '4')", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (nils) should (contain atLeastOneOf ('i', 'l') and contain atLeastOneOf ('o', 'e', 'd'))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue("") + " did not contain at least one of ('i', 'l')", thisLineNumber - 2, nils), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiLists) should (contain atLeastOneOf ('i', 'l') and contain atLeastOneOf ('o', 'e', 'd'))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue("i") + " contained at least one of ('i', 'l'), but " + decorateToStringValue("i") + " did not contain at least one of ('o', 'e', 'd')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e5 = intercept[TestFailedException] {
          all (listsString) should (contain atLeastOneOf ('1', '3', '4') and contain atLeastOneOf ('1', '3'))
        }
        checkMessageStackDepth(e5, allErrMsg(2, decorateToStringValue("") + " did not contain at least one of ('1', '3', '4')", thisLineNumber - 2, listsString), fileName, thisLineNumber - 2)
        
        val e6 = intercept[TestFailedException] {
          all (lists) should (contain atLeastOneOf ('1', '2', '8') and contain atLeastOneOf ('1', '3', '4'))
        }
        checkMessageStackDepth(e6, allErrMsg(2, decorateToStringValue("2") + " contained at least one of ('1', '2', '8'), but " + decorateToStringValue("2") + " did not contain at least one of ('1', '3', '4')", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        
        all (hiLists) should (contain atLeastOneOf ('o') and contain atLeastOneOf ('e'))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (contain atLeastOneOf ('i') and contain atLeastOneOf ('o'))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("i") + " did not contain at least one of ('i')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiLists) should (contain atLeastOneOf ('o') and contain atLeastOneOf ('i'))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue("i") + " contained at least one of ('o'), but " + decorateToStringValue("i") + " did not contain at least one of ('i')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (contain atLeastOneOf ('o') and contain atLeastOneOf ('o'))) (decided by invertedStringEquality, decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (contain atLeastOneOf ('i') and contain atLeastOneOf ('o'))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("i") + " did not contain at least one of ('i')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiLists) should (contain atLeastOneOf ('o') and contain atLeastOneOf ('i'))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue("i") + " contained at least one of ('o'), but " + decorateToStringValue("i") + " did not contain at least one of ('i')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (be (..) and contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (be ("1") and contain atLeastOneOf ('1', '3', '4'))
        atLeast (2, lists) should (be ("1") and contain atLeastOneOf ('1', '3', '4'))
        atMost (2, lists) should (be ("1") and contain atLeastOneOf ('2', '3', '4'))
        no (lists) should (be ("8") and contain atLeastOneOf ('3', '4', '5'))
        no (nils) should (be ("8") and contain atLeastOneOf ('1', '3', '4'))
        no (listsString) should (be ("8") and contain atLeastOneOf ('3', '4', '5'))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (be ("1") and contain atLeastOneOf ('1', '3', '4'))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue("[2]") + " was not equal to " + decorateToStringValue("[1]"), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (list1s) should (be ("1") and contain atLeastOneOf ('2', '3', '8'))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue("1") + " was equal to " + decorateToStringValue("1") + ", but " + decorateToStringValue("1") + " did not contain at least one of ('2', '3', '8')", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (nils) should (be ("e") and contain atLeastOneOf ('o', 'e', 'd'))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue("[]") + " was not equal to " + decorateToStringValue("[e]"), thisLineNumber - 2, nils), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiLists) should (be ("i") and contain atLeastOneOf ('o', 'e', 'd'))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue("i") + " was equal to " + decorateToStringValue("i") + ", but " + decorateToStringValue("i") + " did not contain at least one of ('o', 'e', 'd')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e5 = intercept[TestFailedException] {
          all (listsString) should (be ("1") and contain atLeastOneOf ('1', '3'))
        }
        checkMessageStackDepth(e5, allErrMsg(2, decorateToStringValue("[]") + " was not equal to " + decorateToStringValue("[1]"), thisLineNumber - 2, listsString), fileName, thisLineNumber - 2)
        
        val e6 = intercept[TestFailedException] {
          all (list1s) should (be ("1") and contain atLeastOneOf ('2', '3', '8'))
        }
        checkMessageStackDepth(e6, allErrMsg(0, decorateToStringValue("1") + " was equal to " + decorateToStringValue("1") + ", but " + decorateToStringValue("1") + " did not contain at least one of ('2', '3', '8')", thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        
        all (hiLists) should (be ("i") and contain atLeastOneOf ('e'))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (be ("o") and contain atLeastOneOf ('o'))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("[i]") + " was not equal to " + decorateToStringValue("[o]"), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiLists) should (be ("i") and contain atLeastOneOf ('i'))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue("i") + " was equal to " + decorateToStringValue("i") + ", but " + decorateToStringValue("i") + " did not contain at least one of ('i')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (be ("i") and contain atLeastOneOf ('o'))) (decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (be ("o") and contain atLeastOneOf ('o'))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("[i]") + " was not equal to " + decorateToStringValue("[o]"), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiLists) should (be ("i") and contain atLeastOneOf ('i'))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue("i") + " was equal to " + decorateToStringValue("i") + ", but " + decorateToStringValue("i") + " did not contain at least one of ('i')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not contain oneOf (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not contain atLeastOneOf ('3', '2', '8') and not contain atLeastOneOf ('8', '3', '4'))
        atLeast (2, lists) should (not contain atLeastOneOf ('3', '8', '5') and not contain atLeastOneOf ('8', '3', '4'))
        atMost (2, lists) should (not contain atLeastOneOf ('3', '6', '8') and contain atLeastOneOf ('5', '3', '4'))
        no (lists) should (not contain atLeastOneOf ('1', '2', '9') and not contain atLeastOneOf ('2', '1', '5'))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain atLeastOneOf ('2', '6', '8') and not contain atLeastOneOf ('2', '3', '4'))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue("2") + " contained at least one of ('2', '6', '8')", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not contain atLeastOneOf ('3', '6', '8') and not contain atLeastOneOf ('2', '3', '4'))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue("2") + " did not contain at least one of ('3', '6', '8'), but " + decorateToStringValue("2") + " contained at least one of ('2', '3', '4')", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hiLists) should (not contain atLeastOneOf ('i', 'l') and not contain atLeastOneOf ('o', 'e', 'd'))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue("i") + " contained at least one of ('i', 'l')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiLists) should (not contain atLeastOneOf ('o', 'e', 'd') and not contain atLeastOneOf ('i', 'l'))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue("i") + " did not contain at least one of ('o', 'e', 'd'), but " + decorateToStringValue("i") + " contained at least one of ('i', 'l')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        
        all (hiLists) should (not contain atLeastOneOf ('i') and not contain atLeastOneOf ('i'))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (not contain atLeastOneOf ('o') and not contain atLeastOneOf ('i'))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("i") + " contained at least one of ('o')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiLists) should (not contain atLeastOneOf ('i') and not contain atLeastOneOf ('o'))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue("i") + " did not contain at least one of ('i'), but " + decorateToStringValue("i") + " contained at least one of ('o')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (not contain atLeastOneOf ('i') and not contain atLeastOneOf ('i'))) (decided by invertedStringEquality, decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (not contain atLeastOneOf ('o') and not contain atLeastOneOf ('i'))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("i") + " contained at least one of ('o')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiLists) should (not contain atLeastOneOf ('i') and not contain atLeastOneOf ('o'))) (decided by invertedStringEquality, decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue("i") + " did not contain at least one of ('i'), but " + decorateToStringValue("i") + " contained at least one of ('o')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
    }
    
    object `when used with (not be (..) and not contain oneOf (..))` {
      
      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (not be ("2") and not contain atLeastOneOf ('8', '3', '4'))
        atLeast (2, lists) should (not be ("3") and not contain atLeastOneOf ('8', '3', '4'))
        atMost (2, lists) should (not be ("3") and contain atLeastOneOf ('5', '3', '4'))
        no (list1s) should (not be ("1") and not contain atLeastOneOf ('2', '1', '5'))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not be ("2") and not contain atLeastOneOf ('2', '3', '4'))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue("2") + " was equal to " + decorateToStringValue("2"), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (lists) should (not be ("3") and not contain atLeastOneOf ('2', '3', '4'))
        }
        checkMessageStackDepth(e2, allErrMsg(2, decorateToStringValue("[2]") + " was not equal to " + decorateToStringValue("[3]") + ", but " + decorateToStringValue("2") + " contained at least one of ('2', '3', '4')", thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
        
        val e3 = intercept[TestFailedException] {
          all (hiLists) should (not be ("i") and not contain atLeastOneOf ('o', 'e', 'd'))
        }
        checkMessageStackDepth(e3, allErrMsg(0, decorateToStringValue("i") + " was equal to " + decorateToStringValue("i"), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e4 = intercept[TestFailedException] {
          all (hiLists) should (not be ("o") and not contain atLeastOneOf ('i', 'l'))
        }
        checkMessageStackDepth(e4, allErrMsg(0, decorateToStringValue("[i]") + " was not equal to " + decorateToStringValue("[o]") + ", but " + decorateToStringValue("i") + " contained at least one of ('i', 'l')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use the implicit Equality in scope` {
        implicit val ise = invertedStringEquality
        
        all (hiLists) should (not be ("o") and not contain atLeastOneOf ('i'))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (not be ("i") and not contain atLeastOneOf ('i'))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("i") + " was equal to " + decorateToStringValue("i"), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          all (hiLists) should (not be ("o") and not contain atLeastOneOf ('o'))
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue("[i]") + " was not equal to " + decorateToStringValue("[o]") + ", but " + decorateToStringValue("i") + " contained at least one of ('o')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (not be ("o") and not contain atLeastOneOf ('i'))) (decided by invertedStringEquality)
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (not be ("i") and not contain atLeastOneOf ('i'))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("i") + " was equal to " + decorateToStringValue("i"), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
        
        val e2 = intercept[TestFailedException] {
          (all (hiLists) should (not be ("o") and not contain atLeastOneOf ('o'))) (decided by invertedStringEquality)
        }
        checkMessageStackDepth(e2, allErrMsg(0, decorateToStringValue("[i]") + " was not equal to " + decorateToStringValue("[o]") + ", but " + decorateToStringValue("i") + " contained at least one of ('o')", thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
    }
  }
}
