/*
* Copyright 2001-2013 Artima, Inc.
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Normalization
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue
import scala.collection.mutable.LinkedList

class StringShouldContainTheSameElementsInOrderAsLogicalOrSpec extends FreeSpec with Matchers {
  
  val invertedStringEquality =
    new Equality[Char] {
      def areEqual(a: Char, b: Any): Boolean = a != b
    }
  
  val invertedListOfStringEquality = 
    new Equality[String] {
      def areEqual(a: String, b: Any): Boolean = a != b
    }
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: List[_] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case _ => value
    }
  
  val upperCaseStringEquality =
    new Equality[Char] {
      def areEqual(a: Char, b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  val lowerCased: Normalization[Char] = new Normalization[Char] {
    def normalized(c: Char): Char = c.toString.toLowerCase.toCharArray()(0)
    def normalizedIfInstanceOfA(b: Any) =
      b match {
        case c: Char => normalized(c)
        case _ => b
      }
    }
  
  val fileName: String = "StringShouldContainTheSameElementsInOrderAsLogicalOrSpec.scala"
  
  "a List" - {
    
    val fumList: String = "upie"
    val toList: String = "yobh"
    
    "when used with (contain theSameElementsInOrderAs xx or contain theSameElementsInOrderAs xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumList should (contain theSameElementsInOrderAs LinkedList('u', 'p', 'i', 'e') or contain theSameElementsInOrderAs LinkedList('u', 'p', 'i', 'e'))
        fumList should (contain theSameElementsInOrderAs LinkedList('e', 'i', 'p', 'u') or contain theSameElementsInOrderAs LinkedList('u', 'p', 'i', 'e'))
        fumList should (contain theSameElementsInOrderAs LinkedList('u', 'p', 'i', 'e') or contain theSameElementsInOrderAs LinkedList('e', 'i', 'p', 'u'))
        val e1 = intercept[TestFailedException] {
          fumList should (contain theSameElementsInOrderAs LinkedList('e', 'i', 'p', 'u') or contain theSameElementsInOrderAs LinkedList('h', 'b', 'o', 'y'))
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(LinkedList('e', 'i', 'p', 'u'))) + ", and " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(LinkedList('h', 'b', 'o', 'y'))), fileName, thisLineNumber - 2)
      }

      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumList should (contain theSameElementsInOrderAs LinkedList('U', 'P', 'I', 'E') or contain theSameElementsInOrderAs LinkedList('U', 'P', 'I', 'E'))
        fumList should (contain theSameElementsInOrderAs LinkedList('E', 'I', 'P', 'U') or contain theSameElementsInOrderAs LinkedList('U', 'P', 'I', 'E'))
        fumList should (contain theSameElementsInOrderAs LinkedList('U', 'P', 'I', 'E') or contain theSameElementsInOrderAs LinkedList('E', 'I', 'P', 'U'))
        val e1 = intercept[TestFailedException] {
          fumList should (contain theSameElementsInOrderAs LinkedList('E', 'I', 'P', 'U') or (contain theSameElementsInOrderAs LinkedList('I', 'E', 'A', 'P')))
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(LinkedList('E', 'I', 'P', 'U'))) + ", and " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(LinkedList('I', 'E', 'A', 'P'))), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumList should (contain theSameElementsInOrderAs LinkedList('U', 'P', 'I', 'E') or contain theSameElementsInOrderAs LinkedList('U', 'P', 'I', 'E'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumList should (contain theSameElementsInOrderAs LinkedList('E', 'I', 'P', 'U') or contain theSameElementsInOrderAs LinkedList('U', 'P', 'I', 'E'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumList should (contain theSameElementsInOrderAs LinkedList('U', 'P', 'I', 'E') or contain theSameElementsInOrderAs LinkedList('E', 'I', 'P', 'U'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (contain theSameElementsInOrderAs LinkedList('E', 'I', 'P', 'U') or contain theSameElementsInOrderAs LinkedList('I', 'E', 'A', 'P'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(LinkedList('E', 'I', 'P', 'U'))) + ", and " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(LinkedList('I', 'E', 'A', 'P'))), fileName, thisLineNumber - 2)
        (fumList should (contain theSameElementsInOrderAs LinkedList('U', 'P', 'I', 'E') or contain theSameElementsInOrderAs LinkedList('U', 'P', 'I', 'E'))) (after being lowerCased, after being lowerCased)
      }
    }
    
    "when used with (equal xx and contain theSameElementsInOrderAs xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumList should (equal (fumList) or contain theSameElementsInOrderAs LinkedList('u', 'p', 'i', 'e'))
        fumList should (equal (toList) or contain theSameElementsInOrderAs LinkedList('u', 'p', 'i', 'e'))
        fumList should (equal (fumList) or contain theSameElementsInOrderAs LinkedList('e', 'i', 'p', 'u'))
        val e1 = intercept[TestFailedException] {
          fumList should (equal (toList) or contain theSameElementsInOrderAs LinkedList('e', 'i', 'p', 'u'))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", and " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(LinkedList('e', 'i', 'p', 'u'))), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumList should (equal (fumList) or contain theSameElementsInOrderAs LinkedList('U', 'P', 'I', 'E'))
        fumList should (equal (toList) or contain theSameElementsInOrderAs LinkedList('U', 'P', 'I', 'E'))
        fumList should (equal (fumList) or contain theSameElementsInOrderAs LinkedList('E', 'I', 'P', 'U'))
        val e1 = intercept[TestFailedException] {
          fumList should (equal (toList) or (contain theSameElementsInOrderAs LinkedList('E', 'I', 'P', 'U')))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", and " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(LinkedList('E', 'I', 'P', 'U'))), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumList should (equal (toList) or contain theSameElementsInOrderAs LinkedList('U', 'P', 'I', 'E'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        (fumList should (equal (fumList) or contain theSameElementsInOrderAs LinkedList('U', 'P', 'I', 'E'))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        (fumList should (equal (toList) or contain theSameElementsInOrderAs (LinkedList('E', 'I', 'P', 'U')))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (equal (fumList) or contain theSameElementsInOrderAs (LinkedList('E', 'I', 'P', 'U')))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(LinkedList('E', 'I', 'P', 'U'))), fileName, thisLineNumber - 2)
        (fumList should (equal (toList) or contain theSameElementsInOrderAs LinkedList('E', 'I', 'P', 'U'))) (decided by invertedListOfStringEquality, after being lowerCased)
      }
    }
    
    "when used with (legacyEqual xx and contain theSameElementsInOrderAs xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumList should (legacyEqual (fumList) or contain theSameElementsInOrderAs LinkedList('u', 'p', 'i', 'e'))
        fumList should (legacyEqual (toList) or contain theSameElementsInOrderAs LinkedList('u', 'p', 'i', 'e'))
        fumList should (legacyEqual (fumList) or contain theSameElementsInOrderAs LinkedList('e', 'i', 'p', 'u'))
        val e1 = intercept[TestFailedException] {
          fumList should (legacyEqual (toList) or contain theSameElementsInOrderAs LinkedList('e', 'i', 'p', 'u'))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", and " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(LinkedList('e', 'i', 'p', 'u'))), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumList should (legacyEqual (fumList) or contain theSameElementsInOrderAs LinkedList('U', 'P', 'I', 'E'))
        fumList should (legacyEqual (toList) or contain theSameElementsInOrderAs LinkedList('U', 'P', 'I', 'E'))
        fumList should (legacyEqual (fumList) or contain theSameElementsInOrderAs LinkedList('E', 'I', 'P', 'U'))
        val e1 = intercept[TestFailedException] {
          fumList should (legacyEqual (toList) or (contain theSameElementsInOrderAs LinkedList('E', 'I', 'P', 'U')))
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", and " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(LinkedList('E', 'I', 'P', 'U'))), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumList should (legacyEqual (fumList) or contain theSameElementsInOrderAs LinkedList('U', 'P', 'I', 'E'))) (decided by upperCaseStringEquality)
        (fumList should (legacyEqual (toList) or contain theSameElementsInOrderAs LinkedList('U', 'P', 'I', 'E'))) (decided by upperCaseStringEquality)
        (fumList should (legacyEqual (fumList) or contain theSameElementsInOrderAs LinkedList('E', 'I', 'P', 'U'))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (legacyEqual (toList) or contain theSameElementsInOrderAs LinkedList('E', 'I', 'P', 'U'))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")) + ", and " + Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(LinkedList('E', 'I', 'P', 'U'))), fileName, thisLineNumber - 2)
        (fumList should (legacyEqual (fumList) or contain theSameElementsInOrderAs LinkedList('U', 'P', 'I', 'E'))) (after being lowerCased)
      }
    }

    "when used with (contain theSameElementsInOrderAs xx and legacyEqual xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumList should (contain theSameElementsInOrderAs LinkedList('u', 'p', 'i', 'e') or legacyEqual (fumList))
        fumList should (contain theSameElementsInOrderAs LinkedList('e', 'i', 'p', 'u') or legacyEqual (fumList))
        fumList should (contain theSameElementsInOrderAs LinkedList('u', 'p', 'i', 'e') or legacyEqual (toList))
        val e1 = intercept[TestFailedException] {
          fumList should (contain theSameElementsInOrderAs LinkedList('e', 'i', 'p', 'u') or legacyEqual (toList))
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(LinkedList('e', 'i', 'p', 'u'))) + ", and " + Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumList should (contain theSameElementsInOrderAs LinkedList('U', 'P', 'I', 'E') or legacyEqual (fumList))
        fumList should (contain theSameElementsInOrderAs LinkedList('E', 'I', 'P', 'U') or legacyEqual (fumList))
        fumList should (contain theSameElementsInOrderAs LinkedList('U', 'P', 'I', 'E') or legacyEqual (toList))
        val e1 = intercept[TestFailedException] {
          fumList should (contain theSameElementsInOrderAs LinkedList('E', 'I', 'P', 'U') or legacyEqual (toList))
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(LinkedList('E', 'I', 'P', 'U'))) + ", and " + Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumList should (contain theSameElementsInOrderAs LinkedList('U', 'P', 'I', 'E') or legacyEqual (fumList))) (decided by upperCaseStringEquality)
        (fumList should (contain theSameElementsInOrderAs LinkedList('E', 'I', 'P', 'U') or legacyEqual (fumList))) (decided by upperCaseStringEquality)
        (fumList should (contain theSameElementsInOrderAs LinkedList('U', 'P', 'I', 'E') or legacyEqual (toList))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (contain theSameElementsInOrderAs LinkedList('E', 'I', 'P', 'U') or legacyEqual (toList))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(LinkedList('E', 'I', 'P', 'U'))) + ", and " + Resources("didNotEqual", decorateToStringValue("[" + fumList + "]"), decorateToStringValue("[" + toList + "]")), fileName, thisLineNumber - 2)
        (fumList should (contain theSameElementsInOrderAs LinkedList('U', 'P', 'I', 'E') or legacyEqual (fumList))) (after being lowerCased)
      }
    }
    
    "when used with (not contain theSameElementsInOrderAs xx and not contain theSameElementsInOrderAs xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumList should (not contain theSameElementsInOrderAs (LinkedList('e', 'i', 'p', 'u')) or not contain theSameElementsInOrderAs (LinkedList('e', 'i', 'p', 'u')))
        fumList should (not contain theSameElementsInOrderAs (LinkedList('u', 'p', 'i', 'e')) or not contain theSameElementsInOrderAs (LinkedList('e', 'i', 'p', 'u')))
        fumList should (not contain theSameElementsInOrderAs (LinkedList('e', 'i', 'p', 'u')) or not contain theSameElementsInOrderAs (LinkedList('u', 'p', 'i', 'e')))
        val e1 = intercept[TestFailedException] {
          fumList should (not contain theSameElementsInOrderAs (LinkedList('u', 'p', 'i', 'e')) or not contain theSameElementsInOrderAs (LinkedList('u', 'p', 'i', 'e')))
        }
        checkMessageStackDepth(e1, Resources("containedSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(LinkedList('u', 'p', 'i', 'e'))) + ", and " + Resources("containedSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(LinkedList('u', 'p', 'i', 'e'))), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumList should (not contain theSameElementsInOrderAs (LinkedList('E', 'I', 'P', 'U')) or not contain theSameElementsInOrderAs (LinkedList('E', 'I', 'P', 'U')))
        fumList should (not contain theSameElementsInOrderAs (LinkedList('U', 'P', 'I', 'E')) or not contain theSameElementsInOrderAs (LinkedList('E', 'I', 'P', 'U')))
        fumList should (not contain theSameElementsInOrderAs (LinkedList('E', 'I', 'P', 'U')) or not contain theSameElementsInOrderAs (LinkedList('U', 'P', 'I', 'E')))
        val e1 = intercept[TestFailedException] {
          fumList should (not contain theSameElementsInOrderAs (LinkedList('U', 'P', 'I', 'E')) or not contain theSameElementsInOrderAs (LinkedList('U', 'P', 'I', 'E')))
        }
        checkMessageStackDepth(e1, Resources("containedSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(LinkedList('U', 'P', 'I', 'E'))) + ", and " + Resources("containedSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(LinkedList('U', 'P', 'I', 'E'))), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumList should (not contain theSameElementsInOrderAs (LinkedList('E', 'I', 'P', 'U')) or not contain theSameElementsInOrderAs (LinkedList('E', 'I', 'P', 'U')))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumList should (not contain theSameElementsInOrderAs (LinkedList('U', 'P', 'I', 'E')) or not contain theSameElementsInOrderAs (LinkedList('E', 'I', 'P', 'U')))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (fumList should (not contain theSameElementsInOrderAs (LinkedList('E', 'I', 'P', 'U')) or not contain theSameElementsInOrderAs (LinkedList('U', 'P', 'I', 'E')))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not contain theSameElementsInOrderAs (LinkedList('U', 'P', 'I', 'E')) or not contain theSameElementsInOrderAs (LinkedList('U', 'P', 'I', 'E')))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("containedSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(LinkedList('U', 'P', 'I', 'E'))) + ", and " + Resources("containedSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(LinkedList('U', 'P', 'I', 'E'))), fileName, thisLineNumber - 2)
      }
    }
    
    "when used with (not equal xx and not contain theSameElementsInOrderAs xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumList should (not equal (toList) or not contain theSameElementsInOrderAs (LinkedList('e', 'i', 'p', 'u')))
        fumList should (not equal (fumList) or not contain theSameElementsInOrderAs (LinkedList('e', 'i', 'p', 'u')))
        fumList should (not equal (toList) or not contain theSameElementsInOrderAs (LinkedList('u', 'p', 'i', 'e')))
        val e1 = intercept[TestFailedException] {
          fumList should (not equal (fumList) or not contain theSameElementsInOrderAs (LinkedList('u', 'p', 'i', 'e')))
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("containedSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(LinkedList('u', 'p', 'i', 'e'))), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumList should (not equal (toList) or not contain theSameElementsInOrderAs (LinkedList('E', 'I', 'P', 'U')))
        fumList should (not equal (fumList) or not contain theSameElementsInOrderAs (LinkedList('E', 'I', 'P', 'U')))
        fumList should (not equal (toList) or not contain theSameElementsInOrderAs (LinkedList('U', 'P', 'I', 'E')))
        val e2 = intercept[TestFailedException] {
          fumList should (not equal (fumList) or (not contain theSameElementsInOrderAs (LinkedList('U', 'P', 'I', 'E'))))
        }
        checkMessageStackDepth(e2, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("containedSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(LinkedList('U', 'P', 'I', 'E'))), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumList should (not equal (fumList) or not contain theSameElementsInOrderAs (LinkedList('E', 'I', 'P', 'U')))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        (fumList should (not equal (toList) or not contain theSameElementsInOrderAs (LinkedList('E', 'I', 'P', 'U')))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        (fumList should (not equal (fumList) or not contain theSameElementsInOrderAs (LinkedList('U', 'P', 'I', 'E')))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not equal (toList) or not contain theSameElementsInOrderAs (LinkedList('U', 'P', 'I', 'E')))) (decided by invertedListOfStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("equaled", decorateToStringValue(fumList), decorateToStringValue(toList)) + ", and " + Resources("containedSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(LinkedList('U', 'P', 'I', 'E'))), fileName, thisLineNumber - 2)
      }
    }
    
    "when used with (not be xx and not contain theSameElementsInOrderAs xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        fumList should (not be (toList) or not contain theSameElementsInOrderAs (LinkedList('e', 'i', 'p', 'u')))
        fumList should (not be (fumList) or not contain theSameElementsInOrderAs (LinkedList('e', 'i', 'p', 'u')))
        fumList should (not be (toList) or not contain theSameElementsInOrderAs (LinkedList('u', 'p', 'i', 'e')))
        val e1 = intercept[TestFailedException] {
          fumList should (not be (fumList) or not contain theSameElementsInOrderAs (LinkedList('u', 'p', 'i', 'e')))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("containedSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(LinkedList('u', 'p', 'i', 'e'))), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        fumList should (not be (toList) or not contain theSameElementsInOrderAs (LinkedList('E', 'I', 'P', 'U')))
        fumList should (not be (fumList) or not contain theSameElementsInOrderAs (LinkedList('E', 'I', 'P', 'U')))
        fumList should (not be (toList) or not contain theSameElementsInOrderAs (LinkedList('U', 'P', 'I', 'E')))
        val e1 = intercept[TestFailedException] {
          fumList should (not be (fumList) or (not contain theSameElementsInOrderAs (LinkedList('U', 'P', 'I', 'E'))))
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("containedSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(LinkedList('U', 'P', 'I', 'E'))), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (fumList should (not be (toList) or not contain theSameElementsInOrderAs (LinkedList('E', 'I', 'P', 'U')))) (decided by upperCaseStringEquality)
        (fumList should (not be (fumList) or not contain theSameElementsInOrderAs (LinkedList('E', 'I', 'P', 'U')))) (decided by upperCaseStringEquality)
        (fumList should (not be (toList) or not contain theSameElementsInOrderAs (LinkedList('U', 'P', 'I', 'E')))) (decided by upperCaseStringEquality)
        val e1 = intercept[TestFailedException] {
          (fumList should (not be (fumList) or not contain theSameElementsInOrderAs (LinkedList('U', 'P', 'I', 'E')))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, Resources("wasEqualTo", decorateToStringValue(fumList), decorateToStringValue(fumList)) + ", and " + Resources("containedSameElementsInOrder", decorateToStringValue(fumList), decorateToStringValue(LinkedList('U', 'P', 'I', 'E'))), fileName, thisLineNumber - 2)
        (fumList should (not contain theSameElementsInOrderAs (Set('E', 'I', 'P', 'D')) or not contain theSameElementsInOrderAs (Set('E', 'I', 'P', 'D')))) (after being lowerCased, after being lowerCased)
      }
    }
    
  }
  
  "collection of Lists" - {
    
    val list1s: Vector[String] = Vector("123", "123", "123")
    val lists: Vector[String] = Vector("123", "123", "234")
    val nils: Vector[String] = Vector("", "", "")
    val listsString: Vector[String] = Vector("123", "123", "")
    val hiLists: Vector[String] = Vector("il", "il", "il")
    val toLists: Vector[String] = Vector("yo", "yo", "yo")
    
    def allErrMsg(index: Int, message: String, lineNumber: Int, left: Any): String = 
      "'all' inspection failed, because: \n" +
      "  at index " + index + ", " + message + " (" + fileName + ":" + (lineNumber) + ") \n" +
      "in " + left
    
    "when used with (contain theSameElementsInOrderAs xx and contain theSameElementsInOrderAs xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        all (list1s) should (contain theSameElementsInOrderAs LinkedList('1', '2', '3') or contain theSameElementsInOrderAs LinkedList('1', '2', '3'))
        all (list1s) should (contain theSameElementsInOrderAs LinkedList(3, 2, 5) or contain theSameElementsInOrderAs LinkedList('1', '2', '3'))
        all (list1s) should (contain theSameElementsInOrderAs LinkedList('1', '2', '3') or contain theSameElementsInOrderAs LinkedList('2', '3', '4'))
        
        atLeast (2, lists) should (contain theSameElementsInOrderAs LinkedList('1', '2', '3') or contain theSameElementsInOrderAs LinkedList('1', '2', '3'))
        atLeast (2, lists) should (contain theSameElementsInOrderAs LinkedList(3, 6, 5) or contain theSameElementsInOrderAs LinkedList('1', '2', '3'))
        atLeast (2, lists) should (contain theSameElementsInOrderAs LinkedList('1', '2', '3') or contain theSameElementsInOrderAs LinkedList('8', '3', '4'))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (contain theSameElementsInOrderAs LinkedList('1', '2', '3') or contain theSameElementsInOrderAs LinkedList('1', '2', '3'))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue("234") + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList('1', '2', '3')) + ", and " + decorateToStringValue("234") + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList('1', '2', '3')), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (contain theSameElementsInOrderAs LinkedList('I', 'L') or contain theSameElementsInOrderAs LinkedList('i', 'l'))
        all (hiLists) should (contain theSameElementsInOrderAs LinkedList('L', 'O') or contain theSameElementsInOrderAs LinkedList('i', 'l'))
        all (hiLists) should (contain theSameElementsInOrderAs LinkedList('I', 'L') or contain theSameElementsInOrderAs LinkedList('l', 'o'))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (contain theSameElementsInOrderAs LinkedList('L', 'O') or contain theSameElementsInOrderAs LinkedList('l', 'o'))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("il") + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList('L', 'O')) + ", and " + decorateToStringValue("il") + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList('l', 'o')), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (all (hiLists) should (contain theSameElementsInOrderAs LinkedList('I', 'L') or contain theSameElementsInOrderAs LinkedList('i', 'l'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hiLists) should (contain theSameElementsInOrderAs LinkedList('L', 'O') or contain theSameElementsInOrderAs LinkedList('i', 'l'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hiLists) should (contain theSameElementsInOrderAs LinkedList('I', 'L') or contain theSameElementsInOrderAs LinkedList('l', 'o'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (contain theSameElementsInOrderAs LinkedList('L', 'O') or contain theSameElementsInOrderAs LinkedList('l', 'o'))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("il") + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList('L', 'O')) + ", and " + decorateToStringValue("il") + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList('l', 'o')), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
    }
    
    "when used with (be xx and contain theSameElementsInOrderAs xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        all (list1s) should (be ("123") or contain theSameElementsInOrderAs LinkedList('1', '2', '3'))
        all (list1s) should (be ("234") or contain theSameElementsInOrderAs LinkedList('1', '2', '3'))
        all (list1s) should (be ("123") or contain theSameElementsInOrderAs LinkedList('2', '3', '4'))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (be ("234") or contain theSameElementsInOrderAs LinkedList('2', '3', '4'))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("[123]") + " was not equal to " + decorateToStringValue("[234]") + ", and " + decorateToStringValue("123") + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList('2', '3', '4')), thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (be ("il") or contain theSameElementsInOrderAs LinkedList('I', 'L'))
        all (hiLists) should (be (List('o', 'l')) or contain theSameElementsInOrderAs LinkedList('I', 'L'))
        all (hiLists) should (be ("il") or contain theSameElementsInOrderAs LinkedList('L', 'I'))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (be (List('o', 'l')) or contain theSameElementsInOrderAs LinkedList('L', 'I'))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("il") + " was not equal to " + decorateToStringValue(List('o', 'l')) + ", and " + decorateToStringValue("il") + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList('L', 'I')), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (all (hiLists) should (be ("il") or contain theSameElementsInOrderAs LinkedList('I', 'L'))) (decided by upperCaseStringEquality)
        (all (hiLists) should (be (List('o', 'l')) or contain theSameElementsInOrderAs LinkedList('I', 'L'))) (decided by upperCaseStringEquality)
        (all (hiLists) should (be ("il") or contain theSameElementsInOrderAs LinkedList('L', 'I'))) (decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (be (List('o', 'l')) or contain theSameElementsInOrderAs LinkedList('L', 'I'))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("il") + " was not equal to " + decorateToStringValue(List('o', 'l')) + ", and " + decorateToStringValue("il") + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList('L', 'I')), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
    }
    
    "when used with (not contain theSameElementsInOrderAs xx and not contain theSameElementsInOrderAs xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        all (list1s) should (not contain theSameElementsInOrderAs (LinkedList('3', '2', '8')) or not contain theSameElementsInOrderAs (LinkedList('8', '3', '4')))
        all (list1s) should (not contain theSameElementsInOrderAs (LinkedList('1', '2', '3')) or not contain theSameElementsInOrderAs (LinkedList('8', '3', '4')))
        all (list1s) should (not contain theSameElementsInOrderAs (LinkedList('3', '2', '8')) or not contain theSameElementsInOrderAs (LinkedList('1', '2', '3')))
        
        val e1 = intercept[TestFailedException] {
          all (lists) should (not contain theSameElementsInOrderAs (LinkedList('2', '3', '4')) or not contain theSameElementsInOrderAs (LinkedList('2', '3', '4')))
        }
        checkMessageStackDepth(e1, allErrMsg(2, decorateToStringValue("234") + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList('2', '3', '4')) + ", and " + decorateToStringValue("234") + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList('2', '3', '4')), thisLineNumber - 2, lists), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (not contain theSameElementsInOrderAs (LinkedList('L', 'I')) or not contain theSameElementsInOrderAs (LinkedList('l', 'i')))
        all (hiLists) should (not contain theSameElementsInOrderAs (LinkedList('I', 'L')) or not contain theSameElementsInOrderAs (LinkedList('l', 'i')))
        all (hiLists) should (not contain theSameElementsInOrderAs (LinkedList('L', 'I')) or not contain theSameElementsInOrderAs (LinkedList('i', 'l')))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (not contain theSameElementsInOrderAs (LinkedList('I', 'L')) or not contain theSameElementsInOrderAs (LinkedList('i', 'l')))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("il") + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList('I', 'L')) + ", and " + decorateToStringValue("il") + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList('i', 'l')), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (all (hiLists) should (not contain theSameElementsInOrderAs (LinkedList('L', 'I')) or not contain theSameElementsInOrderAs (LinkedList('l', 'i')))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hiLists) should (not contain theSameElementsInOrderAs (LinkedList('I', 'L')) or not contain theSameElementsInOrderAs (LinkedList('l', 'i')))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        (all (hiLists) should (not contain theSameElementsInOrderAs (LinkedList('L', 'I')) or not contain theSameElementsInOrderAs (LinkedList('i', 'l')))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (not contain theSameElementsInOrderAs (LinkedList('I', 'L')) or not contain theSameElementsInOrderAs (LinkedList('i', 'l')))) (decided by upperCaseStringEquality, decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("il") + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList('I', 'L')) + ", and " + decorateToStringValue("il") + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList('i', 'l')), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
    }
    
    "when used with (not be xx and not contain theSameElementsInOrderAs xx)" - {
      
      "should do nothing if valid, else throw a TFE with an appropriate error message" in {
        all (list1s) should (not be ("2") or not contain theSameElementsInOrderAs (LinkedList('8', '3', '4')))
        all (list1s) should (not be ("123") or not contain theSameElementsInOrderAs (LinkedList('8', '3', '4')))
        all (list1s) should (not be ("2") or not contain theSameElementsInOrderAs (LinkedList('1', '2', '3')))
        
        val e1 = intercept[TestFailedException] {
          all (list1s) should (not be ("123") or not contain theSameElementsInOrderAs (LinkedList('1', '2', '3')))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("123") + " was equal to " + decorateToStringValue("123") + ", and " + decorateToStringValue("123") + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList('1', '2', '3')), thisLineNumber - 2, list1s), fileName, thisLineNumber - 2)
      }
      
      "should use the implicit Equality in scope" in {
        implicit val ise = upperCaseStringEquality
        
        all (hiLists) should (not be (List('l', 'o')) or not contain theSameElementsInOrderAs (LinkedList('L', 'O')))
        all (hiLists) should (not be ("il") or not contain theSameElementsInOrderAs (LinkedList('L', 'O')))
        all (hiLists) should (not be (List('l', 'o')) or not contain theSameElementsInOrderAs (LinkedList('I', 'L')))
        
        val e1 = intercept[TestFailedException] {
          all (hiLists) should (not be ("il") or not contain theSameElementsInOrderAs (LinkedList('I', 'L')))
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("il") + " was equal to " + decorateToStringValue("il") + ", and " + decorateToStringValue("il") + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList('I', 'L')), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
      
      "should use an explicitly provided Equality" in {
        (all (hiLists) should (not be (List('l', 'o')) or not contain theSameElementsInOrderAs (LinkedList('L', 'O')))) (decided by upperCaseStringEquality)
        (all (hiLists) should (not be ("il") or not contain theSameElementsInOrderAs (LinkedList('L', 'O')))) (decided by upperCaseStringEquality)
        (all (hiLists) should (not be (List('l', 'o')) or not contain theSameElementsInOrderAs (LinkedList('I', 'L')))) (decided by upperCaseStringEquality)
        
        val e1 = intercept[TestFailedException] {
          (all (hiLists) should (not be ("il") or not contain theSameElementsInOrderAs (LinkedList('I', 'L')))) (decided by upperCaseStringEquality)
        }
        checkMessageStackDepth(e1, allErrMsg(0, decorateToStringValue("il") + " was equal to " + decorateToStringValue("il") + ", and " + decorateToStringValue("il") + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList('I', 'L')), thisLineNumber - 2, hiLists), fileName, thisLineNumber - 2)
      }
    }
  }
}
