/*
 * Copyright 2001-2013 Artima, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.scalatest

import org.scalautils.Equality
import org.scalautils.Normalization
import org.scalautils.StringNormalizations._
import SharedHelpers._
import FailureMessages.decorateToStringValue
import scala.collection.mutable.LinkedList

class StringShouldContainTheSameElementsInOrderAsSpec extends Spec with Matchers {
  
  private def upperCase(value: Any): Any = 
    value match {
      case l: List[_] => l.map(upperCase(_))
      case s: String => s.toUpperCase
      case c: Char => c.toString.toUpperCase.charAt(0)
      case (s1: String, s2: String) => (s1.toUpperCase, s2.toUpperCase)
      case _ => value
    }
  
  val upperCaseStringEquality =
    new Equality[Char] {
      def areEqual(a: Char, b: Any): Boolean = upperCase(a) == upperCase(b)
    }
  
  val lowerCased: Normalization[Char] = new Normalization[Char] {
    def normalized(c: Char): Char = c.toString.toLowerCase.toCharArray()(0)
    def normalizedIfInstanceOfA(b: Any) =
      b match {
        case c: Char => normalized(c)
        case _ => b
      }
    }

  object `a List` {

    val fumList: String = "upie"
    val toList: String = "hboy"

    object `when used with contain theSameElementsInOrderAs (..)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        fumList should contain theSameElementsInOrderAs LinkedList('u', 'p', 'i', 'e')
        val e1 = intercept[TestFailedException] {
          fumList should contain theSameElementsInOrderAs LinkedList('e', 'i', 'p', 'u')
        }
        e1.failedCodeFileName.get should be ("StringShouldContainTheSameElementsInOrderAsSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumList), LinkedList('e', 'i', 'p', 'u')))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should contain theSameElementsInOrderAs LinkedList('U', 'P', 'I', 'E')
        intercept[TestFailedException] {
          fumList should contain theSameElementsInOrderAs LinkedList('e', 'i', 'p', 'u')
        }
      }
      def `should use an explicitly provided Equality` {
        (fumList should contain theSameElementsInOrderAs LinkedList('U', 'P', 'I', 'E')) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (fumList should contain theSameElementsInOrderAs LinkedList('e', 'i', 'p', 'u')) (decided by upperCaseStringEquality)
        }
        intercept[TestFailedException] {
          fumList should contain theSameElementsInOrderAs LinkedList('U', 'P', 'I', 'E')
        }
        (fumList should contain theSameElementsInOrderAs LinkedList('U', 'P', 'I', 'E')) (after being lowerCased)
      }
    }

    object `when used with (contain theSameElementsInOrderAs (..))` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {

        fumList should (contain theSameElementsInOrderAs LinkedList('u', 'p', 'i', 'e'))
        val e1 = intercept[TestFailedException] {
          fumList should (contain theSameElementsInOrderAs LinkedList('e', 'i', 'p', 'u'))
        }
        e1.failedCodeFileName.get should be ("StringShouldContainTheSameElementsInOrderAsSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("didNotContainSameElementsInOrder", decorateToStringValue(fumList), LinkedList('e', 'i', 'p', 'u')))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        fumList should (contain theSameElementsInOrderAs LinkedList('U', 'P', 'I', 'E'))
        intercept[TestFailedException] {
          fumList should (contain theSameElementsInOrderAs LinkedList('e', 'i', 'p', 'u'))
        }
      }
      def `should use an explicitly provided Equality` {
        (fumList should (contain theSameElementsInOrderAs LinkedList('U', 'P', 'I', 'E'))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (fumList should (contain theSameElementsInOrderAs LinkedList('e', 'i', 'p', 'u'))) (decided by upperCaseStringEquality)
        }
        intercept[TestFailedException] {
          fumList should (contain theSameElementsInOrderAs LinkedList('U', 'P', 'I', 'E'))
        }
        (fumList should (contain theSameElementsInOrderAs LinkedList('U', 'P', 'I', 'E'))) (after being lowerCased)
      }
    }

    object `when used with not contain theSameElementsInOrderAs (..)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        toList should not contain theSameElementsInOrderAs (LinkedList('y', 'o', 'b', 'h'))
        val e1 = intercept[TestFailedException] {
          toList should not contain theSameElementsInOrderAs (LinkedList('h', 'b', 'o', 'y'))
        }
        e1.failedCodeFileName.get should be ("StringShouldContainTheSameElementsInOrderAsSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedSameElementsInOrder", decorateToStringValue(toList), LinkedList('h', 'b', 'o', 'y')))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        toList should not contain theSameElementsInOrderAs (LinkedList('Y', 'O', 'B', 'H'))
        intercept[TestFailedException] {
          toList should not contain theSameElementsInOrderAs (LinkedList('H', 'B', 'O', 'Y'))
        }
      }
      def `should use an explicitly provided Equality` {
        (toList should not contain theSameElementsInOrderAs (LinkedList('Y', 'O', 'B', 'H'))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (toList should not contain theSameElementsInOrderAs (LinkedList('H', 'B', 'O', 'Y'))) (decided by upperCaseStringEquality)
        }
        toList should not contain theSameElementsInOrderAs (LinkedList('H', 'B', 'O', 'Y'))
        intercept[TestFailedException] {
          (toList should not contain theSameElementsInOrderAs (LinkedList('H', 'B', 'O', 'Y'))) (after being lowerCased)
        }
      }
    }

    object `when used with (not contain theSameElementsInOrderAs (..))` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        toList should (not contain theSameElementsInOrderAs (LinkedList('y', 'o', 'b', 'h')))
        val e1 = intercept[TestFailedException] {
          toList should (not contain theSameElementsInOrderAs (LinkedList('h', 'b', 'o', 'y')))
        }
        e1.failedCodeFileName.get should be ("StringShouldContainTheSameElementsInOrderAsSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message.get should be (Resources("containedSameElementsInOrder", decorateToStringValue(toList), LinkedList('h', 'b', 'o', 'y')))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        toList should (not contain theSameElementsInOrderAs (LinkedList('Y', 'O', 'B', 'H')))
        intercept[TestFailedException] {
          toList should (not contain theSameElementsInOrderAs (LinkedList('H', 'B', 'O', 'Y')))
        }
      }
      def `should use an explicitly provided Equality` {
        (toList should (not contain theSameElementsInOrderAs (LinkedList('Y', 'O', 'B', 'H')))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (toList should (not contain theSameElementsInOrderAs (LinkedList('H', 'B', 'O', 'Y')))) (decided by upperCaseStringEquality)
        }
        toList should (not contain theSameElementsInOrderAs (LinkedList('H', 'B', 'O', 'Y')))
        intercept[TestFailedException] {
          (toList should (not contain theSameElementsInOrderAs (LinkedList('H', 'B', 'O', 'Y')))) (after being lowerCased)
        }
      }
    }
  }

  object `a col of Lists` {

    val list1s: Vector[String] = Vector( "123", "123", "123")
    val lists: Vector[String] = Vector( "123", "123", "234")
    val listsString: Vector[String] = Vector( "123", "123", "")
    val hiLists: Vector[String] = Vector( "ie", "ie", "ie")
    val toLists: Vector[String] = Vector( "oy", "oy", "oy")

    object `when used with contain theSameElementsInOrderAs (..)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should contain theSameElementsInOrderAs LinkedList('1', '2', '3')
        atLeast (2, lists) should contain theSameElementsInOrderAs LinkedList('1', '2', '3')
        atMost (2, lists) should contain theSameElementsInOrderAs LinkedList('1', '2', '3')
        no (lists) should contain theSameElementsInOrderAs LinkedList('3', '4', '5')

        val e1 = intercept[TestFailedException] {
          all (lists) should contain theSameElementsInOrderAs LinkedList('1', '2', '3')
        }
        e1.failedCodeFileName.get should be ("StringShouldContainTheSameElementsInOrderAsSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue("234") + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList('1', '2', '3')) + " (StringShouldContainTheSameElementsInOrderAsSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))

        val e3 = intercept[TestFailedException] {
          all (listsString) should contain theSameElementsInOrderAs LinkedList('1', '2', '3')
        }
        e3.failedCodeFileName.get should be ("StringShouldContainTheSameElementsInOrderAsSpec.scala")
        e3.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e3.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue("") + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList('1', '2', '3')) + " (StringShouldContainTheSameElementsInOrderAsSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(listsString)))
      }

      def `should use the implicit Equality in scope` {
        all (hiLists) should contain theSameElementsInOrderAs LinkedList('i', 'e')
        intercept[TestFailedException] {
          all (hiLists) should contain theSameElementsInOrderAs LinkedList("hi", "ho")
        }
        implicit val ise = upperCaseStringEquality
        all (hiLists) should contain theSameElementsInOrderAs LinkedList('I', 'E')
        intercept[TestFailedException] {
          all (hiLists) should contain theSameElementsInOrderAs LinkedList('I', 'O')
        }
      }
      def `should use an explicitly provided Equality` {
        (all (hiLists) should contain theSameElementsInOrderAs LinkedList('I', 'E')) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (hiLists) should contain theSameElementsInOrderAs LinkedList('I', 'O')) (decided by upperCaseStringEquality)
        }
        implicit val ise = upperCaseStringEquality
        (all (hiLists) should contain theSameElementsInOrderAs LinkedList('i', 'e')) (decided by defaultEquality[Char])
        intercept[TestFailedException] {
          (all (hiLists) should contain theSameElementsInOrderAs LinkedList("hi", "ho")) (decided by defaultEquality[Char])
        }
      }
    }

    object `when used with (contain theSameElementsInOrderAs (..))` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (list1s) should (contain theSameElementsInOrderAs LinkedList('1', '2', '3'))
        atLeast (2, lists) should (contain theSameElementsInOrderAs LinkedList('1', '2', '3'))
        atMost (2, lists) should (contain theSameElementsInOrderAs LinkedList('1', '2', '3'))
        no (lists) should (contain theSameElementsInOrderAs LinkedList('3', '4', '5'))
        no (listsString) should (contain theSameElementsInOrderAs LinkedList('3', '4', '5'))

        val e1 = intercept[TestFailedException] {
          all (lists) should (contain theSameElementsInOrderAs LinkedList('1', '2', '3'))
        }
        e1.failedCodeFileName.get should be ("StringShouldContainTheSameElementsInOrderAsSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue("234") + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList('1', '2', '3')) + " (StringShouldContainTheSameElementsInOrderAsSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(lists)))

        val e4 = intercept[TestFailedException] {
          all (listsString) should (contain theSameElementsInOrderAs LinkedList('1', '2', '3'))
        }
        e4.failedCodeFileName.get should be ("StringShouldContainTheSameElementsInOrderAsSpec.scala")
        e4.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e4.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 2, " + decorateToStringValue("") + " did not contain the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList('1', '2', '3')) + " (StringShouldContainTheSameElementsInOrderAsSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(listsString)))
      }

      def `should use the implicit Equality in scope` {
        all (hiLists) should (contain theSameElementsInOrderAs LinkedList('i', 'e'))
        intercept[TestFailedException] {
          all (hiLists) should (contain theSameElementsInOrderAs LinkedList('e', 'i'))
        }
        implicit val ise = upperCaseStringEquality
        all (hiLists) should (contain theSameElementsInOrderAs LinkedList('I', 'E'))
        intercept[TestFailedException] {
          all (hiLists) should (contain theSameElementsInOrderAs LinkedList('I', 'O'))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (hiLists) should (contain theSameElementsInOrderAs LinkedList('I', 'E'))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (hiLists) should (contain theSameElementsInOrderAs LinkedList('I', 'O'))) (decided by upperCaseStringEquality)
        }
        implicit val ise = upperCaseStringEquality
        (all (hiLists) should (contain theSameElementsInOrderAs LinkedList('i', 'e'))) (decided by defaultEquality[Char])
        intercept[TestFailedException] {
          (all (hiLists) should (contain theSameElementsInOrderAs LinkedList('e', 'i'))) (decided by defaultEquality[Char])
        }
      }
    }

    object `when used with not contain theSameElementsInOrderAs (..)` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (toLists) should not contain theSameElementsInOrderAs (LinkedList('y', 'o'))
        val e1 = intercept[TestFailedException] {
          all (toLists) should not contain theSameElementsInOrderAs (LinkedList('o', 'y'))
        }
        e1.failedCodeFileName.get should be ("StringShouldContainTheSameElementsInOrderAsSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue("oy") + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList('o', 'y')) +  " (StringShouldContainTheSameElementsInOrderAsSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(toLists)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        all (toLists) should not contain theSameElementsInOrderAs (LinkedList('Y', 'O'))
        intercept[TestFailedException] {
          all (toLists) should not contain theSameElementsInOrderAs (LinkedList('O', 'Y'))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (toLists) should not contain theSameElementsInOrderAs (LinkedList('Y', 'O'))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (toLists) should not contain theSameElementsInOrderAs (LinkedList('O', 'Y'))) (decided by upperCaseStringEquality)
        }
        all (toLists) should not contain theSameElementsInOrderAs (LinkedList('O', 'Y'))
        intercept[TestFailedException] {
          (all (toLists) should not contain theSameElementsInOrderAs (LinkedList('O', 'Y'))) (after being lowerCased)
        }
      }
    }

    object `when used with (not contain theSameElementsInOrderAs (..))` {

      def `should do nothing if valid, else throw a TFE with an appropriate error message` {
        all (toLists) should (not contain theSameElementsInOrderAs (LinkedList('y', 'o')))
        val e1 = intercept[TestFailedException] {
          all (toLists) should (not contain theSameElementsInOrderAs (LinkedList('o', 'y')))
        }
        e1.failedCodeFileName.get should be ("StringShouldContainTheSameElementsInOrderAsSpec.scala")
        e1.failedCodeLineNumber.get should be (thisLineNumber - 3)
        e1.message should be (Some("'all' inspection failed, because: \n" +
                                   "  at index 0, " + decorateToStringValue("oy") + " contained the same elements in the same (iterated) order as " + decorateToStringValue(LinkedList('o', 'y')) + " (StringShouldContainTheSameElementsInOrderAsSpec.scala:" + (thisLineNumber - 5) + ") \n" +
                                   "in " + decorateToStringValue(toLists)))
      }
      def `should use the implicit Equality in scope` {
        implicit val ise = upperCaseStringEquality
        all (toLists) should (not contain theSameElementsInOrderAs (LinkedList('Y', 'O')))
        intercept[TestFailedException] {
          all (toLists) should (not contain theSameElementsInOrderAs (LinkedList('O', 'Y')))
        }
      }
      def `should use an explicitly provided Equality` {
        (all (toLists) should (not contain theSameElementsInOrderAs (LinkedList('Y', 'O')))) (decided by upperCaseStringEquality)
        intercept[TestFailedException] {
          (all (toLists) should (not contain theSameElementsInOrderAs (LinkedList('O', 'Y')))) (decided by upperCaseStringEquality)
        }
        all (toLists) should (not contain theSameElementsInOrderAs (LinkedList('O', 'Y')))
        intercept[TestFailedException] {
          (all (toLists) should (not contain theSameElementsInOrderAs (LinkedList('O', 'Y')))) (after being lowerCased)
        }
      }
    }
  }
}
